/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF 
 *
 * Contact: clif@ow2.org
 */
package org.ow2.isac.plugin.httpinjector.actions;

import org.apache.commons.httpclient.Header;
import org.ow2.isac.plugin.httpinjector.SessionObject;

/**
 * This class implements the matrix test methods
 * 
 * @author JC Meillaud
 * @author A Peyrard
 */
public class HttpInjectorTests {	
	/**
	 * Simple test to know if the last response was a not found Http Status Code
	 * 
	 * @param sessionObject
	 *            The SessionObject where we get information about the last
	 *            statuscode
	 * @return true if it was a 404 error
	 */

	public static boolean is404Response(SessionObject sessionObject) {
		return (sessionObject.getLastStatusCode() == 404);
	}

	/**
	 * General test to know if the last response Status code is equal to
	 * statusCode
	 * 
	 * @param sessionObject
	 *            The SessionObject where we get information about the last
	 *            statuscode
	 * @param statusCode
	 *            the code
	 * @return true if it was an error equal to statusCode
	 */
	public static boolean isStatusCodeResponse(SessionObject sessionObject,
			int statusCode) {

		return (sessionObject.getLastStatusCode() == statusCode);
	}

	/**
	 * Test to know if a header value of the last executed method is equal to
	 * the one given
	 * 
	 * @param sessionObject
	 *            The SessionObject where we get information about the last
	 *            headers
	 * @param headerParameterType
	 *            the name of the header value to compare
	 * @param headerParameterValue
	 *            the value of the header value to compare
	 * @return
	 */
	public static boolean isHeaderValue(SessionObject sessionObject,
			String headerParameterType, String headerParameterValue) {
		Header[] headers = sessionObject.getLastHeader();
		for (int i = 0; i < headers.length; i++) {
			if (headers[i].getName().equals(headerParameterType)) {
				return (headers[i].getValue().equals(headerParameterValue));
			}
		}
		return false;
	}
}