/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.api.graph;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.Map;

import javax.swing.JPanel;

// TODO: Auto-generated Javadoc
/**
 * The Interface ChartControl.
 */
public interface ChartControl {

    //  Fractal interface name
    /** The Constant CHART_CONTROL. */
    static public final String CHART_CONTROL = "Chart control";

    /** The Constant LINE_RENDERER. */
    static public final String LINE_RENDERER = "Line";
    
    /** The Constant AREA_RENDERER. */
    static public final String AREA_RENDERER = "Area";
    
    /** The Constant BAR_RENDERER. */
    static public final String BAR_RENDERER = "Bar";
    
    /** The Constant DOT_RENDERER. */
    static public final String DOT_RENDERER = "Dot";
    
    /**
     * Define the X axis for the chart.
     * 
     * @param chartId the chart id
     * @param fieldName the field name
     */
    public void setXAxis(int chartId, String fieldName);
    
    /**
     * Add an Y axis to the chart.
     * 
     * @param chartId the chart id
     * @param testName the name of the test
     * @param bladeId the id of a blade
     * @param eventTypeLabel the label of an event type
     * @param fieldLabel the field label
     */
    public void addYAxis(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel);
    
    /**
     * Sets the Y axis moving statistics.
     * 
     * @param chartId the chart id
     * @param testName the test name
     * @param bladeId the blade id
     * @param eventTypeLabel the event type label
     * @param fieldLabel the field label
     * @param movingStatistic the moving statistic
     * @param timeWindow the time window
     * @param step the step
     */
    public void setYAxisMovingStatistics(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel, String movingStatistic, int timeWindow, int step);
    
    /**
     * Remove a Y axis to the chart.
     * 
     * @param chartId the chart id
     * @param testName the name of the test
     * @param bladeId the id of a blade
     * @param eventTypeLabel the label of an event type
     * @param fieldLabel the field label
     */
    public void removeYAxis(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel);

    /**
     * Define the chart's title.
     * 
     * @param title the chart's title
     * @param chartId the chart id
     */
    public void setChartTitle(int chartId, String title);
    
    
    /**
     * Define the X axis' label.
     * 
     * @param chartId the chart id
     * @param label the label of the X axis
     */
    public void setXAxisLabel(int chartId, String label);
    
    /**
     * Define a Y axis' label.
     * 
     * @param chartId the chart id
     * @param testName the name of the test
     * @param bladeId the id of a blade
     * @param eventTypeLabel the label of an event type
     * @param label the label of the Y axis
     * @param fieldLabel the field label
     */
    public void setYAxisLabel(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel, String label);
    
    /**
     * Get a panel with a chart in.
     * 
     * @param chartId the chart id
     * 
     * @return a panel with the request chart in
     */
    public JPanel getChart(int chartId);
    
    /**
     * Create a new chart of the type type.
     * 
     * @param type the type of chart
     * 
     * @return the chart id
     */
    public int createNewChart(String type);
    
    /**
     * Get an image of the chart.
     * 
     * @param chartId the chart id
     * @param width the width
     * @param height the height
     * 
     * @return an buffered image of the chart
     */
    public BufferedImage getChartImage(int chartId, int width, int height);
    
    /**
     * Sets the color of a plot.
     * 
     * @param chartId the chart id
     * @param color the color
     * @param testName the test name
     * @param bladeId the blade id
     * @param eventTypeLabel the event type label
     * @param fieldLabel the field label
     */
    public void setColor(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel ,Color color);
    
    /**
     * Gets the color.
     * 
     * @param chartId the chart id
     * @param testName the test name
     * @param bladeId the blade id
     * @param eventTypeLabel the event type label
     * @param fieldLabel the field label
     * 
     * @return the color
     */
    public Color getColor(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel);
    
    /**
     * Sets the renderer.
     * 
     * @param chartId the chart id
     * @param testName the test name
     * @param bladeId the blade id
     * @param eventTypeLabel the event type label
     * @param fieldLabel the field label
     * @param rendererType the type renderer
     */
    public void setRenderer(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel, String rendererType);
    
    /**
     * Gets the renderer list.
     * 
     * @return the list of renderer list
     */
    public Map getRendererList();
    
    /**
     * Sets the domain range.
     * 
     * @param chartId the chart id
     * @param lower the lower
     * @param upper the upper
     */
    public void setDomainRange(int chartId, int lower, int upper);
    
    /**
     * Sets the max number of point.
     * 
     * @param chartId the chart id
     * @param number the number
     */
    public void setMaxNumberOfPoint(int chartId, int number);
    
    /**
     * Gets the max number of point.
     * 
     * @param chartId the chart id
     * 
     * @return the max number of point
     */
    public int getMaxNumberOfPoint(int chartId);
    
    /**
     * Gets the upper limit.
     * 
     * @param chartId the chart id
     * 
     * @return the upper limit
     */
    public int getUpperLimit(int chartId);
    
    /**
     * Gets the lower limit.
     * 
     * @param chartId the chart id
     * 
     * @return the lower limit
     */
    public int getLowerLimit(int chartId);
    
}
