/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.api.graph.datasource;

import java.util.Map;

/**
 * The Interface StatisticCollector.
 */
public interface StatisticCollector {

    //  Fractal interface name
    /** The Constant STATISTIC_COLLECTOR. */
    static public final String STATISTIC_COLLECTOR = "Statistic collector";
    
    /**
     * Get statistic on the Y field.
     * 
     * @param testName the test run name
     * @param bladeId the blade identifier from the given test run
     * @param eventTypeLabel the label of the event type
     * @param XFieldLabel the label of the X field
     * @param YFieldLabel the label of the Y field
     * @param start the start
     * @param end the end
     * @param count the count
     * 
     * @return a map that contains the various statistics on the Y field
     */
    public Map getStatistics(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count);
    
    
    /**
     * Gets the means.
     * 
     * @param values the values
     * @param step the step
     * @param timeWindow the time window
     * 
     * @return the means
     */
    public Object[][] getMeans(Object[][] values, int timeWindow, int step);
    
    /**
     * Gets the standard deviation.
     * 
     * @param values the values
     * @param step the step
     * @param timeWindow the time window
     * 
     * @return the standard deviation
     */
    public Object[][] getStandardDeviations(Object[][] values, int timeWindow, int step);
    
    /**
     * Gets the min values.
     * 
     * @param values the values
     * @param timeWindow the time window
     * @param step the step
     * 
     * @return the min values
     */
    public Object[][] getMinValues(Object[][] values, int timeWindow, int step);
    
    /**
     * Gets the max values.
     * 
     * @param values the values
     * @param timeWindow the time window
     * @param step the step
     * 
     * @return the max values
     */
    public Object[][] getMaxValues(Object[][] values, int timeWindow, int step);
}
