/*
* CLIF is a Load Injection Framework
* Copyright (C) 2008 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF
*@author Olivier Liu
*
*
*/
package org.ow2.clif.analyze.lib.graph;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jfree.chart.JFreeChart;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.clif.analyze.api.graph.VirtualDataSupplier;
import org.ow2.clif.analyze.api.graph.virtualdata.AggregateDataControl;
import org.ow2.clif.analyze.api.graph.virtualdata.FilterDataControl;
import org.ow2.clif.analyze.api.graph.virtualdata.RoutineSupplier;
import org.ow2.clif.analyze.api.graph.virtualdata.StatisticDataControl;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.supervisor.api.ClifException;

public class VirtualDataProxyImpl implements BindingController, VirtualDataSupplier, RoutineSupplier{
	/**
	 * The Class VirtualDataProxyImpl
	 */
	
		/** The instance. */
		private static VirtualDataProxyImpl instance= new VirtualDataProxyImpl();
		    
	    /** The interface list. */
	    static private String[] itfList = new String[] { 
	    	FilterDataControl.FILTER_DATA_CONTROL,
	    	AggregateDataControl.AGGREGATE_DATA_CONTROL,
	    	StatisticDataControl.STATISTIC_DATA_CONTROL,
	    	RoutineSupplier.ROUTINE_SUPPLIER
	    };

	    // field for client binding (access to stored data)
	    private FilterDataControl fdcItf;
	    private AggregateDataControl adcItf;
	    private StatisticDataControl sdcItf;
	    private RoutineSupplier rsItf;
	   
	    /**
	     * Instantiates a new chart engine impl.
	     */
	    public VirtualDataProxyImpl()
	    { 
	       //empty
	    }
	    
	    /**
	     * Gets the single instance of FilterDataImpl.
	     * 
	     * @return single instance of FilterDataImpl
	     */
	    public VirtualDataProxyImpl getInstance()
	    {
	        return instance;
	    }

	    /////////////////////////////////
	    // interface BindingController //
	    /////////////////////////////////


	    /* (non-Javadoc)
	     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
	     */
	    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
	    {
	        if (clientItfName.equals(FilterDataControl.FILTER_DATA_CONTROL)){
	        	fdcItf = (FilterDataControl) serverItf;
	        } 
	        else if (clientItfName.equals(AggregateDataControl.AGGREGATE_DATA_CONTROL)){
	        	adcItf = (AggregateDataControl) serverItf;
	        }
	        else if (clientItfName.equals(StatisticDataControl.STATISTIC_DATA_CONTROL)){
	        	sdcItf = (StatisticDataControl) serverItf;
	        }
	        else if (clientItfName.equals(RoutineSupplier.ROUTINE_SUPPLIER)){
	        	rsItf = (RoutineSupplier) serverItf;
	        }
	    }

	    /* (non-Javadoc)
	     * @see org.objectweb.fractal.api.control.BindingController#listFc()
	     */
	    public String[] listFc() 
	    {
	        return itfList;
	    }

	    /* (non-Javadoc)
	     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	     */
	    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException 
	    {
	        if (clientItfName.equals(FilterDataControl.FILTER_DATA_CONTROL)){
	        	return fdcItf;
	        } 
	        else if (clientItfName.equals(AggregateDataControl.AGGREGATE_DATA_CONTROL)){
	        	return adcItf;
	        }
	        else if (clientItfName.equals(StatisticDataControl.STATISTIC_DATA_CONTROL)){
	            return sdcItf;
	        }
	        else if (clientItfName.equals(RoutineSupplier.ROUTINE_SUPPLIER)){
	        	return rsItf;
	        }
	        else{
	        	return null;
	        }
	    }

	    /* (non-Javadoc)
	     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	     */
	    public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
	    {
	        if (clientItfName.equals(FilterDataControl.FILTER_DATA_CONTROL)){
	        	fdcItf = null;
	        } 
	        else if (clientItfName.equals(AggregateDataControl.AGGREGATE_DATA_CONTROL)){
	        	adcItf = null;
	        }
	        else if (clientItfName.equals(StatisticDataControl.STATISTIC_DATA_CONTROL)){
	        	sdcItf = null;
	        }
	        else if (clientItfName.equals(RoutineSupplier.ROUTINE_SUPPLIER)){
	        	rsItf = null;
	        }
	    }

		

	    
	    ////////////////////////////////////////////////////////////////////////
	    // interface VirtualDataSupplier : Filter + Aggregate +Statistic	  //
	    ////////////////////////////////////////////////////////////////////////
	  
	    
	    ////////////////////////////////////
	    // interface FilterDataControl    //
	    ////////////////////////////////////
	   
	    
	    /* (non-Javadoc)
	     * @see org.ow2.clif.analyze.api.graph.VirtualDataSupplier#getCommonMinTime(java.lang.String)
	     */
		public long getCommonMinTime(String testName){
			return fdcItf.getCommonMinTime(testName);
		}
		
		/* (non-Javadoc)
	     * @see org.ow2.clif.analyze.api.graph.VirtualDataSupplier#getCommonMaxTime(java.lang.String)
	     */
		public long getCommonMaxTime(String testName){
			return fdcItf.getCommonMaxTime(testName);
		}
		
		/* (non-Javadoc)
	     * @see org.ow2.clif.analyze.api.graph.VirtualDataSupplier#getBladeOneEventValues(java.lang.String, java.lang.String, java.lang.String)
	     */
		public List<BladeEvent> getBladeOneEventValues(String testName,
				String bladeId, String eventTypeLabel) throws ClifException{
			return fdcItf.getBladeOneEventValues(testName, bladeId, eventTypeLabel);
		}
		
		/* (non-Javadoc)
	     * @see org.ow2.clif.analyze.api.graph.VirtualDataSupplier#getBladeOneEventValues(java.lang.String, java.lang.String, java.lang.String)
	     */
		public Map<String, List<BladeEvent>> getBladeAllEventsValues(
				String testName, String bladeId, String eventTypeLabel)throws ClifException{
			return fdcItf.getBladeAllEventsValues(testName, bladeId, eventTypeLabel);
		}
		
		/* (non-Javadoc)
	     * @see org.ow2.clif.analyze.api.graph.VirtualDataSupplier#getBladeOneEventValues(java.lang.String, java.lang.String, java.lang.String, java.util.map, java.lang.string)
	     */
		public List<BladeEvent> getBladeAllEventsValuesFiltered(
	    		String testName, 
	    		String bladeId, 
	    		String eventTypeLabel,
	    		Map<String, Map<String, String>> filter,
	    		String linkingOp){
			return fdcItf.getBladeAllEventsValuesFiltered(
					testName, 
					bladeId, 
					eventTypeLabel, 
					filter, 
					linkingOp);
		}
		

		
		//MAYBE NOT USED
		public BladeEvent[] getBladeEvents(
				String testName,	
				String bladeId, 
				String eventTypeLabel,
				EventFilter filter, 
				long fromIndex,
				int count
				)throws ClifException{
			return fdcItf.getBladeEvents(
					testName, 
					bladeId, 
					eventTypeLabel, 
					filter, 
					fromIndex, 
					count);
		}
	    
		//MAYBE NOT USED
		public List<BladeEvent> fieldFilter(
				String testName, 
				String bladeId, 
				String eventTypeLabel, 
				String fieldLabel, 
				String fieldFilterKeyword, 
				String filterOperator){
			return fdcItf.fieldFilter(
					testName, 
					bladeId, 
					eventTypeLabel, 
					fieldLabel, 
					fieldFilterKeyword, 
					filterOperator);
		}

		
		public List<BladeEvent> fieldFilter(List<BladeEvent> bladeEventList,
				String fieldLabel, String fieldFilterKeyword,
				String filterOperator) {
			return fdcItf.fieldFilter(bladeEventList, fieldLabel, fieldFilterKeyword, filterOperator);
		}

		public List<BladeEvent> getBladeAllEventsValuesFiltered(
				List<BladeEvent> bladeEventList,
				Map<String, Map<String, String>> filter, String linkingOp) {
			return fdcItf.getBladeAllEventsValuesFiltered(bladeEventList, filter, linkingOp);
		}
		
		public List<BladeEvent> getBladeByTime(List<BladeEvent> bladeEventList,
				int start, int end) {
			return fdcItf.getBladeByTime(bladeEventList, start, end);
		}
		
		////////////////////////////////////
	    //interface AggregateDataControl  //
	    ////////////////////////////////////
		/* (non-Javadoc)
	     * @see org.ow2.clif.analyze.api.graph.VirtualDataSupplier#aggregateBladesOneSameEvent(java.util.list)
	     */
	    public List<BladeEvent>  aggregateBladesOneSameEvent(List<List<BladeEvent>> bladeList){
	    	return adcItf.aggregateBladesOneSameEvent(bladeList);
	    }

		

	

	    
	    ////////////////////////////////////
	    //interface StatisticDataControl  //
	    ////////////////////////////////////
	    

	    
	    ////////////////////////////////////
	    //interface RoutineSupplier  //
	    ////////////////////////////////////
	    
	    /* (non-Javadoc)
	     * @see org.ow2.clif.analyze.api.graph.VirtualDataSupplier#saveRoutineToXML(java.lang.String, java.util.Map, java.util.Map, java.util.Map, java.lang.String, java.util.Map, java.util.Map, java.util.Map)
	     */
	    public boolean saveRoutineToXML(
	    		String fileName, 
	    		Map<Integer, List<StatisticDataImpl>> statisticMap,
	    		Map<String, String> commentMap,
	    		Map<Integer, JFreeChart> jFreeChartMap,
	    		String imageType,
	    		Map<Integer, List<Integer>> pageIdAndDatasetIds,
	    		Map<Integer, ChartData> chartDataList,
	    		Map<Integer, DatasetOptions> datasetIdAndGraphOptions){
	    	return rsItf.saveRoutineToXML(
	    			fileName, 
	    			statisticMap, 
	    			commentMap, 
	    			jFreeChartMap, 
	    			imageType, 
	    			pageIdAndDatasetIds, 
	    			chartDataList, 
	    			datasetIdAndGraphOptions);
	    }
}
