/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.probe.cpu;

import org.objectweb.lewys.common.exceptions.NoResourceToProbeException;
import org.objectweb.lewys.probe.Probe;
import org.ow2.clif.probe.util.AbstractDumbInsert;
import org.ow2.clif.storage.api.ProbeEvent;
import org.ow2.clif.supervisor.api.ClifException;


/**
 * insert for CPU probe
 * @author Bruno Dillenseger
 * @author Emmanuel Varoquaux
 */
public class Insert extends AbstractDumbInsert
{
	private static final short OS_UNDEF = -1;
	private static final short OS_LINUX = 0;
	private static final short OS_WINDOWS = 1;
	private static final short OS_MACOSX = 2;


	private Probe probe;
	private int[] resourceIds;
	private short os = OS_UNDEF;
	private long[] previousValues = null;
	private int NBRESOURCE = 3;

	public Insert()
		throws ClifException
	{
		try
		{
			if (System.getProperty("os.name").equalsIgnoreCase("linux"))
			{
				probe = new org.objectweb.lewys.probe.linux.CpuProbe();
				//resourceIds = new int[] { 0, 2, 3 }; // user ticks, kernel ticks, idle ticks
				os = OS_LINUX;
			}
			else if (System.getProperty("os.name").startsWith("Windows"))
			{
				probe = new org.objectweb.lewys.probe.windows.CpuProbe();
				//resourceIds = new int[] { 1, 2, 0 }; // user ticks, priviledged ticks, all ticks
				os = OS_WINDOWS;
			}
			else if (System.getProperty("os.name").equals("Mac OS X"))
			{
				probe = new org.objectweb.lewys.probe.macosx.CpuProbe();
				//resourceIds = new int[] { 0, 2, 3 }; // user ticks, kernel ticks, idle ticks
				os = OS_MACOSX;
			}
			else
			{
				throw new ClifException("No CPU probe available for " + System.getProperty("os.name"));
			}
		}
		catch (NoResourceToProbeException ex)
		{
			throw new ClifException("Can't set CPU probe", ex);
		}
		
		eventStorageStatesMap.put("store-lifecycle-events", storeLifeCycleEvents);
		eventStorageStatesMap.put("store-alarm-events", storeAlarmEvents);
		eventStorageStatesMap.put("store-" + CPUEvent.EVENT_TYPE_LABEL + "-events", storeProbeEvents);
	}

protected void configProbewithArray() throws ClifException{ 
		
        String [] resourceNames = probe.getResourceNames();
        int cpu_index = 0;
        resourceIds = new int[NBRESOURCE];
        
        if (!arg_probe_config.isEmpty()) {
        	String cpuName = (String)arg_probe_config.get(0);
        	
            while (cpu_index < resourceNames.length && !(resourceNames[cpu_index].endsWith(cpuName +" user")))
            	cpu_index++;
        } else {
        	cpu_index = 0;
        }
        if (cpu_index < resourceNames.length) {
        	if (System.getProperty("os.name").equalsIgnoreCase("linux") ||
			    System.getProperty("os.name").equals("Mac OS X"))
			{
				resourceIds = new int[] { 0+ cpu_index, 
										  2+ cpu_index, 
										  3+ cpu_index 
									};
				}
			else if (System.getProperty("os.name").startsWith("Windows"))
			{
				resourceIds = new int[] { 1+ cpu_index, 2+ cpu_index, 0+ cpu_index }; 
			}
			else
			{
				throw new ClifException("No CPU probe available for " + System.getProperty("os.name"));
			}
        	
        	return; 
        }
      
    }
	public ProbeEvent doProbe()
	{
		CPUEvent event = null;
		try
		{
			long[] values = probe.getValue(resourceIds);
			switch(os)
			{
				case OS_LINUX:
				case OS_MACOSX:
					values[2] += values[0] + values[1]; // all ticks = user ticks + kernel ticks + idle ticks
					break;
			}
			if (previousValues == null)
			{
				previousValues = values;
			}
			else
			{
				long[] result = new long[3];
				long delta = values[2] - previousValues[2];
				if (delta != 0)
				{
					result[1] = 100 * (values[0] - previousValues[0]);
					result[2] = 100 * (values[1] - previousValues[1]);
					result[0] = (result[1] + result[2]) / delta;
					result[1] /= delta;
					result[2] /= delta;
					for (int i=0 ; i<2 ; i++)
					{
						if (result[i] > 100)
						{
							result[i] = 100;
						}
					}
					previousValues = values;
					event =  new CPUEvent(
						System.currentTimeMillis(),
						result);
				}
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace(System.err);
		}
		return event;
	}
}
