/*
* CLIF is a Load Injection Framework
* Copyright (C) 2008,2009 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.storage.lib.filestorage;

import java.io.File;

import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.util.ExecutionContext;

abstract class FileStorageCommons {
	static private String testDirbase = null;
	static private boolean testDirbaseFailed = false;
	static public final String COMMENT_PREFIX = "#";
	static public final String REPORT_DIR_PROP = "clif.filestorage.dir";
	static public final String REPORT_DIR_DEFAULT = "report";
	static public final String CSV_EXTENSION = ".csv";
	static public final String CLASSNAME_EXTENSION = ".classname";
	static public final String JVMPROPS_FILENAME = "server.prop";


	static private File getTestDir(String testName)
	{
		return new File(testDirbase, testName);
	}

	static File getBladeDir(String testName, String bladeId)
	{
		return new File(getTestDir(testName), bladeId);
	}

	static File getEventFile(String testName, String bladeId, String eventTypeLabel)
	{
		return new File(getBladeDir(testName, bladeId), eventTypeLabel);
	}

	static File getEventClassFile(String testName, String bladeId, String eventTypeLabel)
	{
		return new File(
			getBladeDir(testName, bladeId),
			eventTypeLabel + CLASSNAME_EXTENSION);
	}

	static boolean isCommentLine(String line)
	{
		return line != null && line.startsWith(COMMENT_PREFIX);
	}


	static String getTestDirbase()
		throws ClifException
	{
		if (! testDirbaseFailed)
		{
			synchronized (FileStorageCommons.class)
			{
				if (! testDirbaseFailed)
				{
					String dirname = System.getProperty(REPORT_DIR_PROP, REPORT_DIR_DEFAULT);
					File dir = new File(dirname);
					if (dir.isAbsolute())
					{
						testDirbase = dir.getAbsolutePath();
					}
					else if (ExecutionContext.WORKSPACE_PATH != null)
					{
						testDirbase = ExecutionContext.WORKSPACE_PATH + File.separator + dir;
					}
					else
					{
						testDirbase = ExecutionContext.getBaseDir() + dir;
					}
					File reportDir = new File(testDirbase);
					if (! reportDir.canWrite() && ! reportDir.mkdirs())
					{
						testDirbaseFailed = ! reportDir.canWrite();
					}
				}
			}
		}
		if (testDirbaseFailed)
		{
			throw new ClifException("Storage error: no write access to directory " + testDirbase);
		}
		else
		{
			return testDirbase;
		}
	}
}
