/*
* CLIF is a Load Injection Framework
* Copyright (C) 2005 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.lib.graph;

import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowStateListener;
import java.beans.PropertyVetoException;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.ow2.clif.analyze.api.AnalyzerLink;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.util.ExecutionContext;

import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;

import org.ow2.clif.analyze.api.graph.ChartControl;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.api.graph.ExportControl;
import org.ow2.clif.analyze.api.graph.VirtualDataSupplier;
import org.ow2.clif.analyze.lib.graph.ui.SwingGUI;

/**
 * The Class AnalyzerImpl.
 * 
 * @author Gregory Calonnier
 * @author Jordan Brunier
 */
public class AnalyzerImpl implements BindingController, AnalyzerLink, LifeCycleController
{
	
	/** Fractal ADL definition file of the CLIF Analyzer. */
	public static final String CLIF_ANALYZER = "org.ow2.clif.analyze.lib.graph.ClifAnalyzer";
	
	/** The Constant LABEL. */
	static private final String LABEL = "Quick Graphical Analyzer";

	// list of client interfaces names
	/** The itf list. */
	static private String[] itfList = new String[] { 
        ChartControl.CHART_CONTROL,
        ExportControl.EXPORT_CONTROL, 
        DataSupplier.DATA_SUPPLIER,
        VirtualDataSupplier.VIRTUAL_DATA_SUPPLIER
	};


	// field for client binding (access to stored data)
    /** The dp itf. */
	private DataSupplier dpItf;
    
    /** The cc itf. */
    private ChartControl ccItf;
    
    /** The ec itf. */
    private ExportControl ecItf;
    
    /** The vds itf. */
    private VirtualDataSupplier vdsItf;
    
	/** The gui. */
	SwingGUI gui = null;

	private static JDesktopPane desktop;
	
	public static JInternalFrame quickAnalyzer;
	
	public static JFrame frame;
	
	/**
     * The main method.
     * 
     * @param args the arguments
     */
	static public void main(String[] args)
	{
		try
		{
	        if(System.getSecurityManager() == null) {
	            System.setSecurityManager(new SecurityManager());
	        }
            ExecutionContext.init("./");
            init();
            frame.setVisible(true);
		}
		catch (Throwable t)
		{
			t.printStackTrace(System.err);
			System.exit(-1);
		}
	}

    public static void init() throws ADLException, NoSuchInterfaceException, IllegalLifeCycleException, PropertyVetoException {
        Component clifApp = (Component) FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND
            ).newComponent(
                CLIF_ANALYZER,
                null);
        Map components = new HashMap();
        Component analyzerComp = null;
        Component[] subComp = Fractal.getContentController(clifApp).getFcSubComponents();
        for (int i=0 ; i<subComp.length ; ++i)
        {
            components.put(Fractal.getNameController(subComp[i]).getFcName(), subComp[i]);
            if (Fractal.getNameController(subComp[i]).getFcName().equals("analyzer"))
            {
                analyzerComp = subComp[i];
            }
        }

        Fractal.getLifeCycleController(clifApp).startFc();
        //End Launch Fractal---------------------------------------------------------------------

        //Create Frame
        frame = new JFrame(LABEL);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        Dimension screenDim = frame.getToolkit().getScreenSize();
        frame.setSize((screenDim.width*3)/4, (screenDim.height*3)/4);

        desktop = new JDesktopPane();
        frame.getContentPane().add(desktop);
        frame.addWindowStateListener(new WindowStateListener(){
            public void windowStateChanged(WindowEvent e) {
                windowFrameStateChanged(e);
            }

        });

        quickAnalyzer = new JInternalFrame(LABEL, true, false, true, true);
//			quickAnalyzer.setSize((screenDim.width*9)/16, (screenDim.height*9)/16);
        quickAnalyzer.setVisible(true);
        desktop.add(quickAnalyzer);
        quickAnalyzer.setMaximum(true);
        ((AnalyzerLink) analyzerComp.getFcInterface(AnalyzerLink.ANALYZER_LINK)).setUIContext(quickAnalyzer);
    }

    protected static void windowFrameStateChanged(WindowEvent e) {
		Dimension frameDim = new Dimension();
		frameDim = frame.getSize();
		quickAnalyzer.setSize(frameDim);
	}

	/**
     * Instantiates a new analyzer impl.
     */
	public AnalyzerImpl() {
		
	}
	
	/////////////////////////////////
	// interface BindingController //
	/////////////////////////////////


	/* (non-Javadoc)
	 * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	 */
	public Object lookupFc(String clientItfName)
	{
		if (clientItfName.equals(DataSupplier.DATA_SUPPLIER)){
			return dpItf;
		}
		else if (clientItfName.equals(ChartControl.CHART_CONTROL)){
            return ccItf;
        }
        else if (clientItfName.equals(ExportControl.EXPORT_CONTROL)){
            return ecItf;
        }
        else if (clientItfName.equals(VirtualDataSupplier.VIRTUAL_DATA_SUPPLIER)){
            return vdsItf;
        }
        else{
			return null;
		}
	}


	/* (non-Javadoc)
	 * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
	 */
	public synchronized void bindFc(String clientItfName, Object serverItf)
	{
		if (clientItfName.equals(DataSupplier.DATA_SUPPLIER)){
			dpItf = (DataSupplier) serverItf;
		} 
		else if (clientItfName.equals(ChartControl.CHART_CONTROL)){
			ccItf = (ChartControl) serverItf;
		} 
		else if (clientItfName.equals(ExportControl.EXPORT_CONTROL)){
			ecItf = (ExportControl) serverItf;
		} 
		else if (clientItfName.equals(VirtualDataSupplier.VIRTUAL_DATA_SUPPLIER)){
			vdsItf = (VirtualDataSupplier) serverItf;
		} 
	}


	/* (non-Javadoc)
	 * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	 */
	public synchronized void unbindFc(String clientItfName)
	{
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER)){
            dpItf = null;
        } 
        else if (clientItfName.equals(ChartControl.CHART_CONTROL)){
            ccItf = null;
        } 
        else if (clientItfName.equals(ExportControl.EXPORT_CONTROL)){
            ecItf = null;
        }
        else if (clientItfName.equals(VirtualDataSupplier.VIRTUAL_DATA_SUPPLIER)){
            vdsItf = null;
        } 
	}


	/* (non-Javadoc)
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc()
	{
		return itfList;
	}


	////////////////////////////
	// interface AnalyzerLink //
	////////////////////////////


	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.AnalyzerLink#getLabel()
	 */
	public String getLabel()
	{
		return LABEL;
	}


	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.AnalyzerLink#setUIContext(java.lang.Object)
	 */
	public void setUIContext(Object internalFrame)
	{
		try
		{
			if (gui != null)
			{				
				((JInternalFrame)internalFrame).getContentPane().removeAll();
			}
			gui = new SwingGUI((JInternalFrame)internalFrame, this);
		}
		catch (ClifException ex)
		{
			throw new Error(
				"Can't find a compatible Analyzer user interface for " + internalFrame,
				ex);
		}
	}
	
	///////////////////////////////////
	// interface LifeCycleController //
	///////////////////////////////////

	/* (non-Javadoc)
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {
		// TODO Auto-generated method stub
		
	}
}
