/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Gr�gory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.graph;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.AbstractXYItemRenderer;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.ow2.clif.analyze.api.graph.ChartControl;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * The Class ChartEngineImpl.
 */
public class ChartEngineImpl implements BindingController, ChartControl {

    /** The instance. */
    private static ChartEngineImpl instance= new ChartEngineImpl();
    
    /** The interface list. */
    static private String[] itfList = new String[] { DataSupplier.DATA_SUPPLIER };

    /** The charts. key : chartId value : chart */
    /* jfreechart for a chart id */ 
    private Map<Integer, JFreeChart> charts;
    
    /** The last key. */
    private int lastKey;
    
    /** The X axis. key : chartId value : X field label */
    /* X axis name for a chart id */
    private Map<Integer, String> XAxis;
    
    /* map of the Y axis for a chart id */
    /** The Axis mapping. */
    private Map<Integer,Map<String, Integer>> AxisMapping;
    
    /* list of the Y series for a chart id */
    /** The Y axis. */
    private Map<Integer, ArrayList<String>> YAxis;

    // field for client binding (access to stored data)
    /** The data supplier interface. */
    private DataSupplier dsItf;
    
    /* collection of series for a chart id */
    /** The datasets. */
    private Map<Integer, XYSeriesCollection> datasets; 
    
    /** The windows max lenght. */
    private Map<Integer, Integer> windowsMaxLenght;
    
    /** The windows start. */
    private Map<Integer, Integer> windowsStart;
    
    /** The windows end. */
    private Map<Integer, Integer> windowsEnd;
    
    /** The lenght. */
    private static int start = 0, end = 2000, lenght = 1000;
    
    /**
     * Instantiates a new chart engine impl.
     */
    public ChartEngineImpl()
    { 
        lastKey = -1;
        charts = new HashMap<Integer, JFreeChart>();
        XAxis = new HashMap<Integer, String>();
        YAxis = new HashMap<Integer, ArrayList<String>>();
        AxisMapping = new HashMap<Integer, Map<String, Integer>>();
        datasets = new HashMap<Integer, XYSeriesCollection>();
        windowsMaxLenght = new HashMap<Integer, Integer>();
        windowsStart = new HashMap<Integer, Integer>();
        windowsEnd = new HashMap<Integer, Integer>();
    }
    
    
    /**
     * Gets the single instance of ChartEngineImpl.
     * 
     * @return single instance of ChartEngineImpl
     */
    public ChartEngineImpl getInstance()
    {
        return instance;
    }


    /////////////////////////////////
    // interface BindingController //
    /////////////////////////////////


    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            dsItf = (DataSupplier)serverItf;
        }
    }
    

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() 
    {
        return itfList;
    }

    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            return dsItf;
        }
        else
        {
            return null;
        }
    }

    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            dsItf = null;
        }
    }


    
    ////////////////////////////
    // interface ChartControl //
    ////////////////////////////


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#addYAxis(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public void addYAxis(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel) 
    {
        int datasetIndex;
        String key = testName +"/"+ bladeId +"/"+ eventTypeLabel +"/"+ fieldLabel;
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        XYSeries serie = new XYSeries(fieldLabel);
        String XField = XAxis.get(chartId);
        
        int tempMax = windowsMaxLenght.get(chartId).intValue();
        int tempStart =windowsStart.get(chartId).intValue();
        int tempEnd = windowsEnd.get(chartId).intValue();
        
        Object [][] values = dsItf.getFieldsValues(testName, bladeId, eventTypeLabel, XField, fieldLabel, tempStart, tempEnd, tempMax);
        
        double[] Xvalues = toDouble(values[0]);
        double[] Yvalues = toDouble(values[1]);
        
        for (int i = 0; i < Xvalues.length && Xvalues[i]<tempEnd; i++) 
        {
            serie.add(Xvalues[i], Yvalues[i]);
        }
        
        int axisIndex  = findKey(chartId, eventTypeLabel + fieldLabel);
       
        datasetIndex = YAxis.get(chartId).indexOf(key);
        if (datasetIndex ==-1)
        {
            add(chartId, key);
            datasetIndex = YAxis.get(chartId).indexOf(key);
        }
        else
        {
            plot.setDataset(datasetIndex,null);
        }
        
        plot.setDataset(datasetIndex, new XYSeriesCollection(serie));
        plot.setRenderer(datasetIndex, new XYLineAndShapeRenderer(true, false));
        if(axisIndex != -1)
        {
            plot.getRangeAxis(axisIndex).setVisible(true);
        }
        else 
        {
            ValueAxis axis = new NumberAxis(fieldLabel);
            plot.setRangeAxis(datasetIndex, axis);
            axisIndex = datasetIndex;
            AxisMapping.get(chartId).put(eventTypeLabel + fieldLabel, datasetIndex);
        }
        plot.mapDatasetToRangeAxis(datasetIndex, axisIndex);
       
        setDomainRange(chartId, tempStart, tempEnd);

        ((NumberAxis)(plot.getDomainAxis())).setUpperMargin(0.0);
        ((NumberAxis)(plot.getDomainAxis())).setLowerMargin(0.0);
        ((NumberAxis)(plot.getDomainAxis())).setAutoRangeIncludesZero(false);
        ((NumberAxis)(plot.getDomainAxis())).setAutoRange(true);
    }

    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#createNewChart()
     */
    public int createNewChart(String type) 
    {
        lastKey+=1;
        datasets.put(lastKey, new XYSeriesCollection());
        YAxis.put(lastKey, new ArrayList<String>());
        AxisMapping.put(lastKey, new HashMap<String, Integer>());
        windowsMaxLenght.put(lastKey, lenght);
        windowsStart.put(lastKey, start);
        windowsEnd.put(lastKey, end);
        charts.put(lastKey, ChartFactory.createXYLineChart( "", "", "", datasets.get(lastKey), PlotOrientation.VERTICAL , true, true, false));
        
        return lastKey;
    }

    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#getChart(int, java.lang.String)
     */
    public JPanel getChart(int chartId) 
    {
        return new ChartPanel((JFreeChart)charts.get(chartId));
    }

    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#removeYAxis(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public void removeYAxis(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel) 
    {
        String key = testName +"/"+ bladeId +"/"+ eventTypeLabel +"/"+ fieldLabel;
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        ArrayList<String> keys = YAxis.get(chartId);
        int index = keys.indexOf(key);
        
        
        //remove the serie
        ((XYSeriesCollection)(plot.getDataset(index))).removeSeries(0);
        plot.setDataset(index, null);
        keys.set(index, null);
        
        //remove the value axis
        if(countKey(chartId, eventTypeLabel + "/" +  fieldLabel)==0)
        {
            int index2  = findKey(chartId, eventTypeLabel + fieldLabel);
            plot.getRangeAxis(index2).setVisible(false);
            Map axisMap =AxisMapping.get(chartId);
            axisMap.remove(eventTypeLabel + fieldLabel);
        }
    }
    

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setChartTitle(int, java.lang.String)
     */
    public void setChartTitle(int chartId, String title) 
    {
        ((JFreeChart)charts.get(chartId)).setTitle(title);
    }

    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setXAxis(int, java.lang.String)
     */
    public void setXAxis(int chartId, String fieldName) 
    {
        XAxis.put(chartId, fieldName);
        ValueAxis domainAxis = new NumberAxis(fieldName);
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        plot.setDomainAxis(domainAxis);
    }
    

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setXAxisLabel(int, java.lang.String)
     */
    public void setXAxisLabel(int chartId, String label) 
    {
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        plot.getDomainAxis().setLabel(label);
    }
    

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setYAxisLabel(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public void setYAxisLabel(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel, String label) 
    {
        String key = testName +"/"+ bladeId +"/"+ eventTypeLabel +"/"+ fieldLabel;
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        int index = YAxis.get(chartId).indexOf(key);
        ((XYSeriesCollection)(plot.getDataset(index))).getSeries(0).setName(label);
        ((JFreeChart)charts.get(chartId)).fireChartChanged();
    }

    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#getChartImage(int)
     */
    public BufferedImage getChartImage(int chartId, int width, int height ) 
    {
        return ((JFreeChart)charts.get(chartId)).createBufferedImage(width, height);
    }

    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#getColor(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public Color getColor(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel) 
    {
        String key = testName +"/"+ bladeId +"/"+ eventTypeLabel +"/"+ fieldLabel;
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        int index = YAxis.get(chartId).indexOf(key);
        XYItemRenderer ren = plot.getRenderer(index);
        Color color = (Color) ren.getSeriesPaint(0);
        return color;
    }

    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setColor(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.awt.Color)
     */
    public void setColor(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel, Color color) 
    {
        String key = testName +"/"+ bladeId +"/"+ eventTypeLabel +"/"+ fieldLabel;
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        int index = YAxis.get(chartId).indexOf(key);
        plot.getRenderer(index).setSeriesPaint(0, color);
    }
    
    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setRenderer(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public void setRenderer(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel, String rendererType) 
    {
        int index;
        String key = testName +"/"+ bladeId +"/"+ eventTypeLabel +"/"+ fieldLabel;
        
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());

        index = YAxis.get(chartId).indexOf(key);
        if(rendererType == null)
        {
            rendererType = LINE_RENDERER;
        }
        ((XYSeriesCollection)(plot.getDataset(index))).getSeries(0);
        
        AbstractXYItemRenderer renderer= null;
        if(rendererType.equalsIgnoreCase(AREA_RENDERER))
        {
            renderer = new XYAreaRenderer(XYAreaRenderer.AREA);
        }
        else if(rendererType.equalsIgnoreCase(BAR_RENDERER))
        {
            renderer = new XYBarRenderer(0.20);
        }
        else if(rendererType.equalsIgnoreCase(DOT_RENDERER))
        {
            renderer = new XYLineAndShapeRenderer(false, true);
        }
        else
        {
            renderer = new XYLineAndShapeRenderer(true, false);
        }
        plot.setRenderer(index, renderer);
    }

    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#getRendererList()
     */
    public Map getRendererList() 
    {
        Map<String,String> rendererList = new HashMap<String,String>();
        
        rendererList.put("render as bar", BAR_RENDERER );
        rendererList.put("render as line", LINE_RENDERER);
        rendererList.put("render as area", AREA_RENDERER);
        rendererList.put("render as dot", DOT_RENDERER);
        return rendererList;
    }


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setYAxisMovingStatistics(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public void setYAxisMovingStatistics(int chartId, String testName, String bladeId, String eventTypeLabel, String fieldLabel, String movingStatistic, int timeWindow, int step) 
    {
        int index;
        String key = testName +"/"+ bladeId +"/"+ eventTypeLabel +"/"+ fieldLabel;
        
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());

        index = YAxis.get(chartId).indexOf(key);
        String XField = XAxis.get(chartId);
        int max = windowsMaxLenght.get(chartId).intValue();
        int start =(int)plot.getDomainAxis().getLowerBound();
        int end = (int)plot.getDomainAxis().getUpperBound();
        Object [][] values = dsItf.getFieldsValues(testName, bladeId, eventTypeLabel, XField, fieldLabel, start,end, max);
        
        if(movingStatistic.equals(DataSupplier.STAT_MAXIMUMS))
        {
            values = dsItf.getMaxValues(values, timeWindow, step);
        }
        else if (movingStatistic.equals(DataSupplier.STAT_MEANS))
        {
            values = dsItf.getMeans(values, timeWindow, step);
        }
        else if (movingStatistic.equals(DataSupplier.STAT_MINIMUMS))
        {
            values = dsItf.getMinValues(values, timeWindow, step);
        }
        else if (movingStatistic.equals(DataSupplier.STAT_STD_DEVIATION))
        {
            values = dsItf.getStandardDeviations(values, timeWindow, step);
        }

        XYSeries serie = new XYSeries(fieldLabel);
                
        double[] Xvalues = toDouble(values[0]);
        double[] Yvalues = toDouble(values[1]);
        
        for (int i = 0; i < Xvalues.length && Yvalues[i]<end; i++) {
            serie.add(Xvalues[i], Yvalues[i]);
        }
        
        plot.setDataset(index, new XYSeriesCollection(serie));
    }
    

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setDomainRange(int, int, int)
     */
    public void setDomainRange(int chartId, int lower, int upper)
    {
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        windowsStart.put(chartId, lower);
        windowsEnd.put(chartId, upper);
        plot.getDomainAxis().setRange(lower, upper);    
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#getLowerLimit(int)
     */
    public int getLowerLimit(int chartId) 
    {
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        return  (int)plot.getDomainAxis().getLowerBound();
    }


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#getMaxNumberOfPoint(int)
     */
    public int getMaxNumberOfPoint(int chartId) 
    {
        return windowsMaxLenght.get(chartId);
    }


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#getUpperLimit(int)
     */
    public int getUpperLimit(int chartId) 
    {
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        return  (int)plot.getDomainAxis().getUpperBound();
    }


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.ChartControl#setMaxNumberOfPoint(int, int)
     */
    public void setMaxNumberOfPoint(int chartId, int number) 
    {
        windowsMaxLenght.put(chartId, number);
    }
    
    ///////////////////////
    // private functions //
    ///////////////////////
        
    
    /**
     * To double.
     * 
     * @param values the values
     * 
     * @return the double[]
     */
    private double[] toDouble(Object[] values){
        
        double[] vals = new double[values.length];
        if (values.length>0){
            if(values[0] instanceof Number)
            {
                for (int i = 0; i < values.length && values[i]!=null; i++) {
                    vals[i] = ((Number)values[i]).doubleValue();
                }
            }
            else if (values[0] instanceof Boolean)
            {
                for (int i = 0; i < values.length && values[i]!=null; i++) {
                    vals[i] = ((Boolean)values[i]).booleanValue()?1.0:0.0;
                }
            }
            else
            {
                for (int i = 0; i < values.length && values[i]!=null; i++) {
                    vals[i] = 1.0;
                }
            }
        }
        return vals;
    }
    
    
    
    /**
     * Find a key similar to the given key in the given map.
     * 
     * @param key the key to find
     * @param chartId the chart id
     * 
     * @return the key
     */
    private int findKey(int chartId, String key)
    {
        Map mapping = AxisMapping.get(chartId);
        Integer index = (Integer)(mapping.get(key));

        if(index != null)
        {
            return index.intValue();
        }
        return -1;
    }

    /**
     * Count key.
     * 
     * @param chartId the chart id
     * @param key the key
     * 
     * @return the int
     */
    private int countKey(int chartId, String key)
    {
        ArrayList keys = YAxis.get(chartId);
        int cpt = 0;
        for (int i = 0; i < keys.size(); i++) {
            String str = ((String)keys.get(i));
            if(str != null && str.endsWith(key))
            {
                cpt++;
            }
        }
        return cpt;
    }
    
    /**
     * Add.
     * 
     * @param chartId the chart id
     * @param key the key
     */
    private void add(int chartId, String key)
    {
        int index = YAxis.get(chartId).indexOf(null);
        if (index != -1)
        {
            YAxis.get(chartId).set(index, key);
        }
        else
        {
            YAxis.get(chartId).add(key);
        }

    }
    
    /**
     * Update the chart.
     * 
     * @param chartId the chart id
     */
    private void updateChart(int chartId)
    {
        String key;
        XYSeries serie;
        Object [][] values;
        String[] axisName;
        
        ArrayList axis = YAxis.get(chartId);
        String XField = XAxis.get(chartId);
        XYPlot plot = (XYPlot)(((JFreeChart)charts.get(chartId)).getPlot());
        int tempMax = windowsMaxLenght.get(chartId).intValue();
        int tempStart =windowsStart.get(chartId).intValue();
        int tempEnd = windowsEnd.get(chartId).intValue();
        
        for (int i = 0; i < axis.size(); i++) 
        {
            key = ((String)(axis.get(i)));
            if(key != null){
                axisName = ((String)(axis.get(i))).split("/");
                
                serie = new XYSeries(axisName[3]);
                values = dsItf.getFieldsValues(axisName[0], axisName[1], axisName[2], XField, axisName[3], tempStart, tempEnd, tempMax);
    
                double[] Xvalues = toDouble(values[0]);
                double[] Yvalues = toDouble(values[1]);
                
                for (int j = 0; j < Xvalues.length && Yvalues[j]<end; j++) {
                    serie.add(Xvalues[j], Yvalues[j]);
                }
                
                plot.setDataset(i, new XYSeriesCollection(serie));
            }
        }
    }
}
