/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.graph.ui;

import java.awt.Color;

import javax.swing.JOptionPane;


/**
 * 
 * @author Jordan BRUNIER, Grégory CALONNIER
 *
 */
public class DialogMovingStatisticPreferences extends javax.swing.JDialog {
    
	private static final Color ERROR_AREA_COLOR = new Color(255,0,0);
	
	private DataQuickViewMenu menu;
	
	
    /**
     * Creates new form DialogMovingStatisticPreferences.
     * 
     * @param _menu the _menu
     * @param timeWindowPrev the time window prev
     * @param stepPrev the step prev
     */
    public DialogMovingStatisticPreferences(DataQuickViewMenu _menu, 
    		int timeWindowPrev,
    		int stepPrev) {
        initComponents();
        
        this.menu = _menu;
        jtfTimeWindow.setText( Integer.toString(timeWindowPrev) );
        jtfStep.setText( Integer.toString(stepPrev) );
        
        //window center
        this.setLocationRelativeTo(this.getParent());
    }

    
    private void initComponents() {
        jpTimeWindowStep = new javax.swing.JPanel();
        jpTimeWindow = new javax.swing.JPanel();
        jlTimeWindowTitle = new javax.swing.JLabel();
        jlTimeWindowMS = new javax.swing.JLabel();
        jtfTimeWindow = new javax.swing.JTextField();
        jpStep = new javax.swing.JPanel();
        jlStepTitle = new javax.swing.JLabel();
        jlStepMS = new javax.swing.JLabel();
        jtfStep = new javax.swing.JTextField();
        jpButtons = new javax.swing.JPanel();
        jbCancel = new javax.swing.JButton();
        jbOk = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Moving Statistic");
        setAlwaysOnTop(true);
        setModal(true);
        setResizable(false);
        jpTimeWindowStep.setLayout(new javax.swing.BoxLayout(jpTimeWindowStep, javax.swing.BoxLayout.Y_AXIS));

        jpTimeWindowStep.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Preferences:", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 1, 11), new java.awt.Color(0, 102, 204)));
        jpTimeWindow.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jlTimeWindowTitle.setFont(new java.awt.Font("Dialog", 0, 12));
        jlTimeWindowTitle.setText("Time window:");
        jpTimeWindow.add(jlTimeWindowTitle);

        jtfTimeWindow.setMinimumSize(new java.awt.Dimension(100, 19));
        jtfTimeWindow.setPreferredSize(new java.awt.Dimension(100, 19));
        jpTimeWindow.add(jtfTimeWindow);

        jlTimeWindowMS.setFont(new java.awt.Font("Dialog", 0, 12));
        jlTimeWindowMS.setText("ms");
        jpTimeWindow.add(jlTimeWindowMS);
        
        jpTimeWindowStep.add(jpTimeWindow);

        jpStep.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jlStepTitle.setFont(new java.awt.Font("Dialog", 0, 12));
        jlStepTitle.setText("Step:             ");
        jpStep.add(jlStepTitle);

        jtfStep.setMinimumSize(new java.awt.Dimension(100, 19));
        jtfStep.setPreferredSize(new java.awt.Dimension(100, 19));
        jpStep.add(jtfStep);

        jlStepMS.setFont(new java.awt.Font("Dialog", 0, 12));
        jlStepMS.setText("ms");
        jpStep.add(jlStepMS);
        
        jpTimeWindowStep.add(jpStep);

        getContentPane().add(jpTimeWindowStep, java.awt.BorderLayout.CENTER);

        jbCancel.setFont(new java.awt.Font("Dialog", 0, 12));
        jbCancel.setText("Cancel");
        jbCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbCancelActionPerformed(evt);
            }
        });
        
        jpButtons.add(jbCancel);

        jbOk.setFont(new java.awt.Font("Dialog", 0, 12));
        jbOk.setText("Ok");
        jbOk.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbOkActionPerformed(evt);
            }
        });
        
        jpButtons.add(jbOk);

        getContentPane().add(jpButtons, java.awt.BorderLayout.SOUTH);

        pack();
    }
    
    private void jbCancelActionPerformed(java.awt.event.ActionEvent evt) {
    	this.setVisible(false);
    	this.dispose();
    }

    private void jbOkActionPerformed(java.awt.event.ActionEvent evt) {
    	//Number format tests
        try {
        	Integer.parseInt(jtfTimeWindow.getText());
        } catch(NumberFormatException e) {
            JOptionPane.showMessageDialog(this, "Only integer value for time window.", "Warning", JOptionPane.WARNING_MESSAGE);            
            jtfTimeWindow.setBackground(DialogMovingStatisticPreferences.ERROR_AREA_COLOR);
            return;
        }
        
        try {
        	Integer.parseInt(jtfStep.getText());
        } catch(NumberFormatException e) {
            JOptionPane.showMessageDialog(this, "Only integer value for step.", "Warning", JOptionPane.WARNING_MESSAGE);            
            jtfStep.setBackground(DialogMovingStatisticPreferences.ERROR_AREA_COLOR);
            return;
        }
    	
        //It's numbers
        this.menu.updateMovingStatPreferences( Integer.parseInt(jtfTimeWindow.getText()),
        		Integer.parseInt(jtfStep.getText()) );
    	
    	this.setVisible(false);
    	this.dispose();
    }

    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jbOk;
    private javax.swing.JLabel jlStepTitle;
    private javax.swing.JLabel jlStepMS;
    private javax.swing.JLabel jlTimeWindowTitle;
    private javax.swing.JLabel jlTimeWindowMS;
    private javax.swing.JPanel jpButtons;
    private javax.swing.JPanel jpStep;
    private javax.swing.JPanel jpTimeWindow;
    private javax.swing.JPanel jpTimeWindowStep;
    private javax.swing.JTextField jtfStep;
    private javax.swing.JTextField jtfTimeWindow;    
}
