/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2008 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Olivier LIU
 *
 *
 */

package org.ow2.clif.analyze.lib.graph.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListSelectionModel;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.ow2.clif.analyze.lib.graph.FieldFilterAndOp;
import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.lib.util.NameBladeFilter;

/**
 * SimpleDataset class
 * 
 * @author Olivier LIU
 * @author Tomas Perez Segovia
 */
public class SimpleDataset extends JFrame
{

	private static final long serialVersionUID = -6950892841497480610L;

	// QuickViewContainer : Main panel
	private static DataQuickViewContainer dataQuickViewContainer;

	// SwingGUI
	private SwingGUI swingGUI;

	// SIMPLE DATA
	private String testName;
	private String bladeId;
	private String fieldSelected;
	private String eventSelected = null;
	// MULTIPLE DATA
	private List<String> bladeIdList;
	// AGGREGATE
	private List<NameBladeFilter> nameFilterList;

	@SuppressWarnings("unused")
	private String className = null;

	// COMMON DATA
	private long timeStartFilter;
	private long timeEndFilter;
	// CLEANING parameters
	private double kParam;
	private double percentPointsKeptParam;
	// Filter are registered in this list
	private List<FieldFilterAndOp> filterList;
	// for multiple filter : logical operator between them
	private LogicalEnum logicalOp;

	// 3 values AND, OR, NONE
	public enum LogicalEnum
	{
		AND_OP, OR_OP, NONE_OP
	}

	private DatasetType datasetType;

	// Dataset Type
	public enum DatasetType
	{
		SIMPLE_DATASET, MULTIPLE_DATASET, AGGREGATE_DATASET
	}

	// allow to restore default time value
	private long maxTime;
	private long minTime;

	// Filter table model
	private DefaultTableModel modelTableFilter;

	// Vector added to dataQuickViewContainer
	private Vector<String> vData = new Vector<String>();

	// Status bar at bottom wizard
	private String statusTextField = "";

	// if field is selected : TRUE
	private boolean isFieldSelected;

	List<String> eventTypeList = new ArrayList<String>();

	// if TRUE : the dataset is created else it's a double click on the
	// dataset(modification)
	private boolean wizard2;

	// for debug
	private boolean DEBUG = false;

	// Use for the columns in the filter table
	private final int FILTER_TABLE_FIELD = 0;
	private final int FILTER_TABLE_OPERATOR = 1;
	private final int FILTER_TABLE_ARGUMENT = 2;
	private final int FILTER_TABLE_DELETE = 3;

	// in dataQuickViewContainer table
	private final int DATASET_COL_FIELD = 4;
	private final int DATASET_COL_DRAW = 6;

	// All filters listed in this map, field chosen, <operator, keyword>
	private Map<String, Map<String, String>> filter = new HashMap<String, Map<String, String>>();

	// operators used to filter
	private String[] operators = { "equals", "not equals", "contains",
			"not contains", "begins with", "ends with", "greater than",
			"less than", "greater than or equals", "less than or equals" };

	// string for multiple
	private String resultM;

	// string for aggregate
	private String resultA;

	private String bladeAlias = "";

	// draw options
	private boolean isDrawSelected;
	private int drawTimeStart;
	private int drawTimeEnd;

	// Icons
	private final ImageIcon DELETE_ICON = new ImageIcon(getClass()
			.getClassLoader().getResource("icons/delete.gif"));
	private final ImageIcon INJECTOR_ICON = new ImageIcon(getClass()
			.getClassLoader().getResource("icons/injector.png"));
	private final ImageIcon PROBE_ICON = new ImageIcon(getClass()
			.getClassLoader().getResource("icons/probe.png"));

	/** Creates new form SingleDataset */
	public SimpleDataset(SwingGUI _swingGUI)
	{
		this.swingGUI = _swingGUI;
		SimpleDataset.dataQuickViewContainer = _swingGUI
				.getQuickViewContainer();
		initComponentFrame1();
	}

	public SimpleDataset(SwingGUI _swingGUI, boolean wizard2, int datasetType)
	{
		this.swingGUI = _swingGUI;
		SimpleDataset.dataQuickViewContainer = _swingGUI
				.getQuickViewContainer();
		this.wizard2 = wizard2;
		assignDatasetType(datasetType);
		initComponentFrame1();
	}

	public SimpleDataset(SwingGUI _swingGUI, String testName, String bladeId)
	{
		this.testName = testName;
		this.bladeId = bladeId;
		this.swingGUI = _swingGUI;
		SimpleDataset.dataQuickViewContainer = _swingGUI
				.getQuickViewContainer();
		initComponentFrame2(testName, bladeId);
	}

	public SimpleDataset(String testName, String bladeId,
			List<String> bladeIdList, String eventSelected,
			boolean isFieldSelected, String fieldSelected,
			List<FieldFilterAndOp> filterList, LogicalEnum logicalOp,
			Long timeStartFilter, Long timeEndFilter, double kParam,
			double percentPointsKeptParam, long maxTime, int datasetType,
			String bladeAlias, boolean isDrawSelected, int drawTimeStart,
			int drawTimeEnd, SwingGUI _swingGUI)
	{
		this.swingGUI = _swingGUI;
		this.eventSelected = eventSelected;
		this.filterList = filterList;
		this.timeStartFilter = timeStartFilter;
		this.timeEndFilter = timeEndFilter;
		this.isFieldSelected = isFieldSelected;
		this.fieldSelected = fieldSelected;
		this.kParam = kParam;
		this.percentPointsKeptParam = percentPointsKeptParam;
		this.maxTime = maxTime;
		jpMain = new JPanel();
		this.add(jpMain);
		this.testName = testName;
		this.bladeId = bladeId;
		this.bladeIdList = bladeIdList;
		this.logicalOp = logicalOp;
		assignDatasetType(datasetType);
		this.bladeAlias = bladeAlias;
		this.isDrawSelected = isDrawSelected;
		this.drawTimeStart = drawTimeStart;
		this.drawTimeEnd = drawTimeEnd;
		initComponentFrame2(testName, bladeId);
	}

	// assign the correct dataset type to this dataset
	private DatasetType assignDatasetType(int datasetType)
	{
		if (datasetType == 0)
		{
			this.datasetType = DatasetType.SIMPLE_DATASET;
		} else if (datasetType == 1)
		{
			this.datasetType = DatasetType.MULTIPLE_DATASET;
		} else if (datasetType == 2)
		{
			this.datasetType = DatasetType.AGGREGATE_DATASET;
		} else
		{
			if (DEBUG)
			{
				System.out.println("WARNING in DatasetType");
			}
			return null;
		}
		return null;
	}

	private DefaultTableModel model = new javax.swing.table.DefaultTableModel(
			new Object[][] {

			}, new String[] { "Blade", "Type", "Class", "Server Name",
					"Arguments", "Comment" })
	{
		private static final long serialVersionUID = 8102563317567930579L;
		boolean[] canEdit = new boolean[] { false, false, false, false, false,
				false };

		public boolean isCellEditable(int rowIndex, int columnIndex)
		{
			return canEdit[columnIndex];
		}
	};

	// allow to insert to icon(injector icon or probe icon to leaf) selection
	// table
	public class MyTreeRenderer extends DefaultTreeCellRenderer
	{
		/**
		 *
		 */

		private static final long serialVersionUID = 5433449702837137508L;

		public Component getTreeCellRendererComponent(JTree tree, Object value,
				boolean sel, boolean expanded, boolean leaf, int row,
				boolean hasFocus)
		{
			super.getTreeCellRendererComponent(tree, value, sel, expanded,
					leaf, row, hasFocus);
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
			// condition to change the leaf
			if (node.isLeaf())
			{
				String testName = node.getParent().toString();
				String bladeName = node.toString();
				NameBladeFilter filter = new NameBladeFilter(bladeName);

				BladeDescriptor[] bladeDescriptorArray = swingGUI
						.getBladeDescriptor(testName, filter);
				// get only one blade descriptor
				String bladeClass = bladeDescriptorArray[0].getClassname();

// if(node.toString().startsWith("inj")){
				if (bladeClass.equalsIgnoreCase("IsacRunner"))
				{
					this.setIcon(INJECTOR_ICON);
				} else
				{
					this.setIcon(PROBE_ICON);
				}
			}
			return this;
		}
	}

	// allow to insert to icon(injector icon or probe icon to leaf) selection
	// table
	public class MyTableRenderer extends DefaultTableCellRenderer
	{
		/**
		 *
		 */
		private static final long serialVersionUID = 2114928694636416740L;

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column)
		{
			super.getTableCellRendererComponent(table, value, isSelected,
					hasFocus, row, column);
			// condition to change the leaf
			if (column == 1)
			{
				if (table.getValueAt(row, 2).toString().endsWith("IsacRunner"))
				{
					this.setHorizontalAlignment(JLabel.CENTER);
					this.setIcon(INJECTOR_ICON);
				} else
				{
					this.setHorizontalAlignment(JLabel.CENTER);
					this.setIcon(PROBE_ICON);
				}
			}
			return this;
		}
	}

	class InjectorImageRenderer extends DefaultTableCellRenderer
	{
		/**
		 *
		 */
		private static final long serialVersionUID = -7475922333761195531L;
		JLabel lbl = new JLabel();
		ImageIcon icon = null;

		InjectorImageRenderer()
		{
			icon = new ImageIcon(getClass().getClassLoader().getResource(
					"/icons/injector.png"));
		}

		public Component getTableableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column)
		{
			lbl.setText((String) value);
			lbl.setBackground(Color.WHITE);
			lbl.setHorizontalAlignment(JLabel.CENTER);
			lbl.setIcon(icon);

			return lbl;
		}
	}

	class ProbeImageRenderer extends DefaultTableCellRenderer
	{
		/**
		 *
		 */
		private static final long serialVersionUID = 8048523402227022108L;
		JLabel lbl = new JLabel();
		ImageIcon icon = null;

		ProbeImageRenderer()
		{
			icon = new ImageIcon(getClass().getClassLoader().getResource(
					"/icons/probe.png"));
		}

		public Component getTableableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column)
		{
			lbl.setText((String) value);
			lbl.setBackground(Color.WHITE);
			lbl.setHorizontalAlignment(JLabel.CENTER);
			lbl.setIcon(icon);

			return lbl;
		}
	}

	// init the 1st wizard page
	private void initComponentFrame1()
	{
		jbNext = new javax.swing.JButton();
		jbCancel = new javax.swing.JButton();
		jScrollPaneTree = new javax.swing.JScrollPane();
		jScrollTable = new javax.swing.JScrollPane();
		jTree1 = new javax.swing.JTree(this.swingGUI.getBladeExecutionTree());
		jlStatus = new javax.swing.JLabel();
		jScrollTable = new javax.swing.JScrollPane(jTree1);
		jTable1 = new javax.swing.JTable(model);
		jlSelectEvent = new JLabel("Select Event :");
		jspFrame = new javax.swing.JSplitPane();
		jpMain = new javax.swing.JPanel();
		jpTree = new javax.swing.JPanel();
		jpStatus = new javax.swing.JPanel();
		jpMainRight = new javax.swing.JPanel();
		jpEvent = new javax.swing.JPanel();
		jpButtons = new javax.swing.JPanel();
		jpTree = new javax.swing.JPanel();
		jpMainFilter = new javax.swing.JPanel();
		jpEventInside = new javax.swing.JPanel();

		// set tree selection mode
		if (this.datasetType == DatasetType.SIMPLE_DATASET)
		{
			jTree1.getSelectionModel().setSelectionMode(
			// single selection possible
					TreeSelectionModel.SINGLE_TREE_SELECTION);
			this.setTitle("Simple Dataset - Step 1: Selection");
		} else if (this.datasetType == DatasetType.MULTIPLE_DATASET
				|| this.datasetType == DatasetType.AGGREGATE_DATASET)
		{
			jTree1.getSelectionModel().setSelectionMode(
			// multiple selection possible
					TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
			if (this.datasetType == DatasetType.MULTIPLE_DATASET)
			{
				this.setTitle("Multiple Dataset - Step 1: Selection");
			} else
			{
				this.setTitle("Aggregate Dataset - Step 1: Selection");
			}
		} else
		{
			if (DEBUG)
			{
				System.out.println("WARNING TREE SELECTION");
			}
		}

		jTree1
				.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener()
				{
					public void valueChanged(
							javax.swing.event.TreeSelectionEvent evt)
					{
						jTreeValueChanged(evt);
					}
				});

		logicalOp = LogicalEnum.NONE_OP;

		DefaultTreeCellRenderer renderer = new MyTreeRenderer();
		jTree1.setCellRenderer(renderer);
		jbNext.setEnabled(false);

		jpTree.setBorder(javax.swing.BorderFactory
				.createTitledBorder("Simple Dataset"));

		jbNext.setFont(new java.awt.Font("Dialog", 0, 12));
		jbNext.setText("Next");
		jbNext.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jbNextActionPerformed(evt);
			}
		});

		// comboBox that contains the 2 icons(inj and probe)

		jbCancel.setFont(new java.awt.Font("Dialog", 0, 12));
		jbCancel.setText("Cancel");
		jbCancel.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jbPreviousActionPerformed(evt);
			}
		});
		jScrollPaneTree.setViewportView(jTree1);
		jTable1.setRowHeight(25);
		jScrollTable.setViewportView(jTable1);
		jTable1.setRowSelectionAllowed(false);

		jpButtons.add(jbCancel);
		jpButtons.add(jbNext);
		// Status bar at the bottom page
		jpMain.setLayout(new BorderLayout());
		jlStatus.setFont(new java.awt.Font("Dialog", 0, 12));
		jpStatus.add(jlStatus);
		jlStatus.setPreferredSize(new Dimension(800, 13));
		jlStatus.setMinimumSize(new Dimension(800, 13));
		jlStatus.setMaximumSize(new Dimension(800, 13));
		jpMain.add(jpStatus, BorderLayout.SOUTH);
		// Align the text field at left
		FlowLayout layout = (FlowLayout) jpStatus.getLayout();
		layout.setAlignment((int) LEFT_ALIGNMENT);
		// Left Main Component : JTree : test names and blade ids
		jspFrame.setOrientation(javax.swing.JSplitPane.HORIZONTAL_SPLIT);
		jspFrame.setLeftComponent(jScrollPaneTree);
		// Right Main Component : NORTH -> TABLE, CENTER : EVENTS, SOUTH :
		// BUTTONS
		jpMainRight.setLayout(new BorderLayout());
		jpMainRight.add(jScrollTable, BorderLayout.NORTH);
		FlowLayout jpEventLayout = (FlowLayout) jpEvent.getLayout();
		jpEventLayout.setAlignment((int) LEFT_ALIGNMENT);
		jpEventInside.add(jpEvent);
		jpEventInside.setPreferredSize(new Dimension(70, 70));
		jpEventInside.setMinimumSize(new Dimension(70, 70));
		jpEventInside.setMaximumSize(new Dimension(70, 70));
		jpMainRight.add(jpEventInside, BorderLayout.CENTER);
		FlowLayout jpButtonsLayout = (FlowLayout) jpButtons.getLayout();
		jpButtonsLayout.setAlignment((int) RIGHT_ALIGNMENT);
		jpMainRight.add(jpButtons, BorderLayout.SOUTH);
		jspFrame.setRightComponent(jpMainRight);
		jpMain.add(jspFrame, BorderLayout.CENTER);
		this.addComponentListener(new java.awt.event.ComponentAdapter()
		{
			public void componentResized(ComponentEvent e)
			{
				JFrame tmp = (JFrame) e.getSource();
				if (tmp.getWidth() < 400 || tmp.getHeight() < 400)
				{
					tmp.setSize(400, 400);
					tmp.repaint();
				}
			}
		});
		this.add(jpMain);
	}

	protected long max(long a, long b)
	{
		if (a > b)
			return a;
		return b;
	}

	protected long min(long a, long b)
	{
		if (a < b)
			return a;
		return b;
	}

	// init the 2nd wizard
	private void initComponentFrame2(String testName, String bladeId)
	{
		jpMainFilter = new javax.swing.JPanel();
		jpMainFilterRight = new javax.swing.JPanel();
		jpCenter = new javax.swing.JPanel();
		jpStatusFilter = new javax.swing.JPanel();
		jpMainFilteringLeft = new javax.swing.JPanel();
		jlStatusFilter = new javax.swing.JLabel();
		jTableFilter = new javax.swing.JTable();
		jScrollFilterTable = new javax.swing.JScrollPane();
		jbAddFilter = new javax.swing.JButton();
		jlLinkingFilter = new javax.swing.JLabel();
		jrbAnd = new javax.swing.JRadioButton();
		jrbOr = new javax.swing.JRadioButton();
		jpTimeFilter = new javax.swing.JPanel();
		jlTimeFilterInterval = new javax.swing.JLabel();
		jlTimeFilterStart = new javax.swing.JLabel();
		jtfTimeFilterStart = new javax.swing.JTextField(20);
		jlTimeFilterEnd = new javax.swing.JLabel();
		jtfTimeFilterEndFilter = new javax.swing.JTextField(20);
		jpButtonsFilter = new javax.swing.JPanel();
		jbPreviousFilter = new javax.swing.JButton();
		jbNextFilter = new javax.swing.JButton();
		jpTimeFilterStart = new javax.swing.JPanel();
		jpTimeFilterEnd = new javax.swing.JPanel();
		jpAddAndLogic = new javax.swing.JPanel();
		jpLogic = new javax.swing.JPanel();
		jpStAnChoose = new javax.swing.JPanel();
		jcbPerformStAn = new javax.swing.JCheckBox();
		jpStatAnalysis = new javax.swing.JPanel();
		jpCleaning = new javax.swing.JPanel();
		jpKCleaningDescription = new javax.swing.JPanel();
		jpCleaningK = new javax.swing.JPanel();
		jpCleaningPercentPointsKept = new javax.swing.JPanel();
		jlCleaningKDescription = new javax.swing.JLabel(
				"Discard values such than |value-mean| > K * std_deviation.");
		jlCleaningKDescription.setEnabled(false);
		jlCleaningK = new javax.swing.JLabel("K      ");
		jlCleaningPercentPointsKept1 = new javax.swing.JLabel("keep at least ");
		jlCleaningPercentPointsKept2 = new javax.swing.JLabel(
				" % points (Not Implemented Yet)");
		jtfCleaningK = new javax.swing.JTextField(20);
		jtfCleaningPercentPointsKept = new javax.swing.JTextField(3);
// jbDefaultStart = new javax.swing.JButton();
		jbTimeFilterDefaultEnd = new javax.swing.JButton();
// jbgAndOr = new javax.swing.ButtonGroup();
		jcbStAnField = new javax.swing.JComboBox();
		jpTimeFilterDefault = new javax.swing.JPanel();
		jbCancelFilter = new javax.swing.JButton();
		jlStatus = new javax.swing.JLabel();
		jcbDrawOptionsDraw = new javax.swing.JCheckBox();
		jlDrawOptionsDrawStart = new javax.swing.JLabel();
		jlDrawOptionsDrawEnd = new javax.swing.JLabel();
		jtfDrawOptionsDrawStart = new javax.swing.JTextField(8);
		jtfDrawOptionsDrawEnd = new javax.swing.JTextField(8);
		jpDrawOptions_ = new javax.swing.JPanel();
		jpSpareBottom = new javax.swing.JPanel();
		jpSpareRight = new javax.swing.JPanel();

		jpDrawOptionsStart = new javax.swing.JPanel();
		jpDrawOptionsEnd = new javax.swing.JPanel();
// jpDrawOptionsDefault = new javax.swing.JPanel();
		jbDrawOptionsDefault = new javax.swing.JButton();

		InputStream is = this.getClass().getClassLoader().getResourceAsStream(
				"compile.timestamp");
		BufferedReader bf = new BufferedReader(new InputStreamReader(is));
		String timeStamp;
		timeStamp = null;
		try
		{
			timeStamp = bf.readLine();
		} catch (IOException e1)
		{
			System.out.println("SimpleDataset.java / initComponentFrame2\n" 
					+ "reading TimeStamp.\n"
					+ "Exception: " + e1);
			e1.printStackTrace();
		}

		if (this.datasetType == DatasetType.SIMPLE_DATASET)
		{
			setTitle("Simple Dataset - Step 2: Filtering and cleaning");
		} else if (this.datasetType == DatasetType.MULTIPLE_DATASET)
		{
			setTitle("Multiple Dataset - Step 2: Filtering and cleaning");
		} else
		{
			setTitle("Aggregate Dataset - Step 2: Filtering and cleaning");
		}

		jlStatusFilter.setFont(new java.awt.Font("Dialog", 0, 12));

		// update status field
		updateStatusField();

		initFilterTableModel();
		jTableFilter.setModel(modelTableFilter);
		jScrollFilterTable.setViewportView(jTableFilter);
		jScrollFilterTable.addMouseListener(new MouseListener()
		{
			public void mouseClicked(MouseEvent e)
			{
// // validate cell when clicked outside
// JScrollPane jsp = (JScrollPane) e.getSource();
// try{
// Component comp = getFocusOwner();
// if(comp.getName().equals("Table.editor")){
// Color tmp = jTableFilter.getSelectionBackground();
// comp.setBackground(tmp);
// }
// }catch (Exception ex) {
// System.out.println(ex);
// }
// jsp.requestFocus();
			}

			public void mouseEntered(MouseEvent e)
			{
				// NOTHING TO DO
			}

			public void mouseExited(MouseEvent e)
			{
				// NOTHING TO DO
			}

			public void mousePressed(MouseEvent e)
			{
				// NOTHING TO DO
			}

			public void mouseReleased(MouseEvent e)
			{
				// NOTHING TO DO
			}
		});

		jTableFilter.addMouseListener(new MouseListener()
		{
			public void mouseClicked(MouseEvent e)
			{
				int viewRow = jTableFilter.getSelectedRow();
				int viewCol = jTableFilter.getSelectedColumn();
				int rowCount;
				if (viewCol == FILTER_TABLE_DELETE)
				{
					modelTableFilter.removeRow(viewRow);
					rowCount = jTableFilter.getRowCount();
					if (rowCount < 2)
					{
						jpLogic.setVisible(false);
					} else
					{
						jpLogic.setVisible(true);
					}
				}
			}

			public void mouseEntered(MouseEvent e)
			{
				// DO NOTHING
			}

			public void mouseExited(MouseEvent e)
			{
				// DO NOTHING
			}

			public void mousePressed(MouseEvent e)
			{
				// DO NOTHING
			}

			public void mouseReleased(MouseEvent e)
			{
				// DO NOTHING
			}
		});
		initFilterRendererAndEditor();
		jbAddFilter.setFont(new java.awt.Font("Dialog", 0, 12));
		jbAddFilter.setText("Add Filter");
		jbAddFilter.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jbAddFilterActionPerformed(evt);
			}
		});

		jrbAnd.setFont(new java.awt.Font("Dialog", 0, 12));
		jrbAnd.setText("AND");

		jlLinkingFilter.setText("Logical operator :");

		jrbOr.setFont(new java.awt.Font("Dialog", 0, 12));
		jrbOr.setText("OR");

		jlTimeFilterInterval.setText("Time interval :");

		jlTimeFilterStart.setText("Start (ms) ");
		minTime = 0;
		jtfTimeFilterStart.setText(String.valueOf(minTime));
		jtfTimeFilterStart.addKeyListener(new java.awt.event.KeyListener()
		{
			public void keyPressed(KeyEvent evt)
			{
				// JTextField modification
				jtfTimeFilterStart.setBackground(Color.YELLOW);
				if (evt.getKeyCode() == KeyEvent.VK_ENTER)
				{
					// put focus on the parent
					jtfTimeFilterStart.setBackground(Color.WHITE);
					timeStartFilter = Long.parseLong(jtfTimeFilterStart.getText());
					jtfTimeFilterStart.setText(String.valueOf(max(minTime, timeStartFilter)));
					jtfTimeFilterStart.getParent().requestFocus();

					step2TimesVerification();
				}
			}

			public void keyReleased(KeyEvent e)
			{

			}

			public void keyTyped(KeyEvent e)
			{

			}
		});

		jtfTimeFilterStart.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jtfStartActionPerformed(evt);
			}
		});

		jtfTimeFilterStart.addFocusListener(new java.awt.event.FocusListener()
		{
			public void focusGained(FocusEvent e)
			{
			}

			public void focusLost(FocusEvent e)
			{
				jtfTimeFilterStart.setBackground(Color.WHITE);
				timeStartFilter = Long.parseLong(jtfTimeFilterStart.getText());
				jtfTimeFilterStart.setText(String.valueOf(max(minTime, timeStartFilter)));
				jtfTimeFilterStart.getParent().requestFocus();

				step2TimesVerification();
			} // focusLost
		});

		jlTimeFilterEnd.setText("End   (ms) ");
		maxTime = this.getMaxTime(testName, bladeId);
		jtfTimeFilterEndFilter.setText(String.valueOf(maxTime));
		jtfTimeFilterEndFilter.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jtfEndActionPerformed(evt);
			}
		});
		jtfTimeFilterEndFilter.addKeyListener(new java.awt.event.KeyListener()
		{
			public void keyPressed(KeyEvent evt)
			{
				// JTextField modification
				jtfTimeFilterEndFilter.setBackground(Color.YELLOW);
				if (evt.getKeyCode() == KeyEvent.VK_ENTER)
				{
					// put focus on the parent
					timeEndFilter = Long.parseLong(jtfTimeFilterEndFilter
							.getText());
					jtfTimeFilterEndFilter.setText(String.valueOf(
							min(timeEndFilter, maxTime)));
					jtfTimeFilterEndFilter.setBackground(Color.WHITE);
					jtfTimeFilterStart.getParent().requestFocus();

					step2TimesVerification();
				}
			}


			public void keyReleased(KeyEvent e)
			{
			}

			public void keyTyped(KeyEvent e)
			{
			}
		});

		jtfTimeFilterEndFilter.addFocusListener(new java.awt.event.FocusListener()
		{
			public void focusGained(FocusEvent e)
			{
			}

			public void focusLost(FocusEvent e)
			{
				jtfTimeFilterEndFilter.setBackground(Color.WHITE);
				timeEndFilter = Long.parseLong(jtfTimeFilterEndFilter.getText());
				jtfTimeFilterEndFilter.setText(String.valueOf(max(minTime, timeEndFilter)));
				jtfTimeFilterEndFilter.getParent().requestFocus();
				step2TimesVerification();
			} // focusLost
		});


		jbNextFilter.setFont(new java.awt.Font("Dialog", 0, 12));
		jbNextFilter.setText("Finish");
		jbNextFilter.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jbFinishActionPerformed(evt);
			}
		});

		jbPreviousFilter.setFont(new java.awt.Font("Dialog", 0, 12));
		jbPreviousFilter.setText("Previous");
		jbPreviousFilter.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jbPreviousFilterActionPerformed(evt);
			}
		});

		jbCancelFilter.setFont(new java.awt.Font("Dialog", 0, 12));
		jbCancelFilter.setText("Cancel");
		jbCancelFilter.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jbCancelActionPerformed(evt);
			}
		});

		jlStatus.setFont(new java.awt.Font("Dialog", 0, 12));
		jpStatusFilter.add(jlStatusFilter);
		jpMainFilter.setLayout(new BorderLayout());

// FlowLayout jpStatusFilterLayout = (FlowLayout) jpStatusFilter.getLayout();
// jpStatusFilterLayout.setAlignment((int) LEFT_ALIGNMENT);
		jpMainFilter.add(jpStatusFilter, BorderLayout.SOUTH);
		// Left main component : Time interval + button ADD FILTER +
		// JtableFILTER + Logical Operator
		jpCenter.setLayout(new BorderLayout());
		jpMainFilteringLeft.setLayout(new BorderLayout());
		jpMainFilteringLeft.setBorder(javax.swing.BorderFactory
				.createTitledBorder(null, "Filtering",
						javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
						javax.swing.border.TitledBorder.DEFAULT_POSITION,
						new java.awt.Font("Dialog", 1, 11),
						MEDIUM_NAVY_BLUE_COLOR));

		jpStatAnalysis.setBorder(javax.swing.BorderFactory.createTitledBorder(
				null, "Statistic Analysis",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 1, 11), MEDIUM_NAVY_BLUE_COLOR));

		jpCleaning.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
				"Cleaning (Not Implemented Yet)",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 1, 11), UNIMPLEMENTED_COLOR));
		jpCleaning.setLayout(new BoxLayout(jpCleaning, BoxLayout.Y_AXIS));

		jlCleaningKDescription.setForeground(UNIMPLEMENTED_COLOR);
		jpKCleaningDescription.add(jlCleaningKDescription);

		jlCleaningK.setForeground(UNIMPLEMENTED_COLOR);
		jtfCleaningK.setForeground(UNIMPLEMENTED_COLOR);
		jpCleaningK.add(jlCleaningK);
		jpCleaningK.add(jtfCleaningK);
		jlCleaningK.setForeground(Color.GRAY);
		jtfCleaningK.setEnabled(false);

		jlCleaningPercentPointsKept1.setForeground(UNIMPLEMENTED_COLOR);
		jlCleaningPercentPointsKept2.setForeground(UNIMPLEMENTED_COLOR);
		jtfCleaningPercentPointsKept.setEnabled(false);
		jpCleaningPercentPointsKept.add(jlCleaningPercentPointsKept1);
		jpCleaningPercentPointsKept.add(jtfCleaningPercentPointsKept);
		jpCleaningPercentPointsKept.add(jlCleaningPercentPointsKept2);

		jpCleaning.add(jpKCleaningDescription);
		jpCleaning.add(jpCleaningK);
		jpCleaning.add(jpCleaningPercentPointsKept);

		jpTimeFilterStart.add(jlTimeFilterStart);
		jpTimeFilterStart.add(jtfTimeFilterStart);

		FlowLayout jpStartLayout = (FlowLayout) jpTimeFilterStart.getLayout();
		jpStartLayout.setAlignment((int) LEFT_ALIGNMENT);

		jpTimeFilterEnd.add(jlTimeFilterEnd);
		jpTimeFilterEnd.add(jtfTimeFilterEndFilter);
		jbTimeFilterDefaultEnd.setFont(new java.awt.Font("Dialog", 0, 12));
		jbTimeFilterDefaultEnd.setText("Default");
		jbTimeFilterDefaultEnd.setToolTipText("Restore default value");
		jbTimeFilterDefaultEnd.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jtfTimeFilterStart.setText("0");
				timeStartFilter = Long.parseLong(jtfTimeFilterStart.getText());
				jtfTimeFilterStart.setBackground(Color.WHITE);
				jtfTimeFilterEndFilter.setText(String.valueOf(maxTime));
				timeEndFilter = Long.parseLong(jtfTimeFilterEndFilter.getText());
				jtfTimeFilterEndFilter.setBackground(Color.WHITE);
			}
		});
		jpTimeFilterDefault.setSize(new Dimension(15, 5));
		jpTimeFilterDefault.add(jbTimeFilterDefaultEnd);
		jbTimeFilterDefaultEnd.addMouseListener(new MouseListener()
		{
			public void mouseClicked(MouseEvent e)
			{
				// NOTHING TO DO
			}

			public void mouseEntered(MouseEvent e)
			{
				jlStatusFilter.setText("Restore default value");
			}

			public void mouseExited(MouseEvent e)
			{
				updateStatusField();
			}

			public void mousePressed(MouseEvent e)
			{
				// NOTHING TO DO
			}

			public void mouseReleased(MouseEvent e)
			{
				// NOTHING TO DO
			}

		});

		jpTimeFilter.setLayout(new BorderLayout());
		jpTimeFilter.add(jlTimeFilterInterval, BorderLayout.NORTH);
		jpTimeFilter.add(jpTimeFilterStart, BorderLayout.WEST);
		jpTimeFilter.add(jpTimeFilterDefault, BorderLayout.CENTER);
		jpTimeFilter.add(jpTimeFilterEnd, BorderLayout.SOUTH);

		FlowLayout jpEndLayout = (FlowLayout) jpTimeFilterEnd.getLayout();
		jpEndLayout.setAlignment((int) LEFT_ALIGNMENT);

		jpAddAndLogic.setLayout(new BorderLayout());
		jpAddAndLogic.add(jbAddFilter, BorderLayout.WEST);
		jpLogic.add(jlLinkingFilter);
		jpLogic.add(jrbAnd);
		jrbAnd.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if (DEBUG)
				{
					logicalOp = LogicalEnum.AND_OP;
					if (DEBUG)
					{
						System.out.println("AND selected");
					}
				}
			}
		});
		if (wizard2 == true)
		{
			logicalOp = LogicalEnum.AND_OP;
			jrbAnd.setSelected(true);
		} else
		{
			if (this.logicalOp == LogicalEnum.AND_OP)
			{
				jrbAnd.setSelected(true);
			} else if (this.logicalOp == LogicalEnum.OR_OP)
			{
				jrbOr.setSelected(true);
			} else
			{
				if (DEBUG)
				{
					System.out.println("WARNING IN LOGICAL OP");
				}
			}
		}
		jpLogic.add(jrbOr);
		jrbOr.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if (DEBUG)
				{
					logicalOp = LogicalEnum.OR_OP;
					if (DEBUG)
					{
						System.out.println("OR selected");
					}
				}

			}
		});
		jpLogic.setVisible(false);
// jbgAndOr.add(jrbAnd);
// jbgAndOr.add(jrbOr);
		jpAddAndLogic.add(jpLogic, BorderLayout.EAST);

		jpMainFilteringLeft.add(jpTimeFilter, BorderLayout.NORTH);
		jpMainFilteringLeft.add(jScrollFilterTable, BorderLayout.CENTER);
		jpMainFilteringLeft.add(jpAddAndLogic, BorderLayout.SOUTH);

		jpCenter.add(jpMainFilteringLeft, BorderLayout.WEST);

		// Right main component : Field Selector + Reject + Buttons
		jpMainFilterRight.setLayout(new BoxLayout(jpMainFilterRight,
				BoxLayout.Y_AXIS));

		// Statistic Analysis Panel
		jcbPerformStAn.setText("Perform statistic analysis");
		jpStAnChoose.add(jcbPerformStAn);
		try
		{
			BladeEvent bladeEvent;
			if (this.datasetType == DatasetType.SIMPLE_DATASET)
			{
				List<BladeEvent> bladeEvList = this.swingGUI
						.getBladeOneEventValues(this.testName, this.bladeId,
								this.eventSelected);
				bladeEvent = bladeEvList.get(0);
			} else
			{
				List<BladeEvent> bladeEvList = this.swingGUI
						.getBladeOneEventValues(this.testName, bladeIdList
								.get(0), this.eventSelected);
				if (null != bladeEvList){
					bladeEvent = bladeEvList.get(0);
				}else{
					bladeEvent = null;
					System.out.println("SimpleDataset.java / initComponentFrame2\n" 
							+ " Performing statistic analysis\n"
							+ "bladeEvList == null");
				}
				
			}
			String[] fieldArray = bladeEvent.getFieldLabels();
			String[] fieldArrayWithoutData = new String[fieldArray.length];
			for (int i = 1; i < fieldArray.length; i++)
			{
				fieldArrayWithoutData[i - 1] = fieldArray[i];
			}
			for (int i = 0; i < fieldArrayWithoutData.length - 1; i++)
			{
				jcbStAnField.addItem(fieldArrayWithoutData[i]);
			}
			jlChooseField = new JList(fieldArrayWithoutData);
		} catch (Exception e)
		{
			System.out.println("SimpleDataset.java / initComponentFrame2\n" 
					+ " Performing statistic analysis\n"
					+ "Exception: " + e);
		}

		jcbStAnField.addItemListener(new ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				JComboBox jcb = (JComboBox) e.getSource();
				fieldSelected = jcb.getSelectedItem().toString();
			}

		});

		jlChooseField.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
		jlChooseField.addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				try
				{
					JList jListTemp = (JList) e.getSource();
					fieldSelected = jListTemp.getSelectedValue().toString();
				} catch (Exception ex)
				{
					System.out.println("SimpleDataset.java / valueChanged\n" 
							+ "Exception: " + ex);
				}
			}

		});
		jpStAnChoose.add(jcbStAnField);
		jcbStAnField.setEnabled(false);

		jpStAnChoose.updateUI();
		jcbPerformStAn.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jcbAnalysisActionPerformed(evt);
			}
		});
		jpStAnChoose.setLayout(new BoxLayout(jpStAnChoose, BoxLayout.X_AXIS));
		FlowLayout jpChooseAFieldLayout = (FlowLayout) jpStatAnalysis
				.getLayout();
		jpChooseAFieldLayout.setAlignment((int) LEFT_ALIGNMENT);
		jpStatAnalysis.add(jpStAnChoose, BorderLayout.NORTH);

		// Draw Options Panel
		jcbDrawOptionsDraw.setText("Draw");

		jcbDrawOptionsDraw
				.addActionListener(new java.awt.event.ActionListener()
				{
					public void actionPerformed(java.awt.event.ActionEvent evt)
					{
						jcbToDrawActionPerformed(evt);
					}
				});

		jlDrawOptionsDrawStart.setText("TimeStart (ms)");
		jlDrawOptionsDrawEnd.setText("TimeEnd (ms)");

		jpDrawOptions_.setBorder(javax.swing.BorderFactory.createTitledBorder(
				null, "Draw Options",
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION,
				new java.awt.Font("Dialog", 1, 11), MEDIUM_NAVY_BLUE_COLOR));

		jpDrawOptionsStart.add(jlDrawOptionsDrawStart);
		jpDrawOptionsStart.add(jtfDrawOptionsDrawStart);

		jpDrawOptionsEnd.add(jlDrawOptionsDrawEnd);
		jpDrawOptionsEnd.add(jtfDrawOptionsDrawEnd);
		jpDrawOptionsEnd.add(jbDrawOptionsDefault);

		jpDrawOptionsEnd.setBorder(javax.swing.BorderFactory
				.createTitledBorder(null, "Draw Options End Panel",
						javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
						javax.swing.border.TitledBorder.DEFAULT_POSITION,
						new java.awt.Font("Dialog", 1, 11),
						MEDIUM_NAVY_BLUE_COLOR));

		jpDrawOptionsStart.setBorder(
			javax.swing.BorderFactory.createTitledBorder(
					null, "Draw Options Start Panel",
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION,
					new java.awt.Font("Dialog", 1, 11),
					MEDIUM_NAVY_BLUE_COLOR));

		jbDrawOptionsDefault.setFont(new java.awt.Font("Dialog", 0, 12));
		jbDrawOptionsDefault.setText("Default Filtering Range");
		jbDrawOptionsDefault.setSize(new Dimension(15, 5));
		jbDrawOptionsDefault.setToolTipText("Restore default values");
		jbDrawOptionsDefault.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				jtfDrawOptionsDrawStart.setText("0");
				drawTimeStart = (int) Long.parseLong(jtfDrawOptionsDrawStart.getText());
				jtfDrawOptionsDrawStart.setBackground(Color.WHITE);
				jtfDrawOptionsDrawEnd.setText(String.valueOf(maxTime));
				drawTimeEnd = (int) Long.parseLong(jtfDrawOptionsDrawEnd.getText());
				jtfDrawOptionsDrawEnd.setBackground(Color.WHITE);
			}
		});
		jbDrawOptionsDefault.addMouseListener(new MouseListener()
		{
			public void mouseClicked(MouseEvent e)
			{
				// NOTHING TO DO
			}

			public void mouseEntered(MouseEvent e)
			{
				jlStatusFilter.setText("Restore time default values");
			}

			public void mouseExited(MouseEvent e)
			{
				updateStatusField();
			}

			public void mousePressed(MouseEvent e)
			{
				// NOTHING TO DO
			}

			public void mouseReleased(MouseEvent e)
			{
				// NOTHING TO DO
			}

		});

		FlowLayout jpDrawOptionsStartLayout = (FlowLayout) jpDrawOptionsStart.getLayout();
		jpDrawOptionsStartLayout.setAlignment((int) LEFT_ALIGNMENT);

		FlowLayout jpDrawOptionsEndLayout = (FlowLayout) jpDrawOptionsEnd.getLayout();
		jpDrawOptionsEndLayout.setAlignment((int) LEFT_ALIGNMENT);

		GridBagConstraints gGBConstraints = new GridBagConstraints();
		gGBConstraints.insets = new Insets(5, 5, 5, 5);
		GridBagLayout gestionnaire = new GridBagLayout();
		jpDrawOptions_.setLayout(gestionnaire);

		jcbDrawOptionsDraw.setAlignmentX(LEFT_ALIGNMENT);
		gGBConstraints.gridx = 0;
		gGBConstraints.gridy = 0;
		gGBConstraints.gridheight = 1;
		gGBConstraints.gridwidth = 1;
		gGBConstraints.weightx = 0;
		gGBConstraints.weighty = 0;
		gGBConstraints.fill = GridBagConstraints.VERTICAL;
		gestionnaire.setConstraints(jcbDrawOptionsDraw, gGBConstraints);
		jpDrawOptions_.add(jcbDrawOptionsDraw);

		// jlDrawOptionsDrawStart
		jlDrawOptionsDrawStart = new javax.swing.JLabel();
		jlDrawOptionsDrawStart.setText("TimeStart (ms)");
		gGBConstraints.gridx = 0;
		gGBConstraints.gridy = 1;
		gGBConstraints.gridheight = 1;
		gGBConstraints.gridwidth = 1;
		gGBConstraints.weightx = 0;
		gGBConstraints.weighty = 0;
		gGBConstraints.fill = GridBagConstraints.VERTICAL;
		gestionnaire.setConstraints(jlDrawOptionsDrawStart, gGBConstraints);
		jpDrawOptions_.add(jlDrawOptionsDrawStart);

		// jtfDrawOptionsDrawStart
		jtfDrawOptionsDrawStart = new javax.swing.JTextField(8);
		jtfDrawOptionsDrawStart.setText("<start>");
		gGBConstraints.gridx = 1;
		gGBConstraints.gridy = 1;
		gGBConstraints.gridheight = 1;
		gGBConstraints.gridwidth = 1;
		gGBConstraints.weightx = 0;
		gGBConstraints.weighty = 0;
		gestionnaire.setConstraints(jtfDrawOptionsDrawStart, gGBConstraints);
		jpDrawOptions_.add(jtfDrawOptionsDrawStart);

		// jlDrawOptionsDrawEnd
		jlDrawOptionsDrawEnd = new javax.swing.JLabel();
		jlDrawOptionsDrawEnd.setText("TimeEnd (ms)");
		gGBConstraints.gridx = 0;
		gGBConstraints.gridy = 2;
		gGBConstraints.gridheight = 1;
		gGBConstraints.gridwidth = 1;
		gGBConstraints.weightx = 0;
		gGBConstraints.weighty = 0;
		gestionnaire.setConstraints(jlDrawOptionsDrawEnd, gGBConstraints);
		jpDrawOptions_.add(jlDrawOptionsDrawEnd);

		// jtfDrawOptionsDrawEnd
		jtfDrawOptionsDrawEnd = new javax.swing.JTextField(8);
		jtfDrawOptionsDrawEnd.setText("<end>");
		gGBConstraints.gridx = 1;
		gGBConstraints.gridy = 2;
		gGBConstraints.gridheight = 1;
		gGBConstraints.gridwidth = 1;
		gGBConstraints.weightx = 0;
		gGBConstraints.weighty = 0;
		gestionnaire.setConstraints(jtfDrawOptionsDrawEnd, gGBConstraints);
		jpDrawOptions_.add(jtfDrawOptionsDrawEnd);

		// jbDrawOptionsDefault
		jbDrawOptionsDefault = new javax.swing.JButton();
		jbDrawOptionsDefault.setFont(new java.awt.Font("Dialog", 0, 12));
		jbDrawOptionsDefault.setText("Default");		// "Draw Default"
		jbDrawOptionsDefault.setSize(new Dimension(15, 5));
		jbDrawOptionsDefault.setToolTipText("Restore default value");
		jbDrawOptionsDefault.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				timeStartFilter = Long.parseLong(jtfTimeFilterStart.getText());
				jtfDrawOptionsDrawStart.setText(String.valueOf(timeStartFilter));
				drawTimeStart = (int) Long.parseLong(jtfDrawOptionsDrawStart.getText());
				jtfDrawOptionsDrawStart.setBackground(Color.WHITE);
				timeEndFilter = Long.parseLong(jtfTimeFilterEndFilter.getText());
				jtfDrawOptionsDrawEnd.setText(String.valueOf(timeEndFilter));
				drawTimeEnd = (int) Long.parseLong(jtfDrawOptionsDrawEnd.getText());
				jtfDrawOptionsDrawEnd.setBackground(Color.WHITE);
				step2TimesVerification();
			}
		});

		gGBConstraints.gridx = 2;
		gGBConstraints.gridy = 1;
		gGBConstraints.gridheight = 2;
		gGBConstraints.gridwidth = 1;
		gGBConstraints.weightx = 0;
		gGBConstraints.weighty = 0;
		gGBConstraints.fill = GridBagConstraints.HORIZONTAL;
		gestionnaire.setConstraints(jbDrawOptionsDefault, gGBConstraints);
		jpDrawOptions_.add(jbDrawOptionsDefault);

		// jpSpareBottom
		jpSpareBottom = new javax.swing.JPanel();
		gGBConstraints.gridx = 0;
		gGBConstraints.gridy = 3;
		gGBConstraints.gridheight = 1;
		gGBConstraints.gridwidth = 4;
		gGBConstraints.weightx = 1;
		gGBConstraints.weighty = 1;
		gGBConstraints.fill = GridBagConstraints.BOTH;
		gestionnaire.setConstraints(jpSpareBottom, gGBConstraints);
		jpDrawOptions_.add(jpSpareBottom);

		// jpSpareRight
		jpSpareRight = new javax.swing.JPanel();
		jpSpareRight.setSize(new Dimension(15, 5));
		gGBConstraints.gridx = 3;
		gGBConstraints.gridy = 0;
		gGBConstraints.gridheight = 3;
		gGBConstraints.gridwidth = 1;
		gGBConstraints.weightx = 1;
		gGBConstraints.weighty = 0;
		gGBConstraints.fill = GridBagConstraints.BOTH;
		gestionnaire.setConstraints(jpSpareRight, gGBConstraints);
		jpDrawOptions_.add(jpSpareRight);

		jtfDrawOptionsDrawStart.setText("0");
		jtfDrawOptionsDrawStart.addKeyListener(new java.awt.event.KeyListener()
		{
			public void keyPressed(KeyEvent evt)
			{
				// JTextField modification
				jtfDrawOptionsDrawStart.setBackground(Color.YELLOW);
				if (evt.getKeyCode() == KeyEvent.VK_ENTER)
				{
					// put focus on the parent
					jtfDrawOptionsDrawStart.setBackground(Color.WHITE);
					drawTimeStart = Integer.parseInt(jtfDrawOptionsDrawStart.getText());
					jtfDrawOptionsDrawStart.getParent().requestFocus();

					step2TimesVerification();
				}
			}

			public void keyReleased(KeyEvent e)
			{

			}

			public void keyTyped(KeyEvent e)
			{

			}
		});
		
		jtfDrawOptionsDrawStart.addFocusListener(new java.awt.event.FocusListener()
		{
			public void focusGained(FocusEvent e)
			{
			}

			public void focusLost(FocusEvent e)
			{
				jtfDrawOptionsDrawStart.setBackground(Color.WHITE);
				drawTimeStart = Integer.parseInt(jtfDrawOptionsDrawStart.getText());
				jtfDrawOptionsDrawStart.getParent().requestFocus();

				step2TimesVerification();
			} // focusLost
		});



		jtfDrawOptionsDrawEnd.setText(String.valueOf(maxTime));
		jtfDrawOptionsDrawEnd.addKeyListener(new java.awt.event.KeyListener()
		{
			public void keyPressed(KeyEvent evt)
			{
				// JTextField modification
				jtfDrawOptionsDrawEnd.setBackground(Color.YELLOW);
				if (evt.getKeyCode() == KeyEvent.VK_ENTER)
				{
					// put focus on the parent
					jtfDrawOptionsDrawEnd.setBackground(Color.WHITE);
					drawTimeEnd = (int) Long.parseLong(jtfDrawOptionsDrawEnd.getText());
					jtfDrawOptionsDrawEnd.getParent().requestFocus();

					step2TimesVerification();
				}
			}

			public void keyReleased(KeyEvent e)
			{

			}

			public void keyTyped(KeyEvent e)
			{

			}
		});
		
		jtfDrawOptionsDrawEnd.addFocusListener(new java.awt.event.FocusListener()
		{
			public void focusGained(FocusEvent e)
			{
			}

			public void focusLost(FocusEvent e)
			{
				jtfDrawOptionsDrawEnd.setBackground(Color.WHITE);
				drawTimeStart = Integer.parseInt(jtfDrawOptionsDrawEnd.getText());
				jtfDrawOptionsDrawEnd.getParent().requestFocus();

				step2TimesVerification();
			} // focusLost
		});



		// disable all components at start time
		jcbDrawOptionsDraw.setEnabled(false);
		jlDrawOptionsDrawStart.setForeground(Color.GRAY);
		jlDrawOptionsDrawEnd.setForeground(Color.GRAY);
		jtfDrawOptionsDrawStart.setEnabled(false);
		jtfDrawOptionsDrawEnd.setEnabled(false);

		// right side
		jpMainFilterRight.add(jpStatAnalysis);
		jpMainFilterRight.add(jpCleaning);
		jpMainFilterRight.add(jpDrawOptions_);

		// bottom buttons
		jpButtonsFilter.add(jbPreviousFilter);
		if (wizard2 == false)
		{
			jbPreviousFilter.setEnabled(false);
		}
		jpButtonsFilter.add(jbCancelFilter);
		jpButtonsFilter.add(jbNextFilter);
		FlowLayout jpButtonsFilterLayout = (FlowLayout) jpButtonsFilter
				.getLayout();
		jpButtonsFilterLayout.setAlignment((int) RIGHT_ALIGNMENT);
		jpMainFilterRight.add(jpButtonsFilter, BorderLayout.SOUTH);
		jpCenter.add(jpMainFilterRight, BorderLayout.CENTER);
		jpMainFilter.add(jpCenter, BorderLayout.CENTER);
		this.remove(jpMain);
		this.add(jpMainFilter);
		jpMainFilter.updateUI();

		this.testName = testName;
		this.bladeId = bladeId;
		if (wizard2 == false)
		{
			fillWizard2WithData();
		}
	}

	private void updateStatusField()
	{
		if (this.datasetType == DatasetType.SIMPLE_DATASET)
		{
			jlStatusFilter.setText("test name: " + this.testName + " - blade id: "
					+ this.bladeId + " - event: " + this.eventSelected);
		} else if (this.datasetType == DatasetType.MULTIPLE_DATASET)
		{
			String multipleBladeIds = "Multiple: " + testName;
			String pre = " (";
			Iterator<String> i = bladeIdList.iterator();
			while (i.hasNext())
			{
				String string = (String) i.next();
				multipleBladeIds += pre + string; 
				pre = ", ";
			}  
			multipleBladeIds += ")";
			jlStatusFilter.setText(multipleBladeIds);
		} else if (this.datasetType == DatasetType.AGGREGATE_DATASET)
		{
			String aggregateBladeIds = "Aggregate: " + testName + " {";
			Iterator<String> i = bladeIdList.iterator();
			while (i.hasNext())
			{
				String string = (String) i.next();
				aggregateBladeIds += "," + string;
			}
			aggregateBladeIds += "}";
			jlStatusFilter.setText(aggregateBladeIds);
		}
	}

	private void fillWizard2WithData()
	{
		// update the status
		jlStatusFilter.setFont(new java.awt.Font("Dialog", 0, 12));
		updateStatusField();
		// update time
		jtfTimeFilterStart.setText(String.valueOf(timeStartFilter));
		jtfTimeFilterEndFilter.setText(String.valueOf(timeEndFilter));

		// update filter table
		updateFilterTableModel(filterList);

		// update the logical op
		if (jTableFilter.getRowCount() > 1)
		{
			jpLogic.setVisible(true);
			if (this.logicalOp == LogicalEnum.AND_OP)
			{
				jrbAnd.setSelected(true);
			} else if (this.logicalOp == LogicalEnum.OR_OP)
			{
				jrbOr.setSelected(true);
			} else
			{
				if (DEBUG)
				{
					System.out.println("WARNING IN LOGICAL OP");
				}
			}
		}

		// update the checkbox : isFieldSelected, jcbField, K and
		// PercentPointKept
		if (isFieldSelected)
		{
			jcbPerformStAn.setSelected(true);
			jcbStAnField.setEnabled(true);
			jcbStAnField.setSelectedItem(fieldSelected);
			jlCleaningK.setForeground(Color.BLACK);
			jlCleaningPercentPointsKept1.setForeground(UNIMPLEMENTED_COLOR);
			jlCleaningPercentPointsKept2.setForeground(UNIMPLEMENTED_COLOR);
			jtfCleaningK.setForeground(UNIMPLEMENTED_COLOR);
			jtfCleaningK.setEnabled(true);
			jtfCleaningK.setEnabled(false); // unimplemented yet
			// enable draw options
			jcbDrawOptionsDraw.setEnabled(true);
			jlDrawOptionsDrawStart.setForeground(Color.BLACK);
			jlDrawOptionsDrawEnd.setForeground(Color.BLACK);
			jtfDrawOptionsDrawStart.setEnabled(true);
			jtfDrawOptionsDrawEnd.setEnabled(true);

			if (kParam == -1)
			{
				jtfCleaningK.setText("");
			} else
			{
				jtfCleaningK.setText(String.valueOf(kParam));
			}
			jtfCleaningPercentPointsKept.setEnabled(true);
			jtfCleaningPercentPointsKept.setEnabled(false); // unimplemented yet
			if (percentPointsKeptParam == -1)
			{
				jtfCleaningPercentPointsKept.setText("100");
			} else
			{
				jtfCleaningPercentPointsKept.setText(String
						.valueOf(percentPointsKeptParam));
			}
			if (isDrawSelected)
			{
				jcbDrawOptionsDraw.setSelected(true);
				jtfDrawOptionsDrawStart.setText(String
						.valueOf(this.drawTimeStart));
				jtfDrawOptionsDrawEnd.setText(String.valueOf(this.drawTimeEnd));
			} else
			{
				jcbDrawOptionsDraw.setSelected(false);
			}
		}
	}

	private void updateFilterTableModel(List<FieldFilterAndOp> filter)
	{
		// fill the table with the filter
		Iterator<FieldFilterAndOp> i = filter.iterator();
		FieldFilterAndOp fi = new FieldFilterAndOp();
		while (i.hasNext())
		{
			Vector<String> vectorTmp = new Vector<String>();
			fi = i.next();
			vectorTmp.add(fi.getField());
			vectorTmp.add(fi.getOperator());
			vectorTmp.add(fi.getFilter());
			vectorTmp.add("");
			modelTableFilter.addRow(vectorTmp);
		}

	}

	private void updateEvGroupCB(String[] eventType)
	{
		jpEvent.removeAll();
		jcbChooseEvent = new JComboBox();
		for (int i = 0; i < eventType.length; i++)
		{
			jcbChooseEvent.addItem(eventType[i]);
		}
		this.eventSelected = jcbChooseEvent.getItemAt(0).toString();

		jcbChooseEvent.setFont(new java.awt.Font("Dialog", 0, 12));
		jcbChooseEvent.addItemListener(new ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				JComboBox jcbTemp = (JComboBox) e.getSource();
				eventSelected = jcbTemp.getSelectedItem().toString();
				jbNext.setEnabled(true);
			}
		});
		jpEvent.add(jlSelectEvent);
		jpEvent.add(jcbChooseEvent);
		jpEvent.updateUI();
	}

	// Initialisation
	private void initFilterRendererAndEditor()
	{
		TableColumnModel myColumnModel = jTableFilter.getColumnModel();
		TableColumn myTableColumn;
		myTableColumn = myColumnModel.getColumn(FILTER_TABLE_FIELD);
		JComboBox fieldComboBox = new JComboBox();
		try
		{
			// get the first blade event
			if (this.bladeId != null)
			{
				BladeEvent bladeEventFirst = this.swingGUI
						.getBladeOneEventValues(this.testName, this.bladeId,
								this.eventSelected).get(0);
				String[] bladeEventArray = bladeEventFirst.getFieldLabels();
				for (int i = 0; i < bladeEventArray.length; i++)
				{
					fieldComboBox.addItem(bladeEventArray[i]);
				}
			}

		} catch (Exception e)
		{
			System.out.println("SimpleDataset.java / initFilterRendererAndEditor\n" 
					+ "Exception: " + e);
		}
		fieldComboBox.addItemListener(new java.awt.event.ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				if (e.getStateChange() == 1)
				{
// System.out.println("item state changed" + e.getItem().toString());
				}
			}
		});
		myTableColumn.setCellEditor(new DefaultCellEditor(fieldComboBox));

		myTableColumn = myColumnModel.getColumn(FILTER_TABLE_OPERATOR);
		JComboBox operatorComboBox = new JComboBox();

		for (int i = 0; i < operators.length; i++)
		{
			operatorComboBox.addItem(operators[i]);
		}
		operatorComboBox.addItemListener(new java.awt.event.ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				if (e.getStateChange() == 1)
				{
// System.out.println("item state changed" + e.getItem().toString());
				}
			}
		});
		myTableColumn.setCellEditor(new DefaultCellEditor(operatorComboBox));
		myTableColumn = myColumnModel.getColumn(FILTER_TABLE_DELETE);
		myTableColumn.setCellRenderer(new TableCellRenderer()
		{
			public Component getTableCellRendererComponent(JTable table,
					Object value, boolean isSelected, boolean hasFocus,
					int row, int column)
			{
				JLabel jlDelete = new JLabel();
				jlDelete.setIcon(DELETE_ICON);
				jlDelete.setOpaque(true);
				jlDelete.setHorizontalAlignment(JLabel.CENTER);
				if (isSelected)
				{
					Color bgrd = table.getSelectionBackground();
					jlDelete.setBackground(bgrd);
				} else
				{
					jlDelete.setBackground(Color.WHITE);
				}
				jlDelete.setBorder(null);
				return jlDelete;
			}
		});
		myTableColumn.setPreferredWidth(FILTER_TABLE_DELETE);
	}

	private void initFilterTableModel()
	{

		modelTableFilter = new DefaultTableModel(new Object[][] {},
				new String[] { "Field", "Operator", "Argument", "Delete" })
		{
			/**
			 *
			 */
			private static final long serialVersionUID = 6497096643249494298L;
			/**
			 *
			 */

			Class<?>[] types = new Class[] { java.lang.String.class,
					java.lang.String.class, java.lang.String.class,
					java.lang.String.class };
			boolean[] canEdit = new boolean[] { true, true, true, false };

			public Class<?> getColumnClass(int columnIndex)
			{
				return types[columnIndex];
			}

			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				return canEdit[columnIndex];
			}
		};
	}

	// LISTENERS

	protected void step2TimesVerification()
	{
		boolean enableFinish = true;
		timeStartFilter = Long.parseLong(jtfTimeFilterStart.getText());
		timeEndFilter = Long.parseLong(jtfTimeFilterEndFilter.getText());
		drawTimeStart = (int) Long.parseLong(jtfDrawOptionsDrawStart.getText());
		drawTimeEnd = (int) Long.parseLong(jtfDrawOptionsDrawEnd.getText());

		if (timeStartFilter < minTime)
		{
			jtfTimeFilterStart.setBackground(Color.RED);
			enableFinish = false;
		} else
		{
			jtfTimeFilterStart.setBackground(Color.WHITE);
		}

		if ((timeEndFilter > maxTime) || (timeEndFilter < timeStartFilter))
		{
			jtfTimeFilterEndFilter.setBackground(Color.RED);
			enableFinish = false;
		} else
		{
			jtfTimeFilterEndFilter.setBackground(Color.WHITE);
		}

		if ((drawTimeStart < timeStartFilter) || (drawTimeStart > timeEndFilter))
		{
			jtfDrawOptionsDrawStart.setBackground(Color.RED);
			enableFinish = false;
		} else
		{
			jtfDrawOptionsDrawStart.setBackground(Color.WHITE);
		}

		if ((drawTimeEnd < timeStartFilter) || (drawTimeEnd < drawTimeStart)
				|| (drawTimeEnd > timeEndFilter))
		{
			jtfDrawOptionsDrawEnd.setBackground(Color.RED);
			enableFinish = false;
		} else
		{
			jtfDrawOptionsDrawEnd.setBackground(Color.WHITE);
		}
		jbNextFilter.setEnabled(enableFinish)  ;
	}

	
	protected void jbCancelActionPerformed(ActionEvent evt)
	{
		this.dispose();
	}

	protected void jbNextActionPerformed(ActionEvent evt)
	{
		try
		{
			TreeSelectionModel curentSelModel = jTree1.getSelectionModel();
			String testName = ((TreePath) curentSelModel.getLeadSelectionPath())
					.getPathComponent(1).toString();
			String bladeName = ((TreePath) curentSelModel
					.getLeadSelectionPath()).getPathComponent(2).toString();

			vData.add("");
			vData.add(testName);
			vData.add(bladeName);
			if (this.datasetType == DatasetType.SIMPLE_DATASET
					|| this.datasetType == DatasetType.MULTIPLE_DATASET)
			{
				// current frame is the selection wizard
				this.remove(jpMainFilter);
				jpMainFilter.removeAll();
				initComponentFrame2(testName, bladeName);
				jpMainFilter.updateUI();
			}

			if (this.datasetType == DatasetType.AGGREGATE_DATASET)
			{
				if (checkSameBladeType())
				{
					// current frame is the selection wizard
					this.remove(jpMainFilter);
					jpMainFilter.removeAll();
					initComponentFrame2(testName, bladeName);
					jpMainFilter.updateUI();
				} else
				{
					JOptionPane.showMessageDialog(jbNextFilter,
							"You must select same class blades");
				}
			}
		} catch (Exception e)
		{
			System.out.println("SimpleDataset.java / jbNextActionPerformed\n" 
					+ "Exception: " + e);
			JOptionPane.showMessageDialog(jbNextFilter, "Select a blade first \nException: "
					+ e);
		}
	}

	private boolean checkSameBladeType()
	{
		Set<String> oneNameBladeFilter = new HashSet<String>();
		Iterator<NameBladeFilter> iterator = nameFilterList.iterator();
		while (iterator.hasNext())
		{
			NameBladeFilter nbf = (NameBladeFilter) iterator.next();
			BladeDescriptor[] bd = this.swingGUI.getBladeDescriptor(
					this.testName, nbf);
			oneNameBladeFilter.add(bd[0].getClassname());
		}
		if (oneNameBladeFilter.size() == 1)
		{
			return true;
		} else
		{
			return false;
		}
	}

	protected void jbPreviousActionPerformed(ActionEvent evt)
	{
		// quit the wizard
		this.dispose();
	}

	private void jTreeValueChanged(javax.swing.event.TreeSelectionEvent evt)
	{
		if (this.datasetType == DatasetType.SIMPLE_DATASET)
		{
			jSimpleTree(evt);
		} else if (this.datasetType == DatasetType.MULTIPLE_DATASET)
		{
			jMultipleAndAggTree(evt);
		} else if (this.datasetType == DatasetType.AGGREGATE_DATASET)
		{
			jMultipleAndAggTree(evt);
		}
		this.repaint();
	}

	private void jSimpleTree(TreeSelectionEvent evt)
	{
		TreeSelectionModel curentSelModel = jTree1.getSelectionModel();
		// modify the status bar
		statusTextField = "Simple :" + evt.getPath().toString();
		jlStatus.setText(statusTextField);
		// modify the JTable
		if (curentSelModel != null
				&& !curentSelModel.isSelectionEmpty()
				&& ((TreePath) curentSelModel.getLeadSelectionPath())
						.getPathCount() == 3)
		{
			String testName = ((TreePath) curentSelModel.getLeadSelectionPath())
					.getPathComponent(1).toString();
			NameBladeFilter filter = new NameBladeFilter(
					((TreePath) curentSelModel.getLeadSelectionPath())
							.getPathComponent(2).toString());
			BladeDescriptor[] bladeDescriptorArray = this.swingGUI
					.getBladeDescriptor(testName, filter);
			// get only one blade descriptor
			// put the values into a vector to add to the table
			Vector<String> addSel = new Vector<String>();
			this.testName = testName;
			String bladeId = bladeDescriptorArray[0].getId();
			this.bladeId = bladeId;
			String bladeClass = bladeDescriptorArray[0].getClassname();
			String bladeServerName = bladeDescriptorArray[0].getServerName();
			String bladeArgument = bladeDescriptorArray[0].getArgument();
			String bladeComment = bladeDescriptorArray[0].getComment();
			addSel.add(bladeId);
			// TO ADD THE TYPE
			addSel.add("");
			addSel.add(bladeClass);
			addSel.add(bladeServerName);
			addSel.add(bladeArgument);
			addSel.add(bladeComment);
			if (model.getRowCount() > 0)
			{
				model.removeRow(0);
			}
			// Create the first row
			model.insertRow(0, addSel);
			jTable1.getColumnModel().getColumn(1).setCellRenderer(
					new MyTableRenderer());

			String[] eventType = this.swingGUI.getEventTypes(testName, bladeId);
			// modify the buttonGroupList
			updateEvGroupCB(eventType);
			jbNext.setEnabled(true);
		} else
		{
			if (model.getRowCount() == 1)
			{
				model.removeRow(0);
				jlStatus.setText("");
				jpEvent.removeAll();
				eventSelected = null;
				jbNext.setEnabled(false);
			}
		}
	}

	private void jMultipleAndAggTree(TreeSelectionEvent evt)
	{
		int rowMax = model.getRowCount();
		for (int i = 0; i < rowMax; i++)
		{
			model.removeRow(0);
		}
		TreeSelectionModel curentSelModel = jTree1.getSelectionModel();
		TreePath[] treePathArray = curentSelModel.getSelectionPaths();
		if (curentSelModel != null && treePathArray != null)
		{
			boolean allLeaf = true;
			for (int i = 0; i < treePathArray.length; i++)
			{
				if (!(treePathArray[i].getPathCount() == 3))
				{
					allLeaf = false;
				}
			}
			if (!curentSelModel.isSelectionEmpty() && allLeaf)
			{
				List<String> stringList = new ArrayList<String>();
				Set<String> setEvent = new HashSet<String>();
				bladeIdList = new ArrayList<String>();
				nameFilterList = new ArrayList<NameBladeFilter>();
				for (int i = 0; i < treePathArray.length; i++)
				{
					// get the test name
					testName = treePathArray[i].getPathComponent(1).toString();
					// get the filer
					NameBladeFilter filter = new NameBladeFilter(
							treePathArray[i].getPathComponent(2).toString());
					nameFilterList.add(filter);
					// get the blade descriptor
					BladeDescriptor[] bladeDescriptorArray = this.swingGUI
							.getBladeDescriptor(testName, filter);
					// fill the table
					// get only one blade descriptor
					// put the values into a vector to add to the table
					Vector<String> addSel = new Vector<String>();
					String bladeId = bladeDescriptorArray[0].getId();
					bladeIdList.add(bladeId);
					String bladeClass = bladeDescriptorArray[0].getClassname();
					this.className = bladeClass;
					String bladeServerName = bladeDescriptorArray[0]
							.getServerName();
					String bladeArgument = bladeDescriptorArray[0]
							.getArgument();
					String bladeComment = bladeDescriptorArray[0].getComment();
					addSel.add(bladeId);
					// TO ADD THE TYPE
					addSel.add("");
					addSel.add(bladeClass);
					addSel.add(bladeServerName);
					addSel.add(bladeArgument);
					addSel.add(bladeComment);
					jTable1.getColumnModel().getColumn(1).setCellRenderer(
							new MyTableRenderer());
					// Create the first row
					model.insertRow(i, addSel);

					// INTERSECTION FOR ALL EVENTS

					// get all the eventType
					String[] eventType = this.swingGUI.getEventTypes(testName,
							bladeId);

					// eventTypeList contains all the events
					for (int j = 0; j < eventType.length; j++)
					{
						stringList.add(eventType[j]);
					}
					int allEventsArrayLength = stringList.size();
					// convert to array
					String[] allEventsArray = new String[allEventsArrayLength];
					Iterator<String> it = stringList.iterator();
					int cpt = 0;
					while (it.hasNext())
					{
						String string = (String) it.next();
						allEventsArray[cpt] = string;
						cpt++;
					}

					for (int j = 0; j < allEventsArray.length; j++)
					{
						String mark = allEventsArray[j];
						int cptMark = 0;
						for (int k = 0; k < allEventsArray.length; k++)
						{
							if (mark.equals(allEventsArray[k]))
							{
								cptMark++;
							}
						}
						// get all the events combined
						if (cptMark == treePathArray.length)
						{
							setEvent.add(allEventsArray[j]);
						}
					}
					// update status field
					if (this.datasetType == DatasetType.MULTIPLE_DATASET)
					{
						String tests = "";
						String blades = "";
						String statusString = "";
						String pre = "(";
						for (int iq = 0; iq < curentSelModel.getSelectionCount(); iq++)
						{
							tests += pre + ((TreePath) curentSelModel.getSelectionPaths()[iq]).getPathComponent(1).toString();
							blades += pre + ((TreePath) curentSelModel.getSelectionPaths()[iq]).getPathComponent(2).toString();
							statusString += pre 
								+ ((TreePath) curentSelModel.getSelectionPaths()[iq]).getPathComponent(1).toString()
								+ "/"
								+ ((TreePath) curentSelModel.getSelectionPaths()[iq]).getPathComponent(2).toString();
							pre = ", ";
						}
						tests += ")";
						blades += ")";
						statusString += ")";

						jlStatus.setText("Multiple :" + statusString);


					} else if (this.datasetType == DatasetType.AGGREGATE_DATASET)
					{
						String statusString = "";
						String pre = "(";
						for (int iq = 0; iq < curentSelModel.getSelectionCount(); iq++)
						{
							statusString += pre 
								+ ((TreePath) curentSelModel.getSelectionPaths()[iq]).getPathComponent(1).toString()
								+ "/"
								+ ((TreePath) curentSelModel.getSelectionPaths()[iq]).getPathComponent(2).toString();
							pre = ", ";
						}
						statusString += ")";

						jlStatus.setText("Aggregate :" + statusString);
					}
				}
				String[] setEventArray = new String[setEvent.size()];
				Iterator<String> setEventIt = setEvent.iterator();
				int m = 0;
				while (setEventIt.hasNext())
				{
					String string = (String) setEventIt.next();
					setEventArray[m] = string;
					m++;
				}
				// modify the buttonGroupList
				updateEvGroupCB(setEventArray);
				jbNext.setEnabled(true);
			}
		}
	}

	protected void jrbButtonActionPerformed(ActionEvent evt)
	{
		eventSelected = ((javax.swing.JRadioButton) evt.getSource()).getText();
		jbNext.setEnabled(true);
	}

	protected void jbPreviousFilterActionPerformed(ActionEvent evt)
	{
		this.remove(jpMainFilter);
		this.setTitle("Simple Dataset - Step 1: Selection");
		this.add(jpMain);
		vData.removeAllElements();
		jpMain.updateUI();
	}

	protected void jcbAnalysisActionPerformed(ActionEvent evt)
	{
		isFieldSelected = ((javax.swing.JCheckBox) evt.getSource())
				.isSelected();
		// update the common fields
		if (isFieldSelected)
		{
			isFieldSelected = true;
			jlCleaningK.setForeground(Color.BLACK);
			jlCleaningPercentPointsKept1.setForeground(UNIMPLEMENTED_COLOR);
			jlCleaningPercentPointsKept2.setForeground(UNIMPLEMENTED_COLOR);
			jtfCleaningK.setEnabled(true);
			jtfCleaningK.setEnabled(false); // unimplemented yet
			jtfCleaningPercentPointsKept.setEnabled(true);
			jtfCleaningPercentPointsKept.setEnabled(false); // unimplemented yet
			jcbStAnField.setEnabled(true);
			jcbStAnField.setSelectedItem(0);
			fieldSelected = jcbStAnField.getSelectedItem().toString();
			// enable draw options
			jcbDrawOptionsDraw.setEnabled(true);
			if (jcbDrawOptionsDraw.isSelected() == true)
			{
				jlDrawOptionsDrawStart.setForeground(Color.BLACK);
				jlDrawOptionsDrawEnd.setForeground(Color.BLACK);
				jtfDrawOptionsDrawStart.setEnabled(true);
				jtfDrawOptionsDrawEnd.setEnabled(true);
			} else
			{
				jlDrawOptionsDrawStart.setForeground(Color.GRAY);
				jlDrawOptionsDrawEnd.setForeground(Color.GRAY);
				jtfDrawOptionsDrawStart.setEnabled(false);
				jtfDrawOptionsDrawEnd.setEnabled(false);
			}
		} else
		{
			isFieldSelected = false;
			fieldSelected = null;
			jlCleaningK.setForeground(Color.GRAY);
			jlCleaningPercentPointsKept1.setForeground(UNIMPLEMENTED_COLOR);
			jlCleaningPercentPointsKept2.setForeground(UNIMPLEMENTED_COLOR);
			jtfCleaningK.setEnabled(false);
			jtfCleaningPercentPointsKept.setEnabled(false);
			jcbStAnField.setEnabled(false);
			fieldSelected = "";
			// enable draw options
			jcbDrawOptionsDraw.setEnabled(false);
			jcbDrawOptionsDraw.setSelected(false);
			jlDrawOptionsDrawStart.setForeground(Color.GRAY);
			jlDrawOptionsDrawEnd.setForeground(Color.GRAY);
			jtfDrawOptionsDrawStart.setEnabled(false);
			jtfDrawOptionsDrawEnd.setEnabled(false);
		}
	}

	private void jcbToDrawActionPerformed(ActionEvent evt)
	{
		if (jcbDrawOptionsDraw.isSelected() == true)
		{
			isDrawSelected = true;
			jlDrawOptionsDrawStart.setForeground(Color.BLACK);
			jlDrawOptionsDrawEnd.setForeground(Color.BLACK);
			jtfDrawOptionsDrawStart.setEnabled(true);
			jtfDrawOptionsDrawEnd.setEnabled(true);
		} else
		{
			isDrawSelected = false;
			jlDrawOptionsDrawStart.setForeground(Color.GRAY);
			jlDrawOptionsDrawEnd.setForeground(Color.GRAY);
			jtfDrawOptionsDrawStart.setEnabled(false);
			jtfDrawOptionsDrawEnd.setEnabled(false);
		}
	}

	protected void jbAddFilterActionPerformed(ActionEvent evt)
	{
		Vector<String> addSel = new Vector<String>();
		addSel.add("");
		addSel.add("");
		addSel.add("");
		addSel.add("");
		modelTableFilter.addRow(addSel);
		initFilterRendererAndEditor();
		if (modelTableFilter.getRowCount() > 1)
		{
			jpLogic.setVisible(true);
			if (wizard2 == false)
			{
				if (this.logicalOp == LogicalEnum.AND_OP)
				{
					jrbAnd.setSelected(true);
				} else if (this.logicalOp == LogicalEnum.OR_OP)
				{
					jrbOr.setSelected(true);
				} else
				{
					// select and by default
					jrbAnd.setSelected(true);
				}
			} else
			{
				this.logicalOp = LogicalEnum.AND_OP;
				jrbAnd.setSelected(true);
			}
		}

	}

	protected void jtfEndActionPerformed(ActionEvent evt)
	{

	}

	protected void jtfStartActionPerformed(ActionEvent evt)
	{
// System.out.println("TimeStart == "+jtfStartFilter.getText());
	}

	protected void jbFinishActionPerformed(ActionEvent evt)
	{
		if (checkFilterTable())
		{
			// logical op
			if (modelTableFilter.getRowCount() > 1)
			{
				if (jrbAnd.isSelected())
				{
					this.setLogicalOp(LogicalEnum.AND_OP);
					if (DEBUG)
					{
						System.out.println("AND SELECTED");
					}
				} else if (jrbOr.isSelected())
				{
					this.setLogicalOp(LogicalEnum.OR_OP);
					if (DEBUG)
					{
						System.out.println("OR SELECTED YOUII");
					}
				} else
				{
					if (DEBUG)
					{
						System.out.println("WARNING NO SELECTED LOGICAL OP");
					}
				}
			} else
			{
				logicalOp = LogicalEnum.NONE_OP;
			}

			// field and cleaning parameters
			if (isFieldSelected)
			{
				if (jtfCleaningK.getText().equalsIgnoreCase(""))
				{
					// DO NOTHING
					if (DEBUG)
					{
// System.out.println("NO VALUE FOR K");
						this.kParam = -1;
					}
				} else
				{
					try
					{
						kParam = Double.parseDouble(jtfCleaningK.getText());
					} catch (Exception e)
					{
						System.out.println("SimpleDataset.java / jbFinishActionPerformed\n" 
								+ "WARNING bad value for K\n"
								+ "Exception: " + e);
					}
				}
				if (jtfCleaningPercentPointsKept.getText().equalsIgnoreCase(""))
				{
					// DO NOTHING
					if (DEBUG)
					{
						// System.out.println("NO VALUE FOR PercentPointsKept");
						this.percentPointsKeptParam = -1;
					}
				} else
				{
					try
					{
						percentPointsKeptParam = Double
								.parseDouble(jtfCleaningPercentPointsKept
										.getText());
					} catch (Exception e)
					{
							System.out.println("SimpleDataset.java / jbFinishActionPerformed\n" 
									+ "WARNING bad value for PercentPointsKept\n"
									+ "Exception: " + e);
					}
				}
				if (jcbDrawOptionsDraw.isSelected())
				{
					isDrawSelected = true;
					if (jtfDrawOptionsDrawStart.getText().equalsIgnoreCase(""))
					{
						drawTimeStart = Integer.parseInt("0");
					} else
					{
						try
						{
							drawTimeStart = Integer.parseInt(jtfDrawOptionsDrawStart.getText());
						} catch (Exception e)
						{
							System.out.println("SimpleDataset.java / jbFinishActionPerformed\n" 
									+ "WARNING bad value for drawTimeStart\n"
									+ "Exception: " + e);
						}
					}
					if (jtfDrawOptionsDrawEnd.getText().equalsIgnoreCase(""))
					{
						drawTimeEnd = (int) maxTime;
					} else
					{
						try
						{
							drawTimeEnd = Integer.parseInt(jtfDrawOptionsDrawEnd.getText());
						} catch (Exception e)
						{
							System.out.println("SimpleDataset.java / jbFinishActionPerformed\n" 
									+ "WARNING bad value for drawTimeEnd.\n"
									+ "Exception: " + e);
						}
					}

				}
			}

			// time management
			if (jtfTimeFilterStart.getText().equalsIgnoreCase(""))
			{
				// DO NOTHING
			} else
			{
				try
				{
					timeStartFilter = Long.parseLong(jtfTimeFilterStart.getText());
				} catch (Exception e)
				{
					System.out.println("SimpleDataset.java / jbFinishActionPerformed\n" 
							+ "WARNING bad value for time timeStartFilter\n"
							+ "Exception: " + e);
				}
			}
			if (jtfTimeFilterEndFilter.getText().equalsIgnoreCase(""))
			{
				// DO NOTHING
			} else
			{
				try
				{
					timeEndFilter = Long.parseLong(jtfTimeFilterEndFilter
							.getText());
				} catch (Exception e)
				{
					System.out.println("SimpleDataset.java / jbFinishActionPerformed\n" 
							+ "WARNING bad value for time timeEndFilter\n"
							+ "Exception: " + e);
				}
			}

			// filter management Map<field< Map< keyword, operator>, all is save
			// in filter
			Map<String, String> keywordAndOp = new HashMap<String, String>();
			for (int i = 0; i < jTableFilter.getRowCount(); i++)
			{
				String key = modelTableFilter.getValueAt(i,
						FILTER_TABLE_ARGUMENT).toString();
				String op = modelTableFilter.getValueAt(i,
						FILTER_TABLE_OPERATOR).toString();
				String filterTable = modelTableFilter.getValueAt(i,
						FILTER_TABLE_FIELD).toString();
				keywordAndOp.put(key, op);
				filter.put(filterTable, keywordAndOp);
			}

			// first time in wizard
			if (this.wizard2 == true)
			{
				String resultM = "";
				String resultA = "";
				SimpleDataset.dataQuickViewContainer.addNewDataset(this);
				int currentId = SimpleDataset.dataQuickViewContainer
						.getCurrentIdDataset();
				int page = SimpleDataset.dataQuickViewContainer
						.updatePageExistence(currentId);
// SimpleDataset.dataQuickViewContainer.addNewPage(this, page);
				Vector<String> vData = new Vector<String>();
				// first col is not visible
				vData.add(String.valueOf(currentId));
				// test Name
				vData.add(this.testName);
				// blade Name
				if (this.datasetType == DatasetType.SIMPLE_DATASET)
				{
					vData.add(this.bladeId);
				} else if (this.datasetType == DatasetType.MULTIPLE_DATASET)
				{
					String multipleBladeIds = "(";
					Iterator<String> i = bladeIdList.iterator();
					while (i.hasNext())
					{
						String string = (String) i.next();
						multipleBladeIds += string + ",";
					}
					char[] mChar = multipleBladeIds.toCharArray();
					for (int j = 0; j < mChar.length - 1; j++)
					{
						resultM += mChar[j];
					}
					resultM += ")";
					this.resultM = resultM;
					this.bladeId = bladeIdList.get(0);
					vData.add(resultM);
				} else if (this.datasetType == DatasetType.AGGREGATE_DATASET)
				{
					String aggregateBladeIds = "{";
					Iterator<String> i = bladeIdList.iterator();
					while (i.hasNext())
					{
						String string = (String) i.next();
						aggregateBladeIds += string + ",";
					}
					char[] mChar = aggregateBladeIds.toCharArray();
					for (int j = 0; j < mChar.length - 1; j++)
					{
						resultA += mChar[j];
					}
					resultA += "}";
					this.resultA = resultA;
					this.bladeId = bladeIdList.get(0);
					vData.add(resultA);
					aggregateBladeIds += "}";
				}
				// event Name
				vData.add(this.eventSelected);
				// field
				vData.add(this.fieldSelected);
				// page
				vData.add(String.valueOf(page));
				// check if draw is selected
				if (isDrawSelected() == true)
				{
					vData.add("true");
				} else
				{
					vData.add("false");
				}
				SimpleDataset.dataQuickViewContainer.addRowToTable(vData);
				SimpleDataset.dataQuickViewContainer
						.udpatepageIdAndDatasetIds();
				SimpleDataset.dataQuickViewContainer.initCommentTree();
				if (this.datasetType.equals(DatasetType.SIMPLE_DATASET))
				{
					SimpleDataset.dataQuickViewContainer.addNewStatistic(this,
							currentId, this.fieldSelected, this.bladeId);
				} else if (this.datasetType
						.equals(DatasetType.MULTIPLE_DATASET))
				{
					this.bladeAlias = resultM;
					SimpleDataset.dataQuickViewContainer.addNewStatistic(this,
							currentId, this.fieldSelected, resultM);
				} else
				{
					this.bladeAlias = resultA;
					SimpleDataset.dataQuickViewContainer.addNewStatistic(this,
							currentId, this.fieldSelected, resultA);
				}
				if (isFieldSelected)
				{
					SimpleDataset.dataQuickViewContainer
							.setGraphOptionsVisible(true);
				}
				SimpleDataset.dataQuickViewContainer.setRowSelected(currentId);
				vData = null;
			}
			// double click on the wizard
			else
			{
				int selRow = SimpleDataset.dataQuickViewContainer
						.getSelectedRow();
				int datasetId = Integer
						.parseInt(SimpleDataset.dataQuickViewContainer
								.getValueAt(selRow, 0));
				if (isFieldSelected)
				{
					SimpleDataset.dataQuickViewContainer.setValueAt(
							this.fieldSelected, selRow, DATASET_COL_FIELD);
					if (isDrawSelected == true)
					{
						SimpleDataset.dataQuickViewContainer.setValueAt(String
								.valueOf(this.isDrawSelected), selRow,
								DATASET_COL_DRAW);
					} else
					{
						SimpleDataset.dataQuickViewContainer.setValueAt(
								"false", selRow, DATASET_COL_DRAW);
					}
				} else
				{
					SimpleDataset.dataQuickViewContainer.setValueAt("", selRow,
							DATASET_COL_FIELD);
					this.fieldSelected = "";
					this.kParam = -1;
				}
				SimpleDataset.dataQuickViewContainer.modifyDataset(this,
						datasetId);
				if (this.datasetType.equals(DatasetType.SIMPLE_DATASET))
				{
					SimpleDataset.dataQuickViewContainer.addNewStatistic(this,
							datasetId, this.fieldSelected, this.bladeId);
				} else if (this.datasetType
						.equals(DatasetType.MULTIPLE_DATASET))
				{
					SimpleDataset.dataQuickViewContainer.addNewStatistic(this,
							datasetId, this.fieldSelected, this.bladeAlias);
				} else
				{
					SimpleDataset.dataQuickViewContainer.addNewStatistic(this,
							datasetId, this.fieldSelected, this.bladeAlias);
				}
			}
			SimpleDataset.dataQuickViewContainer.updateDatasetOptions();
			// exit from the wizard
			this.dispose();
		} else
		{
			JOptionPane.showMessageDialog(this,
					"Malformed filter. Please correct it", "Warning",
					JOptionPane.WARNING_MESSAGE);
		}
	}

	/**
	 * Check is the table is filled
	 * 
	 * @return true is the filterTable is correct(all fields are filled
	 */
	public boolean checkFilterTable()
	{
		try
		{
			// check if all the cells are filled : OK
			int rowCount = modelTableFilter.getRowCount();
			int colCount = modelTableFilter.getColumnCount();
			for (int i = 0; i < rowCount; i++)
			{
				// don't take the last column which is delete column
				for (int j = 0; j < colCount - 1; j++)
				{
					if ((modelTableFilter.getValueAt(i, j).toString())
							.equals(""))
					{
						return false;
					}
				}
			}

			// ADD values in filter list
			filterList = new ArrayList<FieldFilterAndOp>();
			for (int i = 0; i < rowCount; i++)
			{
				String keyword = modelTableFilter.getValueAt(i,
						FILTER_TABLE_ARGUMENT).toString();
				String operator = modelTableFilter.getValueAt(i,
						FILTER_TABLE_OPERATOR).toString();
				String field = modelTableFilter.getValueAt(i,
						FILTER_TABLE_FIELD).toString();
				FieldFilterAndOp ffo = new FieldFilterAndOp(field, keyword,
						operator);
				filterList.add(ffo);
			}
			if (DEBUG)
			{
				displayTestFilter();
			}
			return true;
			// else NO GOOD
		} catch (Exception e)
		{
			System.out.println("checkFilterTable in SimpleDataset.java.\n" 
					+ "Exception: " + e);
			return false;
		}
	}

	private void displayTestFilter()
	{
		Iterator<FieldFilterAndOp> ibe = filterList.iterator();
		while (ibe.hasNext())
		{
			FieldFilterAndOp tmp = ibe.next();
			System.out.println(tmp.getField() + "," + tmp.getFilter() + ","
					+ tmp.getOperator());
		}
	}

	// Getter and setter

	private long getMaxTime(String testName, String bladeId)
	{
		long maxTime = this.swingGUI.getMaxTimeFor(testName, bladeId);
		return maxTime;
	}

	public String getTestName()
	{
		return this.testName;
	}

	public String getBladeId()
	{
		return this.bladeId;
	}

	public String getEventSelectedIdFromSD()
	{
		return eventSelected;
	}

	// get the filter
	public Map<String, Map<String, String>> getFilter()
	{
		return this.filter;
	}

	public LogicalEnum getLogicalOp()
	{
		return this.logicalOp;
	}

	public String getEventSelected()
	{
		return this.eventSelected;
	}

	public List<FieldFilterAndOp> getFilterList()
	{
		return this.filterList;
	}

	public long getTimeStartFilter()
	{
		return timeStartFilter;
	}

	public long getTimeEndFilter()
	{
		return timeEndFilter;
	}

	public boolean isFieldSelected()
	{
		return isFieldSelected;
	}

	public void setFieldSelected(boolean isFieldSelected)
	{
		this.isFieldSelected = isFieldSelected;
	}

	public double getKParam()
	{
		return kParam;
	}

	public void setKParam(double param)
	{
		kParam = param;
	}

	public double getPercentPointsKeptParam()
	{
		return percentPointsKeptParam;
	}

	public void setPercentPointsKeptParam(double percentPointsKeptParam)
	{
		this.percentPointsKeptParam = percentPointsKeptParam;
	}

	public String getFieldSelected()
	{
		return fieldSelected;
	}

	public long getMaxTime()
	{
		return this.maxTime;
	}

	public int getDatasetType()
	{
		if (this.datasetType == DatasetType.SIMPLE_DATASET)
		{
			return 0;
		} else if (this.datasetType == DatasetType.MULTIPLE_DATASET)
		{
			return 1;
		} else if (this.datasetType == DatasetType.AGGREGATE_DATASET)
		{
			return 2;
		} else
		{
			if (DEBUG)
			{
				System.out.println("WARNING BAD DATASET TYPE");
			}
			return -1;
		}
	}

	public void setLogicalOp(LogicalEnum logicalOp)
	{
		this.logicalOp = logicalOp;
	}

	public String getLogicalOpString()
	{
		if (logicalOp.equals(LogicalEnum.AND_OP))
		{
			return "and";
		} else if (logicalOp.equals(LogicalEnum.OR_OP))
		{
			return "or";
		} else
			return null;
	}

	public String getBladeIdAlias()
	{
		if (this.datasetType.equals(DatasetType.SIMPLE_DATASET))
		{
			return this.bladeId;
		} else if (this.datasetType.equals(DatasetType.MULTIPLE_DATASET))
		{
			return this.resultM;
		} else if (this.datasetType.equals(DatasetType.AGGREGATE_DATASET))
		{
			return this.resultA;
		} else
			return null;
	}

	public List<String> getBladeIdList()
	{
		return this.bladeIdList;
	}

	public String getBladeAlias()
	{
		return this.bladeAlias;
	}

	public boolean isDrawSelected()
	{
		return isDrawSelected;
	}

	public int getDrawTimeStart()
	{
		return drawTimeStart;
	}

	public int getDrawTimeEnd()
	{
		return drawTimeEnd;
	}

	// Variables declaration used for the frame 1
	private javax.swing.JLabel jlSelectEvent;
	private javax.swing.JScrollPane jScrollTable;
	private javax.swing.JScrollPane jScrollPaneTree;
	private javax.swing.JTable jTable1;
	private javax.swing.JLabel jlStatus;
	private javax.swing.JTree jTree1;
	private javax.swing.JButton jbCancel;
	private javax.swing.JButton jbNext;
	private javax.swing.JSplitPane jspFrame;
	private javax.swing.JPanel jpMain;
	private javax.swing.JPanel jpStatus;
	private javax.swing.JPanel jpMainRight;
	private javax.swing.JPanel jpEvent;
	private javax.swing.JPanel jpButtons;
	private javax.swing.JPanel jpTree;
	private javax.swing.JComboBox jcbChooseEvent;
	private javax.swing.JPanel jpEventInside;

	// Variables declaration used for the frame 2
	private javax.swing.JList jlChooseField;

	// frame 2 main panels
	private javax.swing.JPanel jpMainFilter; // jpCenter + jpStatusFilter

	private javax.swing.JPanel jpCenter; 	// jpMainFilteringLeft +
											// jpMainFilterRight
	private javax.swing.JPanel jpMainFilteringLeft; // frame 2 Filtering (Left)
	private javax.swing.JPanel jpTimeFilter;
	private javax.swing.JLabel jlTimeFilterInterval;
	private javax.swing.JPanel jpTimeFilterStart;
	private javax.swing.JLabel jlTimeFilterStart;
	private javax.swing.JTextField jtfTimeFilterStart;
	private javax.swing.JPanel jpTimeFilterDefault;
	private javax.swing.JButton jbTimeFilterDefaultEnd;
	private javax.swing.JPanel jpTimeFilterEnd;
	private javax.swing.JLabel jlTimeFilterEnd;
	private javax.swing.JTextField jtfTimeFilterEndFilter;
	private javax.swing.JScrollPane jScrollFilterTable; // frame 2 left
														// filtering table
	private javax.swing.JTable jTableFilter;
	private javax.swing.JPanel jpAddAndLogic;
	private javax.swing.JButton jbAddFilter;
	private javax.swing.JPanel jpLogic;
	private javax.swing.JLabel jlLinkingFilter;
	private javax.swing.JRadioButton jrbAnd;
	private javax.swing.JRadioButton jrbOr;

	private javax.swing.JPanel jpMainFilterRight; // frame 2 right side
	private javax.swing.JPanel jpStatAnalysis; // frame 2 Statistic Analysis
	private javax.swing.JPanel jpStAnChoose;
	private javax.swing.JCheckBox jcbPerformStAn;
	private javax.swing.JComboBox jcbStAnField; // + items
	private javax.swing.JPanel jpCleaning; // frame 2 Cleaning
	private javax.swing.JPanel jpKCleaningDescription;
	private javax.swing.JLabel jlCleaningKDescription;
	private javax.swing.JPanel jpCleaningK;
	private javax.swing.JLabel jlCleaningK;
	private javax.swing.JTextField jtfCleaningK;
	private javax.swing.JPanel jpCleaningPercentPointsKept;
	private javax.swing.JLabel jlCleaningPercentPointsKept1;
	private javax.swing.JTextField jtfCleaningPercentPointsKept;
	private javax.swing.JLabel jlCleaningPercentPointsKept2;
	private javax.swing.JPanel jpDrawOptions_; // frame 2 Draw Options
	private javax.swing.JCheckBox jcbDrawOptionsDraw;
	private javax.swing.JPanel jpDrawOptionsStart;
	private javax.swing.JLabel jlDrawOptionsDrawStart;
	private javax.swing.JTextField jtfDrawOptionsDrawStart;
	private javax.swing.JPanel jpDrawOptionsEnd;
	private javax.swing.JLabel jlDrawOptionsDrawEnd;
	private javax.swing.JTextField jtfDrawOptionsDrawEnd;
// private javax.swing.JPanel jpDrawOptionsDefault;
	private javax.swing.JButton jbDrawOptionsDefault;
	private javax.swing.JPanel jpSpareBottom;
	private javax.swing.JPanel jpSpareRight;
	private javax.swing.JPanel jpButtonsFilter; // frame 2 Buttons (Previous
												// Cancel Finish)
	private javax.swing.JButton jbPreviousFilter;
	private javax.swing.JButton jbCancelFilter;
	private javax.swing.JButton jbNextFilter;

	private javax.swing.JPanel jpStatusFilter; // frame 2 status line (bottom)
	private javax.swing.JLabel jlStatusFilter;

	// frame 2 not used
// private javax.swing.JButton jbDefaultStart;
// private javax.swing.ButtonGroup jbgAndOr;

	final Color MEDIUM_NAVY_BLUE_COLOR = new Color(0, 102, 204); // (Crayola
																	// Navy
																	// Blue)
	final Color UNIMPLEMENTED_COLOR = new Color(107, 108, 153); // BLUE-GRAY //
																// color for
																// unimplemented
																// functionalities

}
