package org.ow2.clif.analyze.lib.oda.ui.wizards;


import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.util.Properties;
import org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.datatools.connectivity.oda.OdaException;
import org.eclipse.datatools.connectivity.oda.design.DataSetDesign;
import org.eclipse.datatools.connectivity.oda.design.ui.designsession.DesignSessionUtil;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.ow2.clif.analyze.api.AnalyzerExternalAccess;
import org.ow2.clif.deploy.ClifAnalyzerAppFacade;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.lib.util.NameBladeFilter;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.analyze.lib.oda.CommonConstants;

/**
 * The Class ClifDataSetSelectionPage.
 */
public class ClifDataSetSelectionPage extends DataSetWizardPage 
{
	
	private static String DEFAULT_MESSAGE = "Select first a CLIF Blade and next a CLIF Event Type...";
	private static final int ERROR_FOLDER = 1;
	private static final int ERROR_EMPTY_PATH = 2;
	private boolean completePage = false;
	
	AnalyzerExternalAccess analyzer = null;
	ClifAnalyzerAppFacade clifAnalyzerApp = ClifAnalyzerAppFacade.getInstance();
		
	private String fileLocation;
	
	private List bladesList;
	private List eventTypesList;
	private Text labSelTable;
	
	private String testName;
	
	/**
	 * Constructor
	 */
	public ClifDataSetSelectionPage( )
	{
		this( "wizard.title.newDataSet" );
	}

	/**
	 * Constructor
	 * @param pageName
	 */
	public ClifDataSetSelectionPage( String pageName )
	{
		super( pageName );
		setTitle( pageName );
		setMessage( DEFAULT_MESSAGE );
		setPageComplete( false );
	}
	
	
	//////////////////////////////////////
	// abstract class DataSetWizardPage //
	//////////////////////////////////////
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage#createPageCustomControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPageCustomControl( Composite parent )
	{
		GridData gd;
		Composite container = (Composite)createPageControl( parent );
		GridLayout mainArea = new GridLayout();
		mainArea.numColumns = 3;		
		container.setLayout(mainArea);
		setControl(container);
		
		
		/*Blades AREA : Label + List*/
		Composite leftArea = new Composite(container, SWT.NULL);
		GridLayout layoutLeftArea = new GridLayout();
		leftArea.setLayout(layoutLeftArea);
		layoutLeftArea.numColumns = 1;
		
		Label labBladesTitle = new Label(leftArea, SWT.NULL);
		labBladesTitle.setText("1. Local available CLIF Blade(s) :");

		bladesList = new List(leftArea, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL | SWT.H_SCROLL);
		gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 150;
		gd.widthHint = 200;
		bladesList.setLayoutData(gd);
		bladesList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				selectBlade();
			}
		});
		
		/*Arrow*/
		Composite nextArea = new Composite(container, SWT.NULL);
		GridLayout layoutNextArea = new GridLayout();
		nextArea.setLayout(layoutNextArea);
		layoutNextArea.numColumns = 1;
		
		Label labNext = new Label(nextArea, SWT.NULL);
		labNext.setFont(new Font(Display.getDefault(), "Arial", 16, SWT.NORMAL));
		labNext.setText("->");
		
		/*Event Type AREA : Label + List*/
		Composite centerArea = new Composite(container, SWT.NULL);
		GridLayout layoutCenterArea = new GridLayout();
		centerArea.setLayout(layoutCenterArea);
		layoutCenterArea.numColumns = 1;
		
		Label labEventTypesTitle = new Label(centerArea, SWT.NULL);
		labEventTypesTitle.setText("2. Local available CLIF Event Type(s) :");

		eventTypesList = new List(centerArea, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL | SWT.H_SCROLL);
		gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 150;
		gd.widthHint = 200;
		eventTypesList.setLayoutData(gd);
		eventTypesList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				selectEventType();
			}
		});
		
		/*Selection LABEL*/
		Composite rightArea = new Composite(container, SWT.NULL);
		GridLayout layoutRightArea = new GridLayout();
		layoutRightArea.numColumns = 1;
		gd = new GridData();
		gd.horizontalSpan = 3;
		rightArea.setLayoutData(gd);		
		rightArea.setLayout(layoutRightArea);
				
		Label labSelExecutionTitle = new Label(rightArea, SWT.NULL);		
		labSelExecutionTitle.setText("3. Selected Blade.EventType :");		
		
		labSelTable = new Text(rightArea, SWT.SINGLE | SWT.READ_ONLY);
		labSelTable.setText("");
		gd = new GridData();
		gd.widthHint = 350;
		labSelTable.setEditable(false);
		labSelTable.setLayoutData(gd);				
		
		org.eclipse.datatools.connectivity.oda.design.Properties dataSourceProps = getInitializationDesign().getDataSourceDesign().getPublicProperties();
		testName = dataSourceProps.getProperty( CommonConstants.CONN_HOME_DIR_PROP );
		
		initializeControl();		
	}

	 /*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage#collectDataSetDesign(org.eclipse.datatools.connectivity.oda.design.DataSetDesign)
	 */
    protected DataSetDesign collectDataSetDesign( DataSetDesign design )
	{
		try
		{
			savePage( design );
		}
		catch ( OdaException e )
		{
		}
		return design;
	}
    
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage#canLeave()
	 */
    protected boolean canLeave( )
	{
		return isValid( );
	}
    
    
	///////////////////////////////
	// abstract class DialogPage //
	///////////////////////////////
  
    /*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.DialogPage#setVisible(boolean)
	 */
	public void setVisible( boolean visible )
	{
		super.setVisible( visible );
		getControl( ).setFocus( );
	}
	
	/**
	 * set messages
	 */
	public void setMessage( String newMessage )
	{
		super.setMessage( newMessage );
	}
	
	
	///////////////////////////////
	// abstract class WizardPage //
	///////////////////////////////
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage( )
	{
		int result = varifyFileLocation( this.fileLocation );
		boolean doNext = true;
		if ( result == ERROR_FOLDER )
			doNext = MessageDialog.openQuestion( getShell( ),
					DEFAULT_MESSAGE,
					"" );
		else if ( result == ERROR_EMPTY_PATH )
			doNext = MessageDialog.openQuestion( getShell( ),
					DEFAULT_MESSAGE,
					"" );
		if ( isValid( ) && doNext )
		{
			return super.getNextPage( );
		}
		else
			return null;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
	 */
	public boolean canFlipToNextPage( )
	{
		return false;
	}
	
	
	////////////////////
	// public methods //
	////////////////////
	
	/**
	 * 
	 * @param parent
	 * @return composite
	 */
	public Control createPageControl( Composite parent )
	{
		initializeDialogUnits( parent );
		
		Composite composite = new Composite( parent, SWT.NONE );
		GridLayout containerLayout = new GridLayout();
		containerLayout.numColumns = 3;
		containerLayout.verticalSpacing = 9;
		composite.setLayout(containerLayout);
		
		return composite;
	}
	
	
	/////////////////////
	// private methods //
	/////////////////////
	
	/**
	 * initial the info property after the control has been created.
	 *
	 */
	private void initializeControl( )
	{
		BladeDescriptor[] blades ;
		
		try{	
			Component analyzerComp = clifAnalyzerApp.getComponentByName("analyzer");
			analyzer = (AnalyzerExternalAccess) analyzerComp.getFcInterface(AnalyzerExternalAccess.ANALYZER_EXTERNAL_ACCESS);
			blades = analyzer.getTestPlan(testName, null);
			
			for(int i = 0;i<blades.length;i++){
				bladesList.add(blades[i].getId());
			}
		}
		catch (NoSuchInterfaceException ex)
		{
			System.err.println(ex);
		}
		catch(ClifException ce){
			System.err.println(ce);
		}

		
		// Try to get Propety
		DataSetDesign dataSetDesign = getInitializationDesign();

		if ( dataSetDesign == null )
			return; // nothing to initialize
				
		if ( dataSetDesign.getPublicProperties( ) == null )
		{	
			try
			{
				dataSetDesign.setPublicProperties( DesignSessionUtil.createDataSetPublicProperties( dataSetDesign.getOdaExtensionDataSourceId( ),
						dataSetDesign.getOdaExtensionDataSetId( ),
						getPageProperties( ) ) );
			}
			catch ( OdaException e )
			{
				e.printStackTrace( );
			}
		}
		
		String selBladeFile = dataSetDesign.getPublicProperties()
					.getProperty( CommonConstants.CONST_PROP_BLADE_FILE );
		
		// Initialize selected Blade.EventType with property
		labSelTable.setText(selBladeFile);
		
		
		setPageComplete( true );
		if ( !isSessionEditable( ) )
			getControl( ).setEnabled( false );
	}
		
	/**
	 * set page status based on row number
	 *
	 */
	private void setPageStatus( )
	{


	}	
	
	/**
     * verify the file location.
     * 
     * @param fileLocation the file location
     * 
     * @return the int
     */
	private int varifyFileLocation( String fileLocation )
	{
		int result = 0;
		
		if ( fileLocation != null && fileLocation.trim( ).length( ) > 0 )
		{
			try
			{
				new URL( fileLocation );
			}
			catch ( MalformedURLException e )
			{
				File file = new File( fileLocation );
				if ( file.exists( ) )
				{
					setMessage( DEFAULT_MESSAGE );
				}
				else
				{
					setDetailsMessage( "error.selectFolder",
							IMessageProvider.ERROR );
					result = ERROR_FOLDER;
				}
			}
		}
		return result;
	}
	   
    /**
     * Updates the given dataSetDesign with the query and its metadata defined
     * in this page.
     * 
     * @param dataSetDesign
     * @throws OdaException 
     */
    private void savePage( DataSetDesign dataSetDesign ) throws OdaException
	{
		if ( dataSetDesign.getPublicProperties( ) == null )
		{	
			try
			{
				dataSetDesign.setPublicProperties( DesignSessionUtil.createDataSetPublicProperties( dataSetDesign.getOdaExtensionDataSourceId( ),
						dataSetDesign.getOdaExtensionDataSetId( ),
						getPageProperties( ) ) );
			}
			catch ( OdaException e )
			{
				e.printStackTrace( );
			}
		}

		if ( dataSetDesign.getPublicProperties( ) != null )
		{
			if ( dataSetDesign.getPublicProperties( )
					.findProperty( CommonConstants.CONST_PROP_BLADE_FILE ) != null && 
					labSelTable != null)
			{
				dataSetDesign.getPublicProperties( )
						.findProperty( CommonConstants.CONST_PROP_BLADE_FILE )
						.setNameValue( CommonConstants.CONST_PROP_BLADE_FILE, labSelTable.getText() );
				dataSetDesign.setQueryText("SELECT * FROM " + dataSetDesign.getPublicProperties().getProperty(CommonConstants.CONST_PROP_BLADE_FILE));
			}
		}
	}
	
	/**
     * if the numberText has been selected, check the input number is valid or
     * not.
     * 
     * @return true, if is valid
     */
	private boolean isValid( )
	{
		return true;
	}
	
	/**
     * get page property.
     * 
     * @return the page properties
     */
	private static Properties getPageProperties( )
	{
		Properties	prop = new Properties( );
		prop.setProperty( CommonConstants.CONST_PROP_BLADE_FILE, "" );
		return prop;
	}
		
    /**
	 * set detailed message
	 * @param message
	 * @param type
	 */
	private void setDetailsMessage( String message, int type )
	{
		this.setMessage( message, type );
	}
	
	private void selectBlade(){
		
		NameBladeFilter filter;
		
		if(bladesList.getSelectionIndex() != -1)
		{ // There is a selected element
			if(bladesList != null && bladesList.getItem(bladesList.getSelectionIndex()).trim().compareTo("") != 0)
			{ // Item is not empty
				try{
					String bladeId = this.bladesList.getItem(this.bladesList.getSelectionIndex());
					filter = new NameBladeFilter(bladeId);
					BladeDescriptor desc =analyzer.getTestPlan(testName, filter)[0];
					String[] items = desc.getEventTypeLabels();
					this.eventTypesList.setItems(items);
					//labSelTable.setText(bladesList.getItem(bladesList.getSelectionIndex()));	
				}catch(ClifException ce){
					System.err.println("ClifDataSetSelectionPage.selectBlade() error : "+ce);
				}
				this.completePage = false;
				setPageComplete( false );
			}
		} else {
			this.completePage = false;
			setPageComplete( false );
		}
		
		if(getContainer() != null)
		{
			getContainer().updateButtons();
		}
	}
	
	
	private void selectEventType(){
		if(eventTypesList.getSelectionIndex() != -1)
		{ // There is a selected element
			if(eventTypesList != null && eventTypesList.getItem(eventTypesList.getSelectionIndex()).trim().compareTo("") != 0)
			{ // Item is not empty
				labSelTable.setText(bladesList.getItem(bladesList.getSelectionIndex()) + "." + eventTypesList.getItem(eventTypesList.getSelectionIndex()));
				this.completePage = true;	
				setPageComplete( true );
			}
		} else {
			this.completePage = false;
			setPageComplete( false );
		}
		
		if(getContainer() != null)
		{
			getContainer().updateButtons();
		}
	}
    
}
