/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Gr�gory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.oda.ui.wizards;

import java.util.Properties;
import org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSourceWizardPage;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.jface.wizard.WizardPage;

/**
 * Provide a wizard page to select local CLIF executions.
 * @author Gr�gory CALONNIER
 * @author Jordan BRUNIER
 */
public class ExecutionSelectionWizardPage extends DataSourceWizardPage
{
	
	private ExecutionSelectionPageHelper m_pageHelper;
	private Properties m_folderProperties;

	
	/**
     * Instantiates a new execution selection wizard page.
     * 
     * @param pageName the page name
     */
	public ExecutionSelectionWizardPage( String pageName )
	{
		super( pageName );		
		setMessage( ExecutionSelectionPageHelper.DEFAULT_MESSAGE );
	}

	
	/////////////////////////////////////////
	// abstract class DataSourceWizardPage //
	/////////////////////////////////////////
	
	/* (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSourceWizardPage#createPageCustomControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPageCustomControl( Composite parent )
	{		
		if ( m_pageHelper == null )
			m_pageHelper = new ExecutionSelectionPageHelper( (WizardPage) this );
		m_pageHelper.createCustomControl( parent );
		m_pageHelper.initCustomControl( m_folderProperties ); // in case init was called before create 
		
		// Optionally hides the Test Connection button, using
		setPingButtonVisible( false );
	}

	/* (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSourceWizardPage#initPageCustomControl(java.util.Properties)
	 */
	public void setInitialProperties( Properties dataSourceProps )
	{
		m_folderProperties = dataSourceProps;		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSourceWizardPage#collectCustomProperties()
	 */
	public Properties collectCustomProperties( )
	{
		/* 
		 * Optionally assign a custom designer state, for inclusion
		 * in the ODA design session response, using
		 * setResponseDesignerState( DesignerState customState ); 
		 */
		return m_pageHelper.collectCustomProperties(null);
	}

    /*
     * (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSourceWizardPage#refresh()
     */
    public void refresh()
    {
    	if(getContainer() != null)
    	{
    		getContainer().updateButtons();
    	}
    }
    
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.DialogPage#setVisible(boolean)
	 */
	public void setVisible( boolean visible )
	{
		super.setVisible( visible );
		getControl( ).setFocus( );
	}
		
	/**
	 * Check if page is complete
	 * @return true if page is complete else false
	 */
	public boolean isPageComplete() {		
		return(m_pageHelper.isPageComplete());		
	}
		
	/**
	 * Get properties. 
	 * @return properties
	 */
	public Properties getProperties()
	{
		return m_folderProperties;
	}
}
