/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2009, 2010 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */

package org.ow2.clif.console.lib.egui.wizards.clifProject;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Properties;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.ow2.clif.console.lib.egui.ClifConsolePlugin;
import org.ow2.clif.util.ExecutionContext;
import org.ow2.clif.util.Network;

/**
 * Class used by wizard and properties pages. Since there are many common
 * features between wizard and properties pages, this class grouped them.
 * If offers the possibility of creating and initializing controls, checking
 * whether the pages are filled correctly, returning filled fields, 
 * writing CLIF properties files and applying changes.
 * 
 * @author Florian Francheteau
 * @author Bruno Dillenseger
 */
public class InteractionManager {

	private Shell parentShell;
	private IProject project;
	private WizardPage wizardPage;
	private PreferencePage prefPage;

	private IPath containerPath;
	private static boolean initialized=false;
	private static boolean applied=false;

	private static Text reportText;
	private Button browseReport;
	private static Text statsText;
	private Button browseStats;
	private static String initReport="report";
	private static String initStats="stats";

	private static CCombo registryHostCombo;
	private static Text registryPortText;
	private static CCombo codeServerHostCombo;
	private static Text codeServerPortText;
	private static List classes;
	private static String initRegistryHost="localhost";
	private static String initRegistryPort="1234";
	private static String initCodeServerHost="";
	private static String initCodeServerPort="1357";
	private static String initClasses="";

	private static Text fileStorageDelayText;
	private static Text fileStorageMaxPendingText;
	private static Text jonathanHostText;
	private static Text fileStorageHostText;
	private static String initFileStorageDelay="";
	private static String initFileStorageMaxPending="";
	private static String initJonathanHost="";
	private static String initFileStorageHost="";

	private static Text xmsText;
	private static Text xmxText;
	private static String initXms="";
	private static String initXmx="";

	private static Text isacThreadText;
	private static Text groupPeriodText;
	private static Text schedulerPeriodText;
	private static Text jobDelayText;
	private static String initIsacThread="";
	private static String initGroupPeriod="";
	private static String initSchedulerPeriod="";
	private static String initJobDelay="";

	private static List customProps;
	private static java.util.List<String> initCustomProps = new ArrayList<String>();

	/**
	 * Constructor for a wizard page
	 * @param page the wizard page
	 * @param project the project created
	 */
	public InteractionManager(WizardPage page, IProject project) {

		this.wizardPage = page;
		this.parentShell = page.getShell();
		this.project = project;
		applied=false;
	}

	/**
	 * Constructor for a properties page
	 * @param page the properties page
	 * @param project the project where user wants to check or change properties
	 */
	public InteractionManager(PreferencePage page, IProject project) {

		this.prefPage = page;
		this.parentShell = page.getShell();
		this.project = project;
		applied=false;
	}

	//------------------------------------------------------------MAIN CONFIGURATION

	/**
	 * Creates contents of main page
	 */
	public Composite createMainContents(Composite parent){
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		container.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		layout.numColumns = 4;
		layout.verticalSpacing = 9;
		layout.horizontalSpacing = 10;

		//Report Label
		Label label = new Label(container, SWT.NULL);
		label.setText("&Report Directory*:");

		//Report Text
		reportText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		reportText.setLayoutData(gd);
		reportText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mainDialogChanged();
			}
		});

		//Browse Button
		browseReport = new Button(container, SWT.PUSH);
		browseReport.setText("Browse...");
		browseReport.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleReportBrowse();
			}
		});

		//Stats Label
		label = new Label(container, SWT.NULL);
		label.setText("Stats &Directory*:");

		//Stats Text
		statsText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		statsText.setLayoutData(gd);
		statsText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mainDialogChanged();
			}
		});

		//Browse Button
		browseStats = new Button(container, SWT.PUSH);
		browseStats.setText("Browse...");
		browseStats.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleStatsBrowse();
			}
		});
		return container;
	}

	/**
	 * Initializes contents of main page
	 */
	public void initializeMainContents(){
		Path initReportPath=new Path(initReport);
		String report = initReportPath.toOSString();
		if (report.startsWith("//")){
			report = report.substring(1);
		}
		reportText.setText(report);
		Path initStatsPath=new Path(initStats);
		String stats = initStatsPath.toOSString();
		if (stats.startsWith("//")){
			stats = stats.substring(1);
		}
		statsText.setText(stats);
	}

	/**
	 * Uses the standard container selection dialog to choose 
	 * the new value for the container field. 
	 */
	private void handleReportBrowse() {
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(
				parentShell, project.getWorkspace().getRoot(), false,
		"Select new report directory container");
		if (dialog.open() == ContainerSelectionDialog.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				containerPath=((Path) result[0]);
				reportText.setText(containerPath.toOSString().substring(1));
			}
		}
	}

	/**
	 * Uses the standard container selection dialog to choose 
	 * the new value for the container field. 
	 */
	private void handleStatsBrowse() {
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(
				parentShell, project.getWorkspace().getRoot(), false,
		"Select new stats directory container");
		if (dialog.open() == ContainerSelectionDialog.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				containerPath=((Path) result[0]);
				statsText.setText(containerPath.toOSString().substring(1));
			}
		}
	}

	/**
	 * Ensures that both main text fields are correctly filled. 
	 */
	private void mainDialogChanged() {
		//Ensures that report directory name is correct
		if (getReportDir().length() == 0) {
			updateStatus("Report directory must be specified");
			return;
		}else if (getReportDir().startsWith("//")) {
			updateStatus("Report directory must be relative");
			return;
		}
		//Ensures that statistics directory name is correct
		if (getStatsDir().length() == 0) {
			updateStatus("Stats directory must be specified");
			return;
		}else if (getStatsDir().startsWith("//")) {
			updateStatus("Stats directory must be relative");
			return;
		}
		updateStatus(null);
	}
	
	/**
	 * Initialize values for report and stats directories 
	 * @param projectPath path of the project containing those values
	 */
	public void initializeStatsReportText(IPath projectPath){
		initReport="report";
		initStats="stats";
		String report = projectPath.append(initReport).toOSString().substring(1);
		String stats = projectPath.append(initStats).toOSString().substring(1);
		reportText.setText(report);
		statsText.setText(stats);
	}

	public String getReportDir(){
		if (reportText!=null){
			if (!reportText.isDisposed()){
				return reportText.getText();
			}
		}
		return initReport;
	}
	
	public String getStatsDir(){
		if (statsText!=null){
			if (!statsText.isDisposed()){
				return statsText.getText();		
			}
		}
		return initStats;
	}

	//-----------------------------------------------------------CUSTOM CONFIGURATION

	/**
	 * Creates contents of custom properties page
	 * @return the custom properties page Composite
	 */
	public Composite createCustomContents(Composite parent)
	{
		Composite all = new Composite(parent, SWT.NULL);
		all.setLayout(new GridLayout());
		all.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		ImageRegistry imageRegistry = new ImageRegistry();
		imageRegistry.put("addproperty", ClifConsolePlugin.imageDescriptorFromPlugin(
			ClifConsolePlugin.PLUGIN_ID,"icons/addproject.png"));
		imageRegistry.put("edit", ClifConsolePlugin.imageDescriptorFromPlugin(
			ClifConsolePlugin.PLUGIN_ID,"icons/rename.png"));
		imageRegistry.put("delete", ClifConsolePlugin.imageDescriptorFromPlugin(
			ClifConsolePlugin.PLUGIN_ID,"icons/delete.gif"));

		Label label = new Label(all, SWT.NONE);
		label.setText("Custom system properties");

		/* Composite with 2 columns : Composite with buttons + List */
		Composite field = new Composite(all, SWT.NULL);
		field.setLayout(new GridLayout(2, false));

		/* Composite with buttons */
		Composite buttons = new Composite(field, SWT.NULL);
		FillLayout bLayout = new FillLayout(SWT.VERTICAL);
		buttons.setLayout(bLayout);

		// button for adding a property
		Button addProp = new Button(buttons, SWT.PUSH);
		addProp.setImage(imageRegistry.get("addproperty"));
		addProp.setToolTipText("Add system property");
		addProp.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event)
			{
				InputDialog dialog = new InputDialog(
					parentShell,
					"Custom Java system property",
					"property=value",
					"=",
					new IInputValidator() {
						public String isValid(String prop)
						{
							return prop.contains("=") ? null : "Please use this format: property=value";
						}
					});
				dialog.open();
				if (dialog.getReturnCode() == InputDialog.OK)
				{
					customProps.add(dialog.getValue());
				}
			}
		});

		// button for editing a property
		Button editProp = new Button(buttons, SWT.PUSH);
		editProp.setImage(imageRegistry.get("edit"));
		editProp.setToolTipText("Change property");
		editProp.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event)
			{
				if (customProps.getSelectionCount() > 0)
				{
					int index = customProps.getSelectionIndex();
					InputDialog dialog = new InputDialog(
						parentShell,
						"Modify a custom system property",
						"property=value",
						customProps.getItem(index),
						new IInputValidator() {
							public String isValid(String prop)
							{
								return prop.contains("=") ? null : "Please use this format: property=value";
							}
						});
					dialog.open();
					if (dialog.getReturnCode() == InputDialog.OK)
					{
						customProps.setItem(index, dialog.getValue());
					}
				}
			}
		});

		// button for removing a property
		Button remove = new Button(buttons, SWT.PUSH);
		remove.setImage(imageRegistry.get("delete"));
		remove.setToolTipText("Remove system properties");
		remove.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event)
			{
				customProps.remove(customProps.getSelectionIndices());
			}
		});

		// button for moving a property up in the list
		Button up = new Button(buttons, SWT.ARROW | SWT.UP);
		up.setToolTipText("Move property up");
		up.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event)
			{
				if (customProps.getSelectionCount() > 0)
				{
					String name = customProps.getSelection()[0];
					int index = customProps.getSelectionIndex();

					if (index > 0) {
						customProps.remove(index);
						customProps.add(name, index - 1);
						customProps.setSelection(index - 1);
					}
				}
			}
		});

		// button for moving a property down in the list
		Button down = new Button(buttons, SWT.ARROW | SWT.DOWN);
		down.setToolTipText("Move property down");
		down.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event event)
			{
				if (customProps.getSelectionCount() > 0)
				{
					String name = customProps.getSelection()[0];
					int index = customProps.getSelectionIndex();
					if (index < customProps.getItemCount() - 1)
					{
						customProps.remove(index);
						customProps.add(name, index + 1);
						customProps.setSelection(index + 1);
					}
				}
			}
		});

		// list of properties
		customProps = new List(field, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL | SWT.H_SCROLL |SWT.FILL);
		GridData gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		customProps.setLayoutData(gd);
		customProps.addMouseListener(new MouseAdapter() {
			public void mouseDoubleClick(MouseEvent event)
			{
				if (customProps.getSelectionCount() > 0)
				{
					int index = customProps.getSelectionIndex();
					InputDialog dialog = new InputDialog(
						parentShell,
						"Modify a custom system property",
						"property=value",
						customProps.getItem(index),
						new IInputValidator() {
							public String isValid(String prop)
							{
								return prop.contains("=") ? null : "Please use this format: property=value";
							}
						});
					dialog.open();
					if (dialog.getReturnCode() == InputDialog.OK)
					{
						customProps.setItem(index, dialog.getValue());
					}
				}
			}
		});
		return(all);
	}

	/**
	 * Initializes contents of custom properties page
	 */
	public void initializeCustom()
	{
		for (String prop : initCustomProps)
		{
			customProps.add(prop);
		}
	}

	/**
	 * @return list of system properties in the form prop=value
	 */
	public String[] getCustomProperties()
	{
		if (customProps != null && ! customProps.isDisposed())
		{
			return customProps.getItems();
		}
		else
		{
			return new String[0];
		}
	}

	//----------------------------------------------------------ADVANCED CONFIGURATION

	/**
	 * Creates contents of advanced page
	 */
	public Composite createAdvancedContents(Composite parent){
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		container.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		layout.numColumns = 2;
		layout.verticalSpacing = 9;
		layout.horizontalSpacing = 10;

		//Data Collector delay Label
		Label label = new Label(container, SWT.NULL);
		label.setText("File Storage &Delay (s):");

		//Data Collector delay Text
		fileStorageDelayText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		fileStorageDelayText.setLayoutData(gd);
		fileStorageDelayText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				advancedDialogChanged();
			}
		});

		//Data Collector max pending Label
		label = new Label(container, SWT.NULL);
		label.setText("File Storage &Maximum pending events:");

		//Data Collector max pending Text
		fileStorageMaxPendingText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		fileStorageMaxPendingText.setLayoutData(gd);
		fileStorageMaxPendingText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				advancedDialogChanged();
			}
		});

		//File Storage Host Label
		label = new Label(container, SWT.NULL);
		label.setText("&File Storage Host:");

		//File Storage Host Text
		fileStorageHostText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		fileStorageHostText.setLayoutData(gd);
		fileStorageHostText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				advancedDialogChanged();
			}
		});

		//Jonathan Connection Factory Host Label
		label = new Label(container, SWT.NULL);
		label.setText("&Jonathan Connection Factory Host:");

		//Jonathan Connection Factory Host Text
		jonathanHostText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		jonathanHostText.setLayoutData(gd);
		jonathanHostText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				advancedDialogChanged();
			}
		});

		return (container);
	}

	/**
	 * Initializes contents of advanced page
	 */
	public void initializeAdvanced() {
		fileStorageDelayText.setText(initFileStorageDelay);
		fileStorageMaxPendingText.setText(initFileStorageMaxPending);
		jonathanHostText.setText(initJonathanHost);
		fileStorageHostText.setText(initFileStorageHost);
	}

	/**
	 * Ensures that both advanced text fields are correctly filled. 
	 */
	private void advancedDialogChanged() {
		String str = getFileStorageDelay();
		//Ensures that data collector delay host is correct 
		if (str.length() > 0){
			if (!verifyNumber(str, "File Storage write delay")){
				return;
			}
		}
		str = getFileStorageMaxPending();
		if (str.length() > 0){
			if (!verifyNumber(str, "File Storage maximum number of pending events")){
				return;
			}
		}
		updateStatus(null);
	}

	public String getFileStorageDelay(){
		if (fileStorageDelayText!=null){
			if (!fileStorageDelayText.isDisposed()){
				return fileStorageDelayText.getText();
			}
		}
		return initFileStorageDelay;
	}

	public String getFileStorageMaxPending(){
		if (fileStorageMaxPendingText!=null){
			if (!fileStorageMaxPendingText.isDisposed()){
				return fileStorageMaxPendingText.getText();
			}
		}
		return initFileStorageMaxPending;
	}

	public String getJonathanHost(){
		if (jonathanHostText!=null){
			if (!jonathanHostText.isDisposed()){
				return jonathanHostText.getText();
			}
		}
		return initJonathanHost;
	}

	public String getFileStorageHost(){
		if (fileStorageHostText!=null){
			if (!fileStorageHostText.isDisposed()){
				return fileStorageHostText.getText();
			}
		}
		return initFileStorageHost;
	}

	//----------------------------------------------------------JVM CONFIGURATION

	/**
	 * Creates contents of JVM page
	 */
	public Composite createJVMContents(Composite parent){
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		container.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		layout.numColumns = 2;
		layout.verticalSpacing = 9;
		layout.horizontalSpacing = 10;

		//Xms Label
		Label label = new Label(container, SWT.NULL);
		label.setText("Xm&s:");

		//Xms Text
		xmsText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.BEGINNING);
		gd.widthHint=50;
		xmsText.setLayoutData(gd);
		xmsText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				jvmDialogChanged();
			}
		});

		//Xmx Label
		label = new Label(container, SWT.NULL);
		label.setText("Xm&x:");

		//Xmx Text
		xmxText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.BEGINNING);
		gd.widthHint=50;
		xmxText.setLayoutData(gd);
		xmxText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				jvmDialogChanged();
			}
		});
		return(container);
	}

	/**
	 * Initializes contents of JVM page
	 */
	public void initializeJVMParam() {
		xmsText.setText(initXms);
		xmxText.setText(initXmx);
	}

	/**
	 * Ensures that both JVM text fields are correctly filled. 
	 */
	private void jvmDialogChanged() {
		String xms = getXms();
		String xmx = getXmx();
		//Ensures that Xms is correct 
		if (xms.length() > 0){
			if (xms.length()<2){
				updateStatus("Xms value must be formated like xm or xk or xg where x is a number");
				return;
			}
			try {
				if ((!xms.endsWith("m"))&&(!xms.endsWith("k"))&&(!xms.endsWith("g"))){
					updateStatus("Xms value must be formated like xm or xk or xg where x is a number");
					return;
				}
				Integer.parseInt(xms.substring(0,xms.length()-1));
			}catch(NumberFormatException e){
				updateStatus("Xms value must be formated like xm or xk or xg where x is a number");
				return;
			}
		}
		//Ensures that Xmx is correct 
		if (xmx.length() > 0){
			if (xmx.length()<2){
				updateStatus("Xmx value must be formated like xm or xk or xg where x is a number");
				return;
			}
			try {
				Integer.parseInt(xmx.substring(0,xmx.length()-1));
				if ((!xmx.endsWith("m"))&&(!xmx.endsWith("k"))&&(!xmx.endsWith("g"))){
					updateStatus("Xmx value must be formated like xm or xk or xg where x is a number");
					return;
				}
			}catch(NumberFormatException e){
				updateStatus("Xmx value must be formated like xm or xk or xg where x is a number");
				return;
			}
			updateStatus(null);
		}
	}

	public String getXms(){
		if (xmsText!=null){
			if (!xmsText.isDisposed()){
				return xmsText.getText();
			}
		}
		return initXms;
	}
	public String getXmx(){
		if (xmxText!=null){
			if (!xmxText.isDisposed()){
				return xmxText.getText();
			}
		}
		return initXmx;
	}	

	//----------------------------------------------------------ISAC CONFIGURATION

	/**
	 * Creates contents of ISAC page
	 */
	public Composite createISACContents(Composite parent){
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		container.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		layout.numColumns = 2;
		layout.verticalSpacing = 9;
		layout.horizontalSpacing = 10;

		//ISAC Thread Label
		Label label = new Label(container, SWT.NULL);
		label.setText("&Thread pool size");

		//ISAC Thread Text
		isacThreadText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(SWT.BEGINNING);
		gd.widthHint=50;
		isacThreadText.setLayoutData(gd);
		isacThreadText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				isacDialogChanged();
			}
		});

		//Group period Label
		label = new Label(container, SWT.NULL);
		label.setText("&Group period");

		//Group period Text
		groupPeriodText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(SWT.BEGINNING);
		gd.widthHint=50;
		groupPeriodText.setLayoutData(gd);
		groupPeriodText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				isacDialogChanged();
			}
		});

		//Scheduler period Label
		label = new Label(container, SWT.NULL);
		label.setText("&Scheduler period");

		//Scheduler period Text
		schedulerPeriodText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(SWT.BEGINNING);
		gd.widthHint=50;
		schedulerPeriodText.setLayoutData(gd);
		schedulerPeriodText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				isacDialogChanged();
			}
		});

		//Job delay Label
		label = new Label(container, SWT.NULL);
		label.setText("&Job delay");

		//Job delay Text
		jobDelayText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(SWT.BEGINNING);
		gd.widthHint=50;
		jobDelayText.setLayoutData(gd);
		jobDelayText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				isacDialogChanged();
			}
		});

		return(container);
	}

	/**
	 * Initializes contents of ISAC page
	 */
	public void initializeISACParam() {
		isacThreadText.setText(initIsacThread);
		groupPeriodText.setText(initGroupPeriod);
		schedulerPeriodText.setText(initSchedulerPeriod);
		jobDelayText.setText(initJobDelay);
	}

	/**
	 * Ensures that both ISAC text fields are set. 
	 */
	private void isacDialogChanged() {
		String isacThread = getIsacThread();
		String groupPeriod = getGroupPeriod();
		String schedulerPeriod = getSchedulerPeriod();
		String jobDelay = getJobDelay();
		//Ensures that isacThread is correct 
		if (isacThread.length() > 0){
			if (!verifyNumber(isacThread, "Isac Thread")){
				return;
			}
		}
		if (groupPeriod.length() > 0){
			//Ensures that groupPeriod is correct 
			if (!verifyNumber(groupPeriod, "Group Period")){
				return;
			}
		}
		if (schedulerPeriod.length() > 0){
			//Ensures that schedulerPeriod is correct 
			if (!verifyNumber(schedulerPeriod, "Scheduler Period")){
				return;
			}
		}
		if (jobDelay.length() > 0){
			//Ensures that jobDelay is correct 
			if (!verifyNumber(jobDelay, "Job Delay")){
				return;
			}
		}
		updateStatus(null);
	}

	public String getIsacThread(){
		if (isacThreadText!=null){
			if (!isacThreadText.isDisposed()){
				return isacThreadText.getText();
			}
		}
		return initIsacThread;
	}

	public String getGroupPeriod(){
		if (groupPeriodText!=null){
			if (!groupPeriodText.isDisposed()){
				return groupPeriodText.getText();
			}
		}
		return initGroupPeriod;
	}

	public String getSchedulerPeriod(){
		if (schedulerPeriodText!=null){
			if (!schedulerPeriodText.isDisposed()){
				return schedulerPeriodText.getText();
			}
		}
		return initSchedulerPeriod;
	}

	public String getJobDelay(){
		if (jobDelayText!=null){
			if (!jobDelayText.isDisposed()){
				return jobDelayText.getText();
			}
		}
		return initJobDelay;
	}
	//-----------------------------------------------------------DEPLOYMENT CONFIGURATION

	/**
	 * Creates contents of deployment page
	 */
	public Composite createDeploymentContents(Composite parent) {
		Composite all = new Composite(parent, SWT.NULL);
		all.setLayout(new GridLayout());
		all.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		ImageRegistry imageRegistry = new ImageRegistry();
		imageRegistry.put("addproject", ClifConsolePlugin.imageDescriptorFromPlugin(
			ClifConsolePlugin.PLUGIN_ID,"icons/addproject.png"));
		imageRegistry.put("addexternal", ClifConsolePlugin.imageDescriptorFromPlugin(
			ClifConsolePlugin.PLUGIN_ID,"icons/addexternal.png"));
		imageRegistry.put("delete", ClifConsolePlugin.imageDescriptorFromPlugin(
			ClifConsolePlugin.PLUGIN_ID,"icons/delete.gif"));

		/* Composite with 2 columns :
		 * Registry host: Label + combo list
		 * Registry port: Label + textfield
		 * Code server port: Label + textfield */
		Composite field2 = new Composite(all, SWT.NULL);
		GridLayout layout2 = new GridLayout();
		field2.setLayout(layout2);
		layout2.numColumns = 2;
		layout2.verticalSpacing = 9;
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		field2.setLayoutData(gd);


		// registry host
		Label label = new Label(field2, SWT.NULL);
		label.setText("Registry host*:");
		registryHostCombo = new CCombo(field2, SWT.BORDER | SWT.SINGLE);
		registryHostCombo.setEditable(true);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		registryHostCombo.setLayoutData(gd);
		registryHostCombo.addModifyListener(
				new ModifyListener() {
					public void modifyText(
							ModifyEvent e) {
						deploymentDialogChanged();
					}
				});

		// registry port number
		label = new Label(field2, SWT.NULL);
		label.setText("Registry port*:");
		registryPortText = new Text(field2, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.BEGINNING);
		gd.widthHint=50;
		registryPortText.setLayoutData(gd);
		registryPortText.addModifyListener(
				new ModifyListener() {
					public void modifyText(ModifyEvent e) {
						deploymentDialogChanged();
					}
				});

		// code server host
		label = new Label(field2, SWT.NULL);
		label.setText("Code server host*:");
		codeServerHostCombo = new CCombo(field2, SWT.BORDER | SWT.SINGLE);
		codeServerHostCombo.setEditable(true);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		codeServerHostCombo.setLayoutData(gd);
		codeServerHostCombo.addModifyListener(
				new ModifyListener() {
					public void modifyText(
							ModifyEvent e) {
						deploymentDialogChanged();
					}
				});

		// code server port number
		label = new Label(field2, SWT.NULL);
		label.setText("Code server port*:");            
		codeServerPortText = new Text(field2, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.BEGINNING);
		gd.widthHint=50;
		codeServerPortText.setLayoutData(gd);
		codeServerPortText.addModifyListener(
				new ModifyListener() {
					public void modifyText(
							ModifyEvent e) {
						deploymentDialogChanged();
					}
				});

		// Separator
		Label lSeparator = new Label(all, SWT.CENTER);
		lSeparator.setText("");
		Label line = new Label(all, SWT.SEPARATOR | SWT.HORIZONTAL);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		line.setLayoutData(gd);
		lSeparator = new Label(all, SWT.CENTER);	


		label = new Label(all, SWT.NONE);
		label.setText("Code server classpath*:");

		/* Composite with 2 columns :
		 * Classpath : Composite with 2 buttons + list*/
		Composite field3 = new Composite(all, SWT.NULL);
		field3.setLayout(new GridLayout(2, false));

		/* Composite with 2 buttons */
		Composite buttons = new Composite(field3, SWT.NULL);
		FillLayout bLayout = new FillLayout(SWT.VERTICAL);
		buttons.setLayout(bLayout);

		Button addProject = new Button(buttons, SWT.PUSH);
		addProject.setImage(imageRegistry.get("addproject"));
		addProject.setToolTipText("Add project path");
		addProject.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event event) {

				ContainerSelectionDialog dialog = 
					new ContainerSelectionDialog(parentShell, null, false, "Select folder to add to classpath.");

				dialog.showClosedProjects(false);
				dialog.open();
				Object [] array = dialog.getResult();

				if (dialog.getResult() != null) {
					for (int i = 0; i < array.length; i++)
					{
						classes.add(array[i].toString().substring(1));
					}
				}
			}
		});

		Button addExternal = new Button(buttons, SWT.PUSH);
		addExternal.setImage(imageRegistry.get("addexternal"));
		addExternal.setToolTipText("Add external path");
		addExternal.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event event) {
				String nomFichier;
				DirectoryDialog dialog = new DirectoryDialog(parentShell, 
						SWT.OPEN);
				nomFichier = dialog.open();
				if ((nomFichier != null) && (nomFichier.length() != 0)){       
					classes.add(nomFichier);
				}
			}
		});

		Button remove = new Button(buttons, SWT.PUSH);
		remove.setImage(imageRegistry.get("delete"));
		remove.setToolTipText("Remove path");
		remove.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event event) {
				classes.remove(classes.getSelectionIndices());
			}
		});

		Button up = new Button(buttons, SWT.ARROW | SWT.UP);
		up.setToolTipText("Move path up");
		up.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event event) {

				if (classes.getSelectionCount() > 0) {
					String name = classes.getSelection()[0];
					int index = classes.getSelectionIndex();

					if (index > 0) {
						classes.remove(index);
						classes.add(name, index - 1);
						classes.setSelection(index - 1);
					}
				}
			}
		});

		Button down = new Button(buttons, SWT.ARROW | SWT.DOWN);
		down.setToolTipText("Move path down");
		down.addListener(SWT.Selection, new Listener(){
			public void handleEvent(Event event) {
				if (classes.getSelectionCount() > 0) {
					String name = classes.getSelection()[0];
					int index = classes.getSelectionIndex();
					if (index < classes.getItemCount() - 1) {
						classes.remove(index);
						classes.add(name, index + 1);
						classes.setSelection(index + 1);
					}
				}
			}
		});

		classes = new List(field3, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL | SWT.H_SCROLL |SWT.FILL);
		gd = new GridData(SWT.FILL,SWT.FILL,true,true);
		classes.setLayoutData(gd);    

		return(all);
	}

	/**
	 * Initializes contents of deployment page
	 */
	public void initializeDeployment() {
		/* Default port numbers */
		codeServerPortText.setText(initCodeServerPort);
		registryPortText.setText(initRegistryPort);
		/* Default host name */
		InetAddress[] addr = Network.getInetAddresses(null);
		registryHostCombo.removeAll();
		codeServerHostCombo.removeAll();
		classes.removeAll();
		registryHostCombo.add(initRegistryHost);
		codeServerHostCombo.add(initCodeServerHost);
		String[] paths = initClasses.replaceAll("\"","").split(";");
		for (int i = 0; i < paths.length; i++)
		{
			classes.add(paths[i]);
		}
		for (int i = 0; i < addr.length; i++) {
			try {
				registryHostCombo.add(addr[i].getHostAddress()+ " - " +
						NetworkInterface.getByInetAddress(addr[i]).getDisplayName());
				codeServerHostCombo.add(addr[i].getHostAddress()+ " - " +
						NetworkInterface.getByInetAddress(addr[i]).getDisplayName());
			} catch (SocketException e) {
				e.printStackTrace();
			}
		}
		if(registryHostCombo.getItemCount() > 0) {
			registryHostCombo.select(0);
		}
		else {
			deploymentDialogChanged();
		}
		if(codeServerHostCombo.getItemCount() > 0) {
			codeServerHostCombo.select(0);
		}
		else {
			deploymentDialogChanged();
		}
	}

	/**
	 * Ensures that both deployment text fields are correctly filled. 
	 */
	public void deploymentDialogChanged()
	{
		//Ensures that registry host is correct 
		if (getRegistryHost().length() == 0){
			updateStatus("Registry host must be specified");
			return;
		}
		//Ensures that registry port number is correct 
		if (getRegistryPort().length() == 0) {
			updateStatus("Registry port number must be specified");
			return;
		}
		if (!verifyPort(getRegistryPort(), "Registry Port")){
			return;
		}
		//Ensures that code server host is correct 
		if (getCodeServerHost().length() == 0){
			updateStatus("Code Server host must be specified");
			return;
		}
		//Ensures that code server number is correct 
		if (getCodeServerPort().length() == 0) {
			updateStatus("Code Server port number must be specified");
			return;
		}
		if (!verifyPort(getCodeServerPort(), "Code Server Port")){
			return;
		}
		//Ensures that class path is correct 
		if (classes.getItemCount()<=0) {
			updateStatus("At least one class path must be specified");
			return;
		}
		updateStatus(null);
	}

	public String getRegistryHost() {
		if (registryHostCombo!=null){
			if (!registryHostCombo.isDisposed()){
				return registryHostCombo.getText().split(" ")[0];
			}
		}
		return initRegistryHost;
	}

	public String getCodeServerHost() {
		if (codeServerHostCombo!=null){
			if (!codeServerHostCombo.isDisposed()){
				return codeServerHostCombo.getText().split(" ")[0];
			}
		}
		return initCodeServerHost;
	}

	/**
	 * Return all classes repository set in the 
	 * list of classes separated by a ";"
	 * @return list of classes
	 */
	public String getClasses() {
		if (classes!=null){
			if(!classes.isDisposed()){
				String [] paths = classes.getItems();
				String s = "";
				for (int i = 0; i < paths.length-1; i++) {
					s+=paths[i].replaceAll("\\\\","/")+";";
				}
				if (paths.length == 0){
					s="\""+s+"\"";
					return s;
				}else{
					s+=paths[paths.length-1].replaceAll("\\\\","/");
					s="\""+s+"\"";
					return s;
				}
			}
		}
		return initClasses;
	}

	public String getCodeServerPort() {
		if (codeServerPortText!=null){
			if (!codeServerPortText.isDisposed()){
				return codeServerPortText.getText();
			}
		}
		return initCodeServerPort;
	}

	public String getRegistryPort() {
		if (registryPortText!=null){
			if (!registryPortText.isDisposed()){
				return registryPortText.getText();
			}
		}
		return initRegistryPort;
	}

	//----------------------------------------------------------COMMON FEATURES	

	/**
	 * Initializes all properties by reading clif.props file. The file used 
	 * depends of the page type (wizard or properties page).
	 */
	public void initialize(){
		FileInputStream clifProps;
		Properties properties;
		File clifPropsFile = null;
		try {
			// try to get clif.props from current project folder
			if (project != null)
			{
				clifPropsFile = new File(project.getLocation().append(ClifConsolePlugin.CLIF_PROPS).toOSString());
				if (! clifPropsFile.exists())
				{
					clifPropsFile = null;
				}
			}
			// otherwise, get clif.props default file from the CLIF console plug-in
			if (clifPropsFile == null)
			{
				clifPropsFile = new File(ExecutionContext.getBaseDir() + File.separator + ExecutionContext.PROPS_PATH);
			}
			clifProps = new FileInputStream(clifPropsFile);
			properties = new Properties();
			properties.load(clifProps);
			String allParam = (String)properties.get("clif.parameters");
			String[] parameters = allParam.split(" ");
			initCustomProps.clear();
			for (String parameter : parameters)
			{
				if(parameter.startsWith("-Dclif.filestorage.dir")){
					String[] property = parameter.split("=");
					initReport = property[1];
				}else if(parameter.startsWith("-Dclif.eclipse.statdir")){
					String[] property = parameter.split("=");
					initStats = property[1];
				}else if(parameter.startsWith("-Dfractal.registry.host")){
					String[] property = parameter.split("=");
					initRegistryHost = property[1];
				}else if(parameter.startsWith("-Dfractal.registry.port")){
					String[] property = parameter.split("=");
					initRegistryPort = property[1];
				}else if(parameter.startsWith("-Dclif.codeserver.host")){
					String[] property = parameter.split("=");
					initCodeServerHost = property[1];
				}else if(parameter.startsWith("-Dclif.codeserver.port")){
					String[] property = parameter.split("=");
					initCodeServerPort = property[1];
				}else if(parameter.startsWith("-Dclif.codeserver.path")){
					String[] property = parameter.split("=");
					property[1] = property[1].replaceAll("\"","");
					initClasses = property[1];
				}else if(parameter.startsWith("-Dclif.filestorage.delay_s")){
					String[] property = parameter.split("=");
					initFileStorageDelay = property[1];
				}else if(parameter.startsWith("-Dclif.filestorage.maxpending")){
					String[] property = parameter.split("=");
					initFileStorageMaxPending = property[1];
				}else if(parameter.startsWith("-Djonathan.connectionfactory.host")){
					String[] property = parameter.split("=");
					initJonathanHost = property[1];
				}else if(parameter.startsWith("-Dclif.filestorage.host")){
					String[] property = parameter.split("=");
					initFileStorageHost = property[1];
				}else if(parameter.startsWith("-Xms")){
					initXms=parameter.substring(4);
				}else if(parameter.startsWith("-Xmx")){
					initXmx=parameter.substring(4);
				}else if(parameter.startsWith("-Dclif.isac.threads")){
					String[] property = parameter.split("=");
					initIsacThread = property[1];
				}else if(parameter.startsWith("-Dclif.isac.groupperiod")){
					String[] property = parameter.split("=");
					initGroupPeriod = property[1];
				}else if(parameter.startsWith("-Dclif.isac.schedulerperiod")){
					String[] property = parameter.split("=");
					initSchedulerPeriod = property[1];
				}else if(parameter.startsWith("-Dclif.isac.jobdelay")){
					String[] property = parameter.split("=");
					initJobDelay = property[1];
				}
				else if (
					parameter.startsWith("-D")
					&& ! parameter.startsWith("-Djulia.")
					&& ! parameter.startsWith("-Dfractal.provider"))
				{
					initCustomProps.add(parameter.substring("-D".length()));
				}
			}
			initialized=true;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Applies all modifications on CLIF properties files. 
	 */
	public void apply(){
		if (!applied){
			String reportPath = project.getWorkspace().getRoot().getLocation().append(getReportDir()).toOSString();
			File reportDir = new File(reportPath);
			if (!reportDir.exists()){
				reportDir.mkdirs();
			}
			String statsPath = project.getWorkspace().getRoot().getLocation().append(getStatsDir()).toOSString();
			File statsDir = new File(statsPath);
			if (!statsDir.exists()){
				statsDir.mkdirs();
			}
			File clifPropsLocal = new File(project.getLocation().append(ClifConsolePlugin.CLIF_PROPS).toOSString());
			writeClifProperties(clifPropsLocal);
			try {
		        project.refreshLocal(IProject.DEPTH_INFINITE, null);
	        } catch (CoreException e) {
		        e.printStackTrace();
	        }
			applied=true;
		}
	}

	/**
	 * Write all properties in clif properties file by checking 
	 * all filled fields.
	 */
	private void writeClifProperties(File clifFile){
		BufferedWriter outputFile;
		String statDir=getStatsDir();
		statDir = statDir.replaceAll("\\\\","/");
		String reportDir=getReportDir();
		reportDir = reportDir.replaceAll("\\\\","/");
		try {
			outputFile = new BufferedWriter(new FileWriter(clifFile));
			outputFile.write("clif.parameters \\\n");
			outputFile.write("-Dfractal.provider=org.objectweb.fractal.julia.Julia \\\n");
			outputFile.write("-Dfractal.registry.host=");
			outputFile.write((getRegistryHost()+" \\\n"));
			outputFile.write("-Dfractal.registry.port=");
			outputFile.write((getRegistryPort()+" \\\n"));
			outputFile.write("-Djulia.config=etc/julia.cfg \\\n");
			outputFile.write("-Djulia.loader=org.objectweb.fractal.julia.loader.DynamicLoader \\\n");
			outputFile.write("-Dclif.codeserver.port=");
			outputFile.write((getCodeServerPort()+" \\\n"));
			outputFile.write("-Dclif.codeserver.host=");
			outputFile.write((getCodeServerHost()+" \\\n"));
			outputFile.write("-Dclif.codeserver.path=");
			outputFile.write((getClasses()+" \\\n"));
			outputFile.write("-Dclif.eclipse.statdir=");
			outputFile.write((statDir+" \\\n"));
			outputFile.write("-Dclif.filestorage.dir=");
			outputFile.write((reportDir+" \\\n"));
			if (getFileStorageDelay().length()>0){
				outputFile.write("-Dclif.filestorage.delay_s=");
				outputFile.write((getFileStorageDelay()+" \\\n"));
			}
			if (getFileStorageMaxPending().length()>0){
				outputFile.write("-Dclif.filestorage.maxpending=");
				outputFile.write((getFileStorageMaxPending()+" \\\n"));
			}
			if (getIsacThread().length()>0){
				outputFile.write("-Dclif.isac.threads=");
				outputFile.write((getIsacThread()+" \\\n"));
			}
			if (getGroupPeriod().length()>0){
				outputFile.write("-Dclif.isac.groupperiod=");
				outputFile.write((getGroupPeriod()+" \\\n"));
			}
			if (getSchedulerPeriod().length()>0){
				outputFile.write("-Dclif.isac.schedulerperiod=");
				outputFile.write((getSchedulerPeriod()+" \\\n"));
			}
			if (getJobDelay().length()>0){
				outputFile.write("-Dclif.isac.jobdelay=");
				outputFile.write((getJobDelay()+" \\\n"));
			}
			if (getJonathanHost().length()>0){
				outputFile.write("-Djonathan.connectionfactory.host=");
				outputFile.write((getJonathanHost()+" \\\n"));
			}
			if (getFileStorageHost().length()>0){
				outputFile.write("-Dclif.filestorage.host=");
				outputFile.write((getFileStorageHost()+" \\\n"));
			}
			for (String prop : getCustomProperties())
			{
				outputFile.write("-D" + prop + " \\\n");
			}
			if (getXms().length()>0){
				outputFile.write("-Xms");
				outputFile.write((getXms()+" "));
			}
			if (getXmx().length()>0){
				outputFile.write("-Xmx");
				outputFile.write((getXmx()+" \\\n"));
			}
			outputFile.write("-server");
			outputFile.close();			
		} catch (IOException e) {
			catchException(e);
		}
	}

	/**
	 * Verify if the argument is a valid port number
	 * @param number the argument to check
	 * @param label type of this argument
	 * @return boolean true if number is a valid port number
	 */
	private boolean verifyPort(String number, String label){
		if (number.length() == 0) {
			updateStatus(label+" number must be specified");
			return false;
		}
		try {
			int port = Integer.parseInt(number);
			if (port<0 || port>65536){
				updateStatus(label+" number must be valid");
				return false;
			}
		}
		catch(NumberFormatException e){
			updateStatus(label+" number must be an integer");
			return false;
		}
		return true;
	}
	
	/**
	 * Verify if the argument is a valid number
	 * @param number the argument to check
	 * @param label type of this argument
	 * @return true if number is valid
	 */
	private boolean verifyNumber(String number, String label){
		if (number.length() == 0) {
			updateStatus(label+" must be specified");
			return false;
		}
		try {
			Integer.parseInt(number);
		}
		catch(NumberFormatException e){
			updateStatus(label+" must be an integer");
			return false;
		}
		return true;

	}

	/** 
	 * Updates wizard status message
	 */ 
	private void updateStatus(String message) {
		boolean valid=true;
		if (message!=null){
			valid=false;
		}
		if (wizardPage!=null){
			wizardPage.setErrorMessage(message);
			wizardPage.setPageComplete(valid);
		}else if(prefPage!=null){
			prefPage.setErrorMessage(message);
			prefPage.setValid(valid);
		}
	}

	/**
	 * set Project and fill initClasses
	 * @param project
	 */
	public void setProject(IProject project){
		this.project=project;
		initClasses=project.getFullPath().toOSString();
		initClasses=initClasses.substring(1);
	}

	public boolean getInitialized(){
		return initialized;
	}

	public void setInitialized(boolean init){
		initialized=init;
	}

	/**
	 * Open a message error dialog to user, block plug-in activity
	 * in setting valid page to false and close the shell.
	 * @param e Exception to catch
	 */
	public void catchException(Exception e)
	{
		e.printStackTrace(System.err);
		Status status = new Status(Status.ERROR, 
				"CLIF Project",
				Status.OK,
				"Error during file manipulation.",
				e);
		ErrorDialog diag = new ErrorDialog(parentShell, 
				"Plugin error", 
				"Problem detects", 
				status, 
				IStatus.ERROR);	
		diag.open();
		parentShell.close();
	}
}
